//
//  AppDelegate.swift
//  globalfoods
//
//  Created by Apple on 25/12/20.
//

import UIKit
import IQKeyboardManagerSwift
import UserNotifications
import Firebase

@main
class AppDelegate: UIResponder, UIApplicationDelegate {

    var orientationLock = UIInterfaceOrientationMask.portrait
    var window: UIWindow?
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        // Override point for customization after application launch.
        if #available(iOS 13.0, *){
            //do nothing we had a code in SceneceDelegate for this
        } else {
            self.window = UIWindow(frame: UIScreen.main.bounds)
            self.window?.makeKeyAndVisible()
            AppCommonMethods.setIntialScreen()
        }
        if AppConstants.employeeData != nil{
            AppCommonMethods.getCartsFromLayAway()
        }
        FirebaseApp.configure()
        registerForPushNotifications()
        IQKeyboardManager.shared.enable = true
        IQKeyboardManager.shared.keyboardDistanceFromTextField = 0
        return true
    }

    func registerForPushNotifications() {
        UNUserNotificationCenter.current().delegate = self
        UNUserNotificationCenter.current().requestAuthorization(options: [.badge, .sound, .alert], completionHandler: {(granted, error) in
            if (granted)
            {
                DispatchQueue.main.async(execute: {
                    UIApplication.shared.registerForRemoteNotifications()
                })
            }
            else{
                
            }
        })
        
    }
    
    func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data)
    {
        var token: String = ""
        
        for i in 0..<deviceToken.count {
            token += String(format: "%02.2hhx", deviceToken[i] as CVarArg)
        }
        if AppConstants.isInDevlopmentMode{
            print("device token: \(token)")
        }
        AppCommonMethods.saveDataIntoUserDefault(value: token, withKey: AppConstants.userDefaultKey.kDeviceToken)
    }
    
    func application(_ application: UIApplication, supportedInterfaceOrientationsFor window: UIWindow?) -> UIInterfaceOrientationMask {
        if !AppConstants.DeviceType.IS_IPAD{
            return self.orientationLock
        }
        return UIInterfaceOrientationMask.allButUpsideDown
    }
    
    // MARK: UISceneSession Lifecycle
    @available(iOS 13.0, *)
    func application(_ application: UIApplication, configurationForConnecting connectingSceneSession: UISceneSession, options: UIScene.ConnectionOptions) -> UISceneConfiguration {
        // Called when a new scene session is being created.
        // Use this method to select a configuration to create the new scene with.
        return UISceneConfiguration(name: "Default Configuration", sessionRole: connectingSceneSession.role)
    }
    
    @available(iOS 13.0, *)
    func application(_ application: UIApplication, didDiscardSceneSessions sceneSessions: Set<UISceneSession>) {
        // Called when the user discards a scene session.
        // If any sessions were discarded while the application was not running, this will be called shortly after application:didFinishLaunchingWithOptions.
        // Use this method to release any resources that were specific to the discarded scenes, as they will not return.
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        if #available(iOS 13.0, *){
            //do nothing we had a code in SceneceDelegate for this
        } else {
            AppCommonMethods.getCustomerDataAPI { (status) in
                
            }
            AppCommonMethods.getEmployeeDataAPI { (status) in
                            
            }
            AppCommonMethods.getGlobalFoodDataAPI()
        }
    }
    
    
}

extension AppDelegate:UNUserNotificationCenterDelegate{
    func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void){
        completionHandler([.alert, .badge, .sound])
    }
    
    func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        if let userInfo = response.notification.request.content.userInfo as? [String : AnyObject] {
            if AppConstants.isInDevlopmentMode{
                print("payload:", userInfo)
            }
            if AppConstants.userData == nil && AppConstants.employeeData == nil && AppConstants.subCustomerData == nil{
                if let type = userInfo["type"] as? String {
                    if type == "order"{
                        if var VC = UIApplication.topViewController(){
                            if let sheetVC = VC as? SheetViewController{
                                sheetVC.dismiss(animated: true) {

                                }
                                if let controller = UIApplication.topViewController(controller: sheetVC.presentingViewController){
                                    VC = controller
                                }
                                if (VC is OrderHistoryVC){
                                    (VC as! OrderHistoryVC).callGetOrderHistoryAPI()
                                }else{
                                    VC.sideMenuController?.setContentViewController(with: "1", animated: true)
                                }
                            }
                        }
                    }
                }
                return
            }
        }
    }
}
