//
//  Extensions.swift
//  globalfoods
//
//  Created by Kamal Parmar on 10/07/20.
//  Copyright © 2020 Kamal Parmar. All rights reserved.
//

import Foundation
import UIKit

extension UITextField{
    func setTextFieldAlert(message:String){
        self.attributedPlaceholder = NSAttributedString(string: message, attributes: [NSAttributedString.Key.foregroundColor : UIColor.red])
    }
}

extension KeyedDecodingContainer{
    func getStringValue(key:CodingKey)->String?{
        if let value = try? self.decodeIfPresent(String.self, forKey: key as! K){
            return value
        }else if let value = try? self.decodeIfPresent(Int.self, forKey: key as! K){
            return "\(value)"
        }else if let value = try? self.decodeIfPresent(Double.self, forKey: key as! K){
            return "\(value)"
        }
        return nil
    }
    
    func getIntValue(key:CodingKey)->Int?{
        if let value = try? self.decodeIfPresent(Int.self, forKey: key as! K){
            return value
        }else if let value = try? self.decodeIfPresent(String.self, forKey: key as! K){
            return Int(value) ?? 0
        }else if let value = try? self.decodeIfPresent(Double.self, forKey: key as! K){
            return Int(value)
        }
        return nil
    }
    
    func getDoubleValue(key:CodingKey)->Double?{
        if let value = try? self.decodeIfPresent(Double.self, forKey: key as! K){
            return value
        }else if let value = try? self.decodeIfPresent(String.self, forKey: key as! K){
            return Double(value) ?? 0
        }else if let value = try? self.decodeIfPresent(Int.self, forKey: key as! K){
            return Double(value)
        }
        return nil
    }
}

extension UIColor {
    class func hexStr ( hexStr : NSString, alpha : CGFloat) -> UIColor {
        let hexStr = hexStr.replacingOccurrences(of: "#", with: "")
        let scanner = Scanner(string: hexStr as String)
        var color: UInt32 = 0
        if scanner.scanHexInt32(&color) {
            let r = CGFloat((color & 0xFF0000) >> 16) / 255.0
            let g = CGFloat((color & 0x00FF00) >> 8) / 255.0
            let b = CGFloat(color & 0x0000FF) / 255.0
            return UIColor(red:r,green:g,blue:b,alpha:alpha)
        } else {
            print("invalid hex string", terminator: "")
            return UIColor.white;
        }
    }
}
extension UIViewController{
    
    var className: String {
        return NSStringFromClass(self.classForCoder).components(separatedBy: ".").last ?? ""
    }
}

extension UIScrollView {
    var currentPage:Int{
        return Int((self.contentOffset.x+(0.5*self.frame.size.width))/self.frame.width)+1
    }
    
    func scrollTo(horizontalPage: Int? = 0, verticalPage: Int? = 0, animated: Bool? = true) {
        var frame: CGRect = self.frame
        frame.origin.x = frame.size.width * CGFloat(horizontalPage ?? 0)
        frame.origin.y = frame.size.width * CGFloat(verticalPage ?? 0)
        self.scrollRectToVisible(frame, animated: animated ?? true)
    }
}

extension UIApplication {
    class func topViewController(controller: UIViewController? = UIApplication.shared.keyWindow?.rootViewController) -> UIViewController? {
        if let presented = controller?.presentedViewController {
            return topViewController(controller: presented)
        }
        if let navigationController = controller as? UINavigationController {
            return topViewController(controller: navigationController.visibleViewController)
        }
        if let tabController = controller as? UITabBarController {
            if let selected = tabController.selectedViewController {
                return topViewController(controller: selected)
            }
        }
        
        return controller
    }
}

extension UIImageView {
    func setImageColor(color: UIColor) {
        let templateImage = self.image?.withRenderingMode(.alwaysTemplate)
        self.image = templateImage
        self.tintColor = color
    }
}

extension UIView{
    func applyShadowCardView(Corner radi: Int)
    {
        self.layer.cornerRadius = CGFloat(radi)
        self.layer.shadowColor = UIColor.gray.cgColor
        self.layer.shadowOpacity = 0.7
        self.layer.shadowRadius = 3.0
        self.layer.shadowOffset = CGSize(width: 1, height: 1)
    }
}

extension UILabel{
    func setLabelAttributedText(title:String,value:String,fontSize:CGFloat,shoulHideLabel:Bool = false){
        if shoulHideLabel && value == ""{
            self.isHidden = true
        }else{
            self.isHidden = false
            let attrDesc = NSMutableAttributedString.init(string: "\(title): \(value)", attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: fontSize)])
            attrDesc.addFontAttribute(title: title, fontSize: fontSize)
            self.attributedText = attrDesc
        }
    }
}

extension NSMutableAttributedString{
    func addFontAttribute(title:String,fontSize:CGFloat){
        let range = NSString.init(string: self.string).range(of: "\(title):")
        self.addAttribute(NSAttributedString.Key.foregroundColor, value: AppConstants.Colors.kAppThemeTextColor, range: range)
        self.addAttribute(NSAttributedString.Key.font, value: AppConstants.GlobalFontConstants.kBoldFont(size: fontSize), range: range)
    }
}

extension String{
    
    func getStringWidth(font:UIFont)->CGFloat{
        return NSString.init(string: self).size(withAttributes: [NSAttributedString.Key.font:font]).width
    }
    
    func htmlToAttributedString(font:UIFont? = nil,color:UIColor? = nil)-> NSAttributedString? {
        guard let data = data(using: .utf8) else { return nil }
        do {
            let attributedString = try NSMutableAttributedString(data: data, options: [.documentType: NSAttributedString.DocumentType.html, .characterEncoding:String.Encoding.utf8.rawValue], documentAttributes: nil)
            if font != nil{
                let nsrange = NSRange(location: 0, length: attributedString.length)
                attributedString.addAttribute(.font, value: font!, range: nsrange)
            }
            if color != nil{
                let nsrange = NSRange(location: 0, length: attributedString.length)
                attributedString.addAttribute(.foregroundColor, value: color!, range: nsrange)
            }
            return attributedString
        } catch {
            return nil
        }
    }
    
    func changeDateFormat(fromFormat:String,toFormat:String)->String?{
        let dateFormat = DateFormatter.init()
        dateFormat.dateFormat = fromFormat
        dateFormat.timeZone = TimeZone.init(secondsFromGMT: 0)
        dateFormat.locale = Locale.init(identifier: "en_GB")
        if let date = dateFormat.date(from: self){
            dateFormat.dateFormat = toFormat
            return dateFormat.string(from: date)
        }
        return nil
    }
    
    func getDateFromString(format:String,locale:String? = nil)->Date?{
        let dateFormat = DateFormatter.init()
        dateFormat.dateFormat = format
        dateFormat.timeZone = TimeZone.init(secondsFromGMT: 0)
        dateFormat.locale = Locale.init(identifier: "en_GB")
        if locale != nil{
            dateFormat.locale = Locale.init(identifier: locale!)
        }
        return dateFormat.date(from: self)
    }
    
    func getHeightFromString(font:UIFont, width:CGFloat)->CGFloat{
        let textAttributes = [NSAttributedString.Key.font: font]
        return ceil(self.boundingRect(with: CGSize.init(width: width, height: 2000), options: .usesLineFragmentOrigin, attributes: textAttributes, context: nil).height)
    }
    
    func split(every: Int, backwards: Bool = false) -> [String] {
        var result = [String]()
        
        for i in stride(from: 0, to: self.count, by: every) {
            switch backwards {
            case true:
                let endIndex = self.index(self.endIndex, offsetBy: -i)
                let startIndex = self.index(endIndex, offsetBy: -every, limitedBy: self.startIndex) ?? self.startIndex
                result.insert(String(self[startIndex..<endIndex]), at: 0)
            case false:
                let startIndex = self.index(self.startIndex, offsetBy: i)
                let endIndex = self.index(startIndex, offsetBy: every, limitedBy: self.endIndex) ?? self.endIndex
                result.append(String(self[startIndex..<endIndex]))
            }
        }
        
        return result
    }
}

extension NSAttributedString{
    func height(containerWidth: CGFloat) -> CGFloat {

        let rect = self.boundingRect(with: CGSize.init(width: containerWidth, height: CGFloat.greatestFiniteMagnitude),
                                     options: [.usesLineFragmentOrigin, .usesFontLeading],
                                     context: nil)
        return ceil(rect.size.height)
    }
}

extension Date{
    func getDateSuffix()->String{
        var suffix = "th"
        let day = Calendar.current.component(.day, from: self)
        if day < 20 && day > 10{
            suffix = "th"
        }else{
            switch day % 10 {
            case 1:
                suffix = "st"
            case 2:
                suffix = "nd"
            case 3:
                suffix = "rd"
            default:
                suffix = "th"
            }
        }
        return suffix
    }
    
    func gatDateFormat()->String{
        var suffix = "th"
        let day = Calendar.current.component(.day, from: self)
        if day < 20 && day > 10{
            suffix = "th"
        }else{
            switch day % 10 {
            case 1:
                suffix = "st"
            case 2:
                suffix = "nd"
            case 3:
                suffix = "rd"
            default:
                suffix = "th"
            }
        }
        return "dd'\(suffix)' MMM yyyy"
    }
    
    func getDateInString(format:String,locale:String? = nil)->String{
        let dateFormat = DateFormatter.init()
        dateFormat.dateFormat = format
        dateFormat.timeZone = TimeZone.init(secondsFromGMT: 0)
        dateFormat.locale = Locale.init(identifier: "en_GB")
        if locale != nil{
            dateFormat.locale = Locale.init(identifier: locale!)
        }
        return dateFormat.string(from: self)
    }
}


extension NSDictionary{
    
    func percentEncoded() -> Data? {
        return map { key, value in
            let escapedKey = "\(key)".addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) ?? ""
            let escapedValue = "\(value)".addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) ?? ""
            return escapedKey + "=" + escapedValue
        }
        .joined(separator: "&")
        .data(using: .utf8)
    }
    
    func removeNullFromDict () -> NSMutableDictionary
    {
        let dict = NSMutableDictionary()
        for (key, value) in self {
            
            let val : NSObject = value as! NSObject;
            if val is NSNull
            {
                dict.setValue("", forKey: (key as? String)!)
            }
            else
            {
                dict.setValue(value, forKey: key as! String)
            }
        }
        
        return dict
    }
    
    func getIntValue(_ param:String) ->Int?{
        if let id = self[param] as? Int{
            return id
        }
        if let id = self[param] as? String{
            return Int(id)
        }
        return nil
    }
    
    func getIntFromPath(_ param:String)-> Int?{
        if let id = self.value(forKeyPath: param) as? Int{
            return id
        }
        if let id = self.value(forKeyPath: param) as? String{
            return Int(id)
        }
        return nil
    }
    
    func getStringFromPath(_ param:String)-> String?{
        if let id = self.value(forKeyPath: param) as? String{
            return id
        }
        if let id = self.value(forKeyPath: param) as? Int{
            return "\(id)"
        }
        if let id = self.value(forKeyPath: param) as? Double{
            return "\(id)"
        }
        return nil
    }
    
    func getDoubleValue(_ param:String) ->Double?{
        if let id = self[param] as? Double{
            return id
        }
        if let id = self[param] as? String{
            return Double(id)
        }
        return nil
    }
    
    func getFloatValue(_ param:String) ->Float?{
        if let id = self[param] as? Float{
            return id
        }
        if let id = self[param] as? Double{
            return Float(id)
        }
        if let id = self[param] as? String{
            return Float(id)
        }
        return nil
    }
    
    func getStringValue(_ param:String) ->String?{
        if let id = self[param] as? String{
            return id
        }
        if let id = self[param] as? Double{
            return "\(id)"
        }
        if let id = self[param] as? Int{
            return "\(id)"
        }
        return nil
    }
    
    var convertToJson:Data?{
        
        do {
            let jsonData = try JSONSerialization.data(withJSONObject: self, options: .prettyPrinted)
            // here "jsonData" is the dictionary encoded in JSON data
            //print(jsonData)
            return jsonData
            
        } catch {
            print(error.localizedDescription)
            return nil
        }
    }
}

extension UITapGestureRecognizer {

    func didTapAttributedTextInLabel(label: UILabel, inRange targetRange: NSRange) -> Bool {
        // Create instances of NSLayoutManager, NSTextContainer and NSTextStorage
        let layoutManager = NSLayoutManager()
        let textContainer = NSTextContainer(size: CGSize.zero)
        let textStorage = NSTextStorage(attributedString: label.attributedText!)

        // Configure layoutManager and textStorage
        layoutManager.addTextContainer(textContainer)
        textStorage.addLayoutManager(layoutManager)

        // Configure textContainer
        textContainer.lineFragmentPadding = 0.0
        textContainer.lineBreakMode = label.lineBreakMode
        textContainer.maximumNumberOfLines = label.numberOfLines
        let labelSize = label.bounds.size
        textContainer.size = labelSize

        // Find the tapped character location and compare it to the specified range
        let locationOfTouchInLabel = self.location(in: label)
        let textBoundingBox = layoutManager.usedRect(for: textContainer)
        let textContainerOffset = CGPoint(x: (labelSize.width - textBoundingBox.size.width) * 0.5 - textBoundingBox.origin.x, y: (labelSize.height - textBoundingBox.size.height) * 0.5 - textBoundingBox.origin.y)

        let locationOfTouchInTextContainer = CGPoint(x: locationOfTouchInLabel.x - textContainerOffset.x, y: locationOfTouchInLabel.y - textContainerOffset.y)
        let indexOfCharacter = layoutManager.characterIndex(for: locationOfTouchInTextContainer, in: textContainer, fractionOfDistanceBetweenInsertionPoints: nil)
        return NSLocationInRange(indexOfCharacter, targetRange)
    }

}
