//
//  PrinterManager.swift
//  globalfoods
//
//  Created by Apple on 08/09/21.
//

import Foundation

class PrintManager:NSObject {
    
    static let shared = PrintManager()
    private var printerObject:Any?
    private let portNumber = 9100
    private let printer = ThermalPrinterFactory.create()
    var isSalesOrder = false
    var fromDate = ""
    var toDate = ""
    
    private override init() {
        super.init()
        addObserver()
    }
    
    func addObserver(){
        NotificationCenter.default.addObserver(self, selector: #selector(connectedToPrinter), name: NSNotification.Name.PrinterConnected, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(disconnectedToPrinter), name: NSNotification.Name.PrinterDisconnected, object: nil)
    }
    
    @objc private func connectedToPrinter(){
        if let object = printerObject as? OrderHistoryModel{
            printOrder(order: object)
        }else if let _ = printerObject as? EmployeeModel{
            openCashDrawer()
        }else if let object = printerObject as? FullReportModel{
            printReport(report: object)
        }
    }
    
    @objc private func disconnectedToPrinter(){
        DispatchQueue.main.async {
            AppCommonMethods.stopProgressBar()
        }
        printerObject = nil
    }
    
    func connectToPrinterWithPrint(model:Any){
        if AppConstants.printerIP ?? "" == ""{
            return
        }
        DispatchQueue.main.async {
            AppCommonMethods.startProgressBar()
        }
        self.printerObject = model
        let interface = WIFIFactory.create()
        interface?.address = AppConstants.printerIP
        interface?.port = portNumber
        interface?.printerCmdtype = PrinterCmdESC
        printer?.printerPi = interface
        printer?.open()
        
//        self.connectedToPrinter()
    }
    
    func printOrder(order:OrderHistoryModel){
        let imageCMD = ESCFactory.create()
        imageCMD?.clear()
        imageCMD?.encodingType = Encoding_UTF8
        imageCMD?.append(imageCMD?.getHeaderCmd())
        let bitmapSetting = printer?.bitmapSetts
        bitmapSetting?.alignmode = Align_Center
        bitmapSetting?.limitWidth = 30*8
        
        let img = UIImage.init(named: "AppBlackLogo")!
        imageCMD?.append(imageCMD?.getBitMapCmd(bitmapSetting, image: img))
        imageCMD?.append(imageCMD?.getPrintEnd(1))
        printer?.write(imageCMD?.getCmd())
        
        let textCMD = ESCFactory.create()
        textCMD?.clear()
        textCMD?.encodingType = Encoding_GBK
        textCMD?.append(textCMD?.getHeaderCmd())
        
        let textst = self.printer?.textSets
        textst?.isUnderline = Set_DisEnable
        
        textst?.isBold = Set_Enabled
        textst?.isTimes_Wide = Set_Enabled
        textst?.isTimes_Heigh = Set_Enabled
        textst?.alignmode = Align_Center
        
        if let name = AppConstants.dictGlobalFoodInfo.getStringValue("client_name"){
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(name)\n"))
        }
        
        
        textst?.isTimes_Wide = Set_DisEnable
        textst?.isTimes_Heigh = Set_DisEnable
        var strAddress = ""
        if let address1 = AppConstants.dictGlobalFoodInfo.getStringValue("client_address1"){
            strAddress = address1
        }
        if let street = AppConstants.dictGlobalFoodInfo.getStringValue("client_street"){
            if strAddress == ""{
                strAddress = street
            }else{
                strAddress = "\(strAddress),\(street)"
            }
        }
        if strAddress != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(strAddress)\n"))
        }
        
        var strCity = ""
        if let city = AppConstants.dictGlobalFoodInfo.getStringValue("client_city"){
            strCity = city
        }
        if let pincode = AppConstants.dictGlobalFoodInfo.getStringValue("client_pincode"){
            if strCity == ""{
                strCity = pincode
            }else{
                strCity = "\(strCity)-\(pincode)"
            }
        }
        if strCity != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(strCity)\n"))
        }
        
        var strCounty = ""
        if let state = AppConstants.dictGlobalFoodInfo.getStringValue("client_state"){
            strCounty = state
        }
        if let country = AppConstants.dictGlobalFoodInfo.getStringValue("client_country"){
            if strCounty == ""{
                strCounty = country
            }else{
                strCounty = "\(strCounty),\(country)"
            }
        }
        if strCounty != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(strCounty)\n"))
        }
        
        if let email = AppConstants.dictGlobalFoodInfo.getStringValue("client_email"){
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "E:\(email.lowercased())\n"))
        }
        
        var strPhone = ""
        if let phone = AppConstants.dictGlobalFoodInfo.getStringValue("client_contact_number_1"){
            strPhone = phone
        }
        if let phone = AppConstants.dictGlobalFoodInfo.getStringValue("client_contact_number_2"){
            if strPhone == ""{
                strPhone = phone
            }else{
                strPhone = "\(strPhone)\n\(phone)"
            }
        }
        if strPhone != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "P:\(strPhone)\n"))
        }
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "VAT NO: 229179190\n"))
        
        textst?.isTimes_Wide = Set_Enabled
        textst?.isTimes_Heigh = Set_Enabled
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n\n"))
        textst?.isTimes_Wide = Set_DisEnable
        textst?.isTimes_Heigh = Set_DisEnable
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Order Info\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n\n"))
        
        textst?.alignmode = Align_Left
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Order No: \(order.orderNo ?? "")\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Date: \(order.date ?? "")\n\n"))
        
        textst?.isBold = Set_DisEnable
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "For:\n\(order.customerName ?? "")\n"))
        if !self.isSalesOrder{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(order.customerEmail ?? "")\n"))
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(order.customerPhone ?? "")\n\n"))
        }
        
        textst?.isBold = Set_Enabled
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(getItemName(price: "Price", name: "Name", strQty: "Qty", VAT: "VAT"))\n"))
        
        textst?.isBold = Set_DisEnable
        
        var vatAmount = 0.0
        order.arrItems?.forEach({ model in
            vatAmount += (model.taxAmount ?? 0)
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(getItemName(price: String.init(format: "%.2f", model.itemTotalPrice ?? 0), name: model.itemName ?? "", strQty: String.init(format: "%.1f", model.itemQty ?? 0), VAT: String.init(format: "%.2f", model.taxAmount ?? 0)))\n"))
        })
        
        textst?.alignmode = Align_Right
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: String.init(format: "Sub Total: %.2f\n", order.subTotal ?? 0)))
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: String.init(format: "VAT: %.2f\n", vatAmount)))
        
        if order.discount ?? 0 > 0{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: String.init(format: "Discount: %.2f\n", order.discount ?? 0)))
        }
        
        textst?.isBold = Set_Enabled
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: String.init(format: "Grand Total: %.2f\n", order.orderTotal ?? 0)))
        
        if isSalesOrder{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: String.init(format: "Cash Payment: %.2f\n", order.cashAmount ?? 0)))
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: String.init(format: "Card Payment: %.2f\n", order.cardAmount ?? 0)))
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: String.init(format: "Cheque Payment: %.2f\n", order.chequeAmount ?? 0)))
        }
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n\n"))
        
        textst?.alignmode = Align_Center
        
        textst?.isTimes_Wide = Set_Enabled
        textst?.isTimes_Heigh = Set_Enabled
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Thank You\n\n\n"))
        
        textCMD?.append(textCMD?.getLFCmd())
        textCMD?.append(textCMD?.getLFCmd())
        textCMD?.append(textCMD?.getLFCmd())
        textCMD?.append(textCMD?.getLFCmd())
        
        textCMD?.append(textCMD?.getCutPaperCmd(CutterMode_Full))
        
        printer?.write(textCMD?.getCmd())
        
        printer?.close()
    }
    
    func printReport(report:FullReportModel){
        let imageCMD = ESCFactory.create()
        imageCMD?.clear()
        imageCMD?.encodingType = Encoding_UTF8
        imageCMD?.append(imageCMD?.getHeaderCmd())
        let bitmapSetting = printer?.bitmapSetts
        bitmapSetting?.alignmode = Align_Center
        bitmapSetting?.limitWidth = 30*8
        
        let img = UIImage.init(named: "AppBlackLogo")!
        imageCMD?.append(imageCMD?.getBitMapCmd(bitmapSetting, image: img))
        imageCMD?.append(imageCMD?.getPrintEnd(1))
        printer?.write(imageCMD?.getCmd())
        
        let textCMD = ESCFactory.create()
        textCMD?.clear()
        textCMD?.encodingType = Encoding_GBK
        textCMD?.append(textCMD?.getHeaderCmd())
        
        let textst = self.printer?.textSets
        textst?.isUnderline = Set_DisEnable
        
        textst?.isBold = Set_Enabled
        textst?.isTimes_Wide = Set_Enabled
        textst?.isTimes_Heigh = Set_Enabled
        textst?.alignmode = Align_Center
        
        if let name = AppConstants.dictGlobalFoodInfo.getStringValue("client_name"){
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(name)\n"))
        }
        
        
        textst?.isTimes_Wide = Set_DisEnable
        textst?.isTimes_Heigh = Set_DisEnable
        var strAddress = ""
        if let address1 = AppConstants.dictGlobalFoodInfo.getStringValue("client_address1"){
            strAddress = address1
        }
        if let street = AppConstants.dictGlobalFoodInfo.getStringValue("client_street"){
            if strAddress == ""{
                strAddress = street
            }else{
                strAddress = "\(strAddress),\(street)"
            }
        }
        if strAddress != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(strAddress)\n"))
        }
        
        var strCity = ""
        if let city = AppConstants.dictGlobalFoodInfo.getStringValue("client_city"){
            strCity = city
        }
        if let pincode = AppConstants.dictGlobalFoodInfo.getStringValue("client_pincode"){
            if strCity == ""{
                strCity = pincode
            }else{
                strCity = "\(strCity)-\(pincode)"
            }
        }
        if strCity != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(strCity)\n"))
        }
        
        var strCounty = ""
        if let state = AppConstants.dictGlobalFoodInfo.getStringValue("client_state"){
            strCounty = state
        }
        if let country = AppConstants.dictGlobalFoodInfo.getStringValue("client_country"){
            if strCounty == ""{
                strCounty = country
            }else{
                strCounty = "\(strCounty),\(country)"
            }
        }
        if strCounty != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "\(strCounty)\n"))
        }
        
        if let email = AppConstants.dictGlobalFoodInfo.getStringValue("client_email"){
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "E:\(email.lowercased())\n"))
        }
        
        var strPhone = ""
        if let phone = AppConstants.dictGlobalFoodInfo.getStringValue("client_contact_number_1"){
            strPhone = phone
        }
        if let phone = AppConstants.dictGlobalFoodInfo.getStringValue("client_contact_number_2"){
            if strPhone == ""{
                strPhone = phone
            }else{
                strPhone = "\(strPhone)\n\(phone)"
            }
        }
        if strPhone != ""{
            textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "P:\(strPhone)\n"))
        }
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "VAT NO: 229179190\n"))
        
        textst?.isTimes_Wide = Set_Enabled
        textst?.isTimes_Heigh = Set_Enabled
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n\n"))
        textst?.isTimes_Wide = Set_DisEnable
        textst?.isTimes_Heigh = Set_DisEnable
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Sales Report\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n\n"))
        textst?.isBold = Set_DisEnable
        
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Date: \(fromDate) - \(toDate)\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n"))
        
        textst?.alignmode = Align_Left
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Total Item Purchased:  \(String.init(format: "%.1f", report.totalItemCount ?? 0))\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Total Cash Order:      \(report.totalCodOrderCount ?? 0)\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Value Cash Order:      \(String.init(format: "%.2f", report.codOrderValue ?? 0))\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Total Card Order:      \(report.totalCardOrderCount ?? 0)\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Value Card Order:      \(String.init(format: "%.2f", report.cardOrderValue ?? 0))\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Total Cheque Order:    \(report.totalChequeOrderCount ?? 0)\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Value Cheque Order:    \(String.init(format: "%.2f", report.chequeOrderValue ?? 0))\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Total Refunded Amount: \(String.init(format: "%.2f", report.totalRefundAmount ?? 0))\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Total Order:           \(report.totalOrderCount ?? 0)\n"))
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Value Order:           \(String.init(format: "%.2f", report.orderValue ?? 0))\n"))
        
        textst?.alignmode = Align_Center
        textst?.isBold = Set_Enabled
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "--------------------\n\n"))
        
        textst?.isTimes_Wide = Set_Enabled
        textst?.isTimes_Heigh = Set_Enabled
        
        textCMD?.append(textCMD?.getCmdWithPrint(textst, text: "Thank You\n\n\n"))
        
        textCMD?.append(textCMD?.getLFCmd())
        textCMD?.append(textCMD?.getLFCmd())
        textCMD?.append(textCMD?.getLFCmd())
        textCMD?.append(textCMD?.getLFCmd())
        
        textCMD?.append(textCMD?.getCutPaperCmd(CutterMode_Full))
        
        printer?.write(textCMD?.getCmd())
        
        printer?.close()
    }
    
    func openCashDrawer(){
        let textCMD = ESCFactory.create()
        textCMD?.clear()
        textCMD?.encodingType = Encoding_GBK
        textCMD?.append(textCMD?.getHeaderCmd())
        
        textCMD?.append(textCMD?.getOpenDrawerCmd(0, startTime: 5, endTime: 0))

        printer?.write(textCMD?.getCmd())
        printer?.close()
    }
    
    func getItemName(price:String, name:String ,strQty:String, VAT:String)->String{
        var strName = ""
        if strQty.count == 3{
            strName = "  \(strQty) \(name)"
        }else if strQty.count == 4{
            strName = " \(strQty) \(name)"
        }else{
            strName = "\(strQty) \(name)"
        }
//        var strVAT = VAT
//        if VAT.count == 3{
//            strVAT = "\(VAT)    "
//        }else if VAT.count == 4{
//            strVAT = "\(VAT)   "
//        }else if VAT.count == 5{
//            strVAT = "\(VAT)  "
//        }else if VAT.count == 6{
//            strVAT = "\(VAT) "
//        }
        var strPrice = price
        if price.count == 3{
            strPrice = "    \(price)"
        }else if price.count == 4{
            strPrice = "   \(price)"
        }else if price.count == 5{
            strPrice = "  \(price)"
        }else if price.count == 6{
            strPrice = " \(price)"
        }
        
        let count = strName.count
        if count > (46-strPrice.count){
            let arrStr = strName.split(every: (46-strPrice.count))
            var strNewName = ""
            for i in 0..<arrStr.count{
                if i == 0{
                    strNewName = "\(arrStr[i]) \(strPrice)\n"
                }else{
                    strNewName = "\(strNewName)\(arrStr[i])\n"
                }
            }
            return strNewName
        }else{
            while strName.count < (46-strPrice.count){
                strName += " "
            }
            return "\(strName) \(strPrice)\n"
        }
    }
}

extension Cmd{
    func getCmdWithPrint(_ textSetting: TextSetting!, text: String!) -> Data{
        if AppConstants.isInDevlopmentMode{
            print(text ?? "")
        }
        return self.getTextCmd(textSetting, text: text)
    }
}
