//
//  Validation.swift
//  globalfoods
//
//  Created by Kamal Parmar on 10/07/20.
//  Copyright © 2020 Kamal Parmar. All rights reserved.
//

import Foundation
import UIKit

enum ValidationType {
    case email
    case stringWithFirstLetterCaps
    case phoneNo
    case alphabeticString
    case password
    case emptyField
}

enum RegEx: String {
    case email = "[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,4}" // Email
    case password = "(?=.*[a-z])(?=.*[0-9])(?=.*[A-Za-z]).{8,16}" // Password length 6-15
    case alphabeticStringWithSpace = "^[a-zA-Z ]*$" // e.g. hello sandeep
    case alphabeticStringFirstLetterCaps = "^[A-Z]+[a-zA-Z]*$" // SandsHell
    case phoneNo = "[0-9]{10,14}" // PhoneNo 10-14 Digits
    
    //Change RegEx according to your Requirement
}

enum BottomAlertType{
    case kNormal
    case kNewOrder
    case kNewReservation
}

struct BottomAlertSettings{
    
    static let shared = BottomAlertSettings()
    
    var strMessage = ""
    var isClickable = false
    var id = 0
    var type = BottomAlertType.kNormal
    var orderType = ""
    var timeToHide = 5
}

class AppValidation: NSObject {
    
    static let shared = AppValidation()
    var settings:BottomAlertSettings!
    var bottomAlert:UIView?
    
    //MARK: - BottomAlertView
    func showBottomAlertView(settings:BottomAlertSettings){
        if settings.strMessage == ""{
            return
        }
        if bottomAlert != nil{
            self.bottomAlert!.removeFromSuperview()
            self.bottomAlert = nil
        }
        self.settings = settings
        var height = settings.strMessage.getHeightFromString(font: AppConstants.GlobalFontConstants.kRegularFont(size: 17), width: UIScreen.main.bounds.size.width-32)
        height = max(40, (height + 16))
        let width = UIScreen.main.bounds.size.width
        bottomAlert = UIView.init(frame: CGRect.init(x: 0, y: UIScreen.main.bounds.size.height + 30 , width: width, height: height + AppConstants.ScreenSize.bottomPadding))
        bottomAlert!.backgroundColor = AppConstants.Colors.kAppMainThemeColor
        
        let lblMessage = UILabel.init(frame: CGRect.init(x: 8, y: 8, width: width - 16, height: height - 16))
        lblMessage.textColor = .white
        lblMessage.numberOfLines = 0
        lblMessage.font = AppConstants.GlobalFontConstants.kRegularFont(size: 17)
        if let attr = settings.strMessage.htmlToAttributedString(font: AppConstants.GlobalFontConstants.kRegularFont(size: 17), color: .white){
            lblMessage.attributedText = attr
        }else{
            lblMessage.text = settings.strMessage
        }
        bottomAlert!.addSubview(lblMessage)
        
        if settings.isClickable{
            let btnView = UIButton.init(frame: CGRect.init(x: 0, y: 0, width: width, height: height))
            btnView.addTarget(self, action: #selector(btnViewAction), for: .touchUpInside)
            bottomAlert!.addSubview(btnView)
        }
        
        AppConstants.appDelegate.window!.addSubview(bottomAlert!)
        
        UIView.animate(withDuration: 0.5) {
            self.bottomAlert!.frame.origin.y = UIScreen.main.bounds.size.height - AppConstants.ScreenSize.bottomPadding - height
        }
        DispatchQueue.main.asyncAfter(deadline: .now() + DispatchTimeInterval.seconds(settings.timeToHide)) {
            if self.bottomAlert != nil{
                UIView.animate(withDuration: 0.5, animations: {
                    self.bottomAlert!.frame.origin.y = UIScreen.main.bounds.size.height + 30
                }) { (success) in
                    self.bottomAlert!.removeFromSuperview()
                    self.bottomAlert = nil
                }
            }
        }
    }
    
    @IBAction func btnViewAction(){
        UIView.animate(withDuration: 0.5, animations: {
            self.bottomAlert!.frame.origin.y = UIScreen.main.bounds.size.height + 30
        }) { (success) in
            self.bottomAlert!.removeFromSuperview()
            self.bottomAlert = nil
        }
        
    }
    
    //MARK: - UIAlertController
    func showAlertView(parentVC:UIViewController,withAlertTile title:String?,withAlertMsg msg:String,withActionsTitle btnTitles:[String],withActionsType styles:[UIAlertAction.Style],withHandler handler:@escaping ((Int) -> Void )){
        if msg == ""{
            return
        }
        let alert : UIAlertController = UIAlertController.init(title: title, message: msg, preferredStyle: .alert)
        for i in 0..<btnTitles.count{
            let action = UIAlertAction.init(title: btnTitles[i], style: styles[i]) { (action) in
                handler(i)
            }
            alert.addAction(action)
        }
        parentVC.present(alert, animated: true, completion: nil)
    }
    
    //MARK: Validation
    func isValidRegEx(_ testStr: String, _ regex: RegEx) -> Bool {
        let stringTest = NSPredicate(format:"SELF MATCHES %@", regex.rawValue)
        let result = stringTest.evaluate(with: testStr)
        return result
    }
}
