//
//  WebServiceManager.swift
//   globalfoods
//
//  Created by Tushar Premal on 5/29/18.
//  Copyright © 2018 globalfoods. All rights reserved.
//

import Foundation
import UIKit
import MobileCoreServices

public class WebServiceManager: NSObject
{
    
    public var numOfAPIRequests : NSInteger = 0
    static let shared = WebServiceManager()
    
    private override init() {
        
    }
    
    //create query string from parameter dictionary for GET method
    func queryString(parameters : NSDictionary) -> String
    {
        var firstPass = true
        var result = ""
        for (key, value) in parameters
        {
            
            let encodedKey = (key as! NSString).addingPercentEncoding(withAllowedCharacters: .urlHostAllowed)!
            let encodedValue = (value as! NSString).addingPercentEncoding(withAllowedCharacters: .urlHostAllowed)!
            result += firstPass ? "\(encodedKey)=\(encodedValue)" : "&\(encodedKey)=\(encodedValue)"
            firstPass = false;
        }
        
        return result
    }
    
    func requestAPI(params:NSMutableDictionary, urlString : String, method : String, postCompleted : @escaping (_ msg: String, _ responsedata: NSDictionary?) -> ())
    {
        if Reachabilities.isConnectedToNetwork(){
            var strURL = urlString
            if method=="GET"
            {
                strURL = "\(urlString)?\(queryString(parameters: params))" //append query string at the end of the url
                
            }
            
            //create url connection from url
            var request = URLRequest(url:URL(string:strURL.addingPercentEncoding(withAllowedCharacters: CharacterSet.urlQueryAllowed) ?? "")!)
            request.httpMethod=method
            request.timeoutInterval=60
            
            if method=="POST" || method=="PUT"
            {
                //create body from dictionary paramter for POST method
                do {
                    request.httpBody = try JSONSerialization.data(withJSONObject: params, options: .prettyPrinted)//try createBody(params, boundary as String)
                } catch let error{
                    print(error.localizedDescription)
                }
            }
            if AppConstants.isInDevlopmentMode{
                print("URL: ",strURL)
            }
            request.setValue("application/json", forHTTPHeaderField: "Content-Type")
            DispatchQueue.main.async {
                UIApplication.shared.isNetworkActivityIndicatorVisible = true //show network indicator in main thread
            }
            numOfAPIRequests += 1
            let task = URLSession.shared.dataTask(with: request) { (data, response, error) in
                self.numOfAPIRequests-=1
                if(self.numOfAPIRequests==0)
                {
                    DispatchQueue.main.async {
                        UIApplication.shared.isNetworkActivityIndicatorVisible = false //hide network indicator in main thread if all api calling is completed
                    }
                    
                }
                
                if (error != nil)
                {
                    postCompleted(error!.localizedDescription, nil) //completion handler with error message
                }
                else
                {
                    
                    //                    if AppConstants.isInDevlopmentMode{
                    //                        let outputString = String(data:data!, encoding:.utf8)!
                    //                        print(urlString+"\n"+outputString)
                    //                    }
                    
                    //parse jsondata into dictionary
                    let jsonData: Any?
                    do {
                        jsonData = try JSONSerialization.jsonObject(with: data!, options:JSONSerialization.ReadingOptions.mutableContainers )
                        if let dictResponse = jsonData as? NSDictionary{
                            postCompleted("SUCCESS",dictResponse) //completion handler with response
                        }else{
                            postCompleted(AppConstants.GlobalAlert.kServerErrorMsg, nil)
                        }
                    } catch let error {
                        jsonData = nil
                        postCompleted(error.localizedDescription, nil)
                    }
                }
            }
            task.resume()
        }else{
            postCompleted("", nil)
        }
        
    }
    
    func requestAPIwithURlEncoding(params:[String:String], urlString : String, postCompleted : @escaping (_ msg: String, _ responsedata: NSDictionary?) -> ())
    {
        if Reachabilities.isConnectedToNetwork(){
            let strURL = urlString
            
            //create url connection from url
            var request = URLRequest(url:URL(string:strURL)!)
            request.httpMethod = "POST"
            request.timeoutInterval = 120
            
            let jsonString = params.reduce("") { "\($0)\($1.0)=\($1.1)&" }
            let jsonData = jsonString.data(using: .utf8, allowLossyConversion: false)!
            request.addValue("application/x-www-form-urlencoded", forHTTPHeaderField:"Content-Type")
            request.httpBody  = jsonData
            DispatchQueue.main.async {
                UIApplication.shared.isNetworkActivityIndicatorVisible = true //show network indicator in main thread
            }
            numOfAPIRequests += 1
            let task = URLSession.shared.dataTask(with: request) { (data, response, error) in
                self.numOfAPIRequests-=1
                if(self.numOfAPIRequests==0)
                {
                    DispatchQueue.main.async {
                        UIApplication.shared.isNetworkActivityIndicatorVisible = false //hide network indicator in main thread if all api calling is completed
                    }
                    
                }
                
                if (error != nil)
                {
                    postCompleted(error!.localizedDescription, nil) //completion handler with error message
                }
                else
                {
                    if AppConstants.isInDevlopmentMode{
                        let outputString = String(data:data!, encoding:.utf8)!
                        print(urlString+"\n"+outputString)
                    }
                    //parse jsondata into dictionary
                    let jsonData: Any?
                    do {
                        jsonData = try JSONSerialization.jsonObject(with: data!, options:JSONSerialization.ReadingOptions.mutableContainers )
                        if let dictResponse = jsonData as? NSDictionary{
                            postCompleted("SUCCESS",dictResponse) //completion handler with response
                        }else{
                            postCompleted(AppConstants.GlobalAlert.kServerErrorMsg, nil)
                        }
                    } catch let error {
                        jsonData = nil
                        postCompleted(error.localizedDescription, nil)
                    }
                }
            }
            task.resume()
        }else{
            postCompleted("", nil)
        }
        
    }
    
    func requestAPIwithMultipart(params:NSMutableDictionary, urlString : String, postCompleted : @escaping (_ msg: String, _ responsedata: NSDictionary?) -> ())
    {
        if Reachabilities.isConnectedToNetwork(){
            
            //create url connection from url
            var request = URLRequest(url:URL(string:urlString)!)
            request.httpMethod = "POST"
            request.timeoutInterval = 60
            
            request.addValue("multipart/form-data", forHTTPHeaderField:"Content-Type")
            let boundary : NSString = "---------------------------14737809831466499882746641449"
            let contentType : NSString = NSString(format:"multipart/form-data; boundary=%@",boundary)
            
            request.addValue(contentType as String, forHTTPHeaderField: "Content-Type")
            do {
                request.httpBody = try createBody(params, boundary as String)
                
            } catch let error{
                postCompleted(error.localizedDescription, nil)
            }
            
            DispatchQueue.main.async {
                UIApplication.shared.isNetworkActivityIndicatorVisible = true //show network indicator in main thread
            }
            numOfAPIRequests += 1
            let task = URLSession.shared.dataTask(with: request) { (data, response, error) in
                self.numOfAPIRequests-=1
                if(self.numOfAPIRequests==0)
                {
                    DispatchQueue.main.async {
                        UIApplication.shared.isNetworkActivityIndicatorVisible = false //hide network indicator in main thread if all api calling is completed
                    }
                    
                }
                
                if (error != nil)
                {
                    postCompleted(error!.localizedDescription, nil) //completion handler with error message
                }
                else
                {
                    
                    //                    if AppConstants.isInDevlopmentMode{
                    //                        let outputString = String(data:data!, encoding:.utf8)!
                    //                        print(urlString+"\n"+outputString)
                    //                    }
                    
                    //parse jsondata into dictionary
                    let jsonData: Any?
                    do {
                        jsonData = try JSONSerialization.jsonObject(with: data!, options:JSONSerialization.ReadingOptions.mutableContainers )
                        if let dictResponse = jsonData as? NSDictionary{
                            postCompleted("SUCCESS",dictResponse) //completion handler with response
                        }else{
                            postCompleted(AppConstants.GlobalAlert.kServerErrorMsg, nil)
                        }
                    } catch let error {
                        jsonData = nil
                        postCompleted(error.localizedDescription, nil)
                    }
                }
            }
            task.resume()
        }else{
            postCompleted("", nil)
        }
        
    }
    
    func createBody(_ parameters: NSMutableDictionary,_ boundary: String) throws -> Data {
        
        var body = Data()
        
        for (key, value) in parameters {
            
            if(value is String) {
                body.append("--\(boundary)\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                body.append("Content-Disposition: form-data; name=\"\(key)\"\r\n\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                body.append("\(value)\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
            }else if let FileData = value as? NSDictionary {
                for (key2, value) in FileData {
                    let filename = key2 as! String
                    let data = value as! Data
                    let mimetype = mimeTypeForPath(path: filename)
                    body.append("--\(boundary)\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                    body.append("Content-Disposition: form-data; name=\"\(key)\"; filename=\"\(filename)\"\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                    body.append("Content-Type: \(mimetype)\r\n\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                    body.append(data)
                    body.append("\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                }
            }else if let arr = value as? [NSDictionary] {
                for FileData in arr{
                    for (key2, value) in FileData {
                        let filename = key2 as! String
                        let data = value as! Data
                        let mimetype = mimeTypeForPath(path: filename)
                        body.append("--\(boundary)\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                        body.append("Content-Disposition: form-data; name=\"\(key)\"; filename=\"\(filename)\"\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                        body.append("Content-Type: \(mimetype)\r\n\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                        body.append(data)
                        body.append("\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
                    }
                }
            }
        }
        
        body.append("--\(boundary)--\r\n".data(using: String.Encoding(rawValue: String.Encoding.utf8.rawValue))!)
        
        return body
        
    }
    
    func mimeTypeForPath(path: String) -> String {
        let pathExtension = path.pathExtension
        var stringMimeType = "application/octet-stream";
        if let uti = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, pathExtension! as CFString, nil)?.takeRetainedValue() {
            if let mimetype = UTTypeCopyPreferredTagWithClass(uti, kUTTagClassMIMEType)?.takeRetainedValue() {
                stringMimeType = mimetype as NSString as String
            }
        }
        return stringMimeType;
    }
}

extension String {
    var ns: NSString {
        return self as NSString
    }
    var pathExtension: String? {
        return ns.pathExtension
    }
    var lastPathComponent: String? {
        return ns.lastPathComponent
    }
}
