//
//  ViewController.swift
//  globalfoods
//
//  Created by Apple on 25/12/20.
//

import UIKit
import SideMenuSwift

struct UserType {
    static let kChef = 1
    static let kEmployee = 2
    static let kCustomer = 0
}

class LoginVC: UIViewController {

    @IBOutlet weak var txtEmailId:UITextField!
    @IBOutlet weak var txtPassword:UITextField!
    @IBOutlet weak var viewForEmployee:UIViewX!
    @IBOutlet weak var viewForCustomer:UIViewX!
    @IBOutlet weak var viewForChef:UIViewX!
    @IBOutlet weak var viewForChefSelect:UIView!
    @IBOutlet weak var viewForEmployeeSelect:UIView!
    @IBOutlet weak var viewForCustomerSelect:UIView!
    var selectedUser = UserType.kCustomer
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        setLoginType()
    }

    //MARK: create viewcontroller instance
    static func instance()->LoginVC?{
        return UIStoryboard.init(name: "Main", bundle: nil).instantiateViewController(withIdentifier: "LoginVC") as? LoginVC
    }
    
    func setLoginType(){
        viewForEmployee.borderColor = AppConstants.Colors.kAppLightGrayColor
        viewForCustomer.borderColor = AppConstants.Colors.kAppLightGrayColor
        viewForEmployeeSelect.isHidden = true
        viewForCustomerSelect.isHidden = true
        viewForChef.borderColor = AppConstants.Colors.kAppLightGrayColor
        viewForChefSelect.isHidden = true
        if selectedUser == UserType.kCustomer{
            viewForCustomer.borderColor = AppConstants.Colors.kAppDarkThemeColor
            viewForCustomerSelect.isHidden = false
        }else if selectedUser == UserType.kEmployee{
            viewForEmployee.borderColor = AppConstants.Colors.kAppDarkThemeColor
            viewForEmployeeSelect.isHidden = false
        }else{
            viewForChef.borderColor = AppConstants.Colors.kAppDarkThemeColor
            viewForChefSelect.isHidden = false
        }
    }
    
    //MARK: validation methods
    func checkLoginValidation()->Bool{
        if (txtEmailId.text?.trimmingCharacters(in: .whitespaces).isEmpty)!
        {
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_EMPTY_EMAIL, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                _ = self.txtEmailId.becomeFirstResponder()
            }
            return false

        }else if !(AppValidation.shared.isValidRegEx(txtEmailId.text!, .email))
        {
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_INVALID_EMAIL, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                _ = self.txtEmailId.becomeFirstResponder()
            }
            return false
        }
        else if (txtPassword.text?.trimmingCharacters(in: .whitespaces).isEmpty)!
        {
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_EMPTY_PASSWORD, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                _ = self.txtPassword.becomeFirstResponder()
            }
            return false
        }
        return true
    }
    
    @IBAction func btnLoginAction(_ sender:UIButton){
        self.view.endEditing(true)
        if checkLoginValidation(){
            callLoginAPI()
        }
    }
    
    @IBAction func btnLoginTypeAction(_ sender:UIButton){
        self.view.endEditing(true)
        selectedUser = sender.tag
        setLoginType()
    }
    
    func callLoginAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["email"] = txtEmailId.text ?? ""
        param["imei"] = AppCommonMethods.getUUID()
        param["refreshToken"] = AppConstants.deviceToken ?? ""
        param["password"] = txtPassword.text ?? ""
        AppCommonMethods.startProgressBar()
        if selectedUser == UserType.kCustomer{
            param["s"] = "1"
        }else if selectedUser == UserType.kChef{
            param["s"] = "125"
        }else{
            param["s"] = "11"
        }
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: (selectedUser != UserType.kEmployee) ? AppConstants.APIURLs.kServiceCustomerURL : AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Login Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if self.selectedUser == UserType.kCustomer{
                            AppCommonMethods.getCustomerDataAPI(userId: result.getIntValue("id")) { (status) in
                                if status{
                                    var setting = BottomAlertSettings()
                                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                                    AppValidation.shared.showBottomAlertView(settings: setting)
                                    if let VC = SelectBranchVC.instance(){
                                        self.navigationController?.pushViewController(VC, animated: true)
                                    }
                                    AppCommonMethods.stopProgressBar()
                                }else{
                                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                                        
                                    }
                                }
                            }
                            return
                        }else{
                            AppCommonMethods.stopProgressBar()
                            if let data = AppCommonMethods.convertToJson(object: result){
                                let jsonDecoder = JSONDecoder()
                                if self.selectedUser == UserType.kEmployee{
                                    if let aLoginData = try? jsonDecoder.decode(EmployeeModel.self, from: data){
                                        var setting = BottomAlertSettings()
                                        setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                                        AppValidation.shared.showBottomAlertView(settings: setting)
                                        if let encoded = AppCommonMethods.getjsonDataFromCodable(object: aLoginData) {
                                            AppCommonMethods.saveDataIntoUserDefault(value: encoded, withKey: AppConstants.userDefaultKey.kEmployeeData)
                                        }
                                        if let VC = SelectBranchVC.instance(){
                                            VC.isPrinterView = true
                                            AppConstants.appDelegate.window!.rootViewController = VC
                                        }
                                        AppCommonMethods.getCartsFromLayAway()
                                        return
                                    }
                                }else{
                                    if let aLoginData = try? jsonDecoder.decode(SubCustomerModel.self, from: data){
                                        var setting = BottomAlertSettings()
                                        setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                                        AppValidation.shared.showBottomAlertView(settings: setting)
                                        if let encoded = AppCommonMethods.getjsonDataFromCodable(object: aLoginData) {
                                            AppCommonMethods.saveDataIntoUserDefault(value: encoded, withKey: AppConstants.userDefaultKey.kSubCustomerData)
                                        }
                                        if let VC = UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "SideMenuController") as? SideMenuController{
                                            AppCommonMethods.setupSideView(VC)
                                            AppConstants.appDelegate.window!.rootViewController = VC
                                        }
                                        return
                                    }
                                }
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppCommonMethods.stopProgressBar()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
}

