//
//  AddSalesRepoVC.swift
//  globalfoods
//
//  Created by Apple on 18/01/21.
//

import UIKit
import DropDown

class AddSalesRepoVC: UIViewController {

    @IBOutlet weak var txtCustomerSearch:UITextField!
    @IBOutlet weak var txtBranchSearch:UITextField!
    @IBOutlet weak var lblPlaceholder:UILabel!
    @IBOutlet weak var lblRemarkCount:UILabel!
    @IBOutlet weak var txtReamark:UITextView!
    
    var customHeaderView:HeaderView?
    var popRecognizer: InteractivePopRecognizer?
    var customerData:UserModel?
    var branchData:branchModel?
    var arrCustomers = [UserModel]()
    var arrSearchCustomers = [UserModel]()
    var isCustomerSearchEnabled = false
    var isBranchSearchEnabled = false
    var arrBranches = [branchModel]()
    var arrSearchBranches = [branchModel]()
    let customerDropDown = DropDown()
    let branchDropDown = DropDown()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        callGetCustomersAPI()
        if branchData == nil{
            self.txtBranchSearch.isUserInteractionEnabled = false
        }else{
            callGetBranchesAPI()
        }
        setHeaderView()
        setInteractiveRecognizer()
    }
    
    //MARK: create viewcontroller instance
    class func instance()->AddSalesRepoVC?{
        return UIStoryboard.init(name: "Employees", bundle: nil).instantiateViewController(withIdentifier: "AddSalesRepoVC") as? AddSalesRepoVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Add Sales Repo", isBackEnbaled: true)
    }

    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func txtCustomerSearchChanged(_ sender:UITextField){
        if sender.text ?? "" == ""{
            isCustomerSearchEnabled = false
        }else{
            isCustomerSearchEnabled = true
            arrSearchCustomers = arrCustomers.filter({ (model) -> Bool in
                return (model.name ?? "").lowercased().contains((sender.text ?? "").lowercased())
            })
        }
        var arrName = [String]()
        for model in (isCustomerSearchEnabled ? arrSearchCustomers : arrCustomers){
            arrName.append(model.name ?? "")
        }
        customerDropDown.dataSource = arrName
        if customerDropDown.isDropdownHidden{
            customerDropDown.show()
        }else{
            customerDropDown.reloadAllComponents()
        }
    }
    
    @IBAction func txtBranchSearchChanged(_ sender:UITextField){
        if sender.text ?? "" == ""{
            isBranchSearchEnabled = false
        }else{
            isBranchSearchEnabled = true
            arrSearchBranches = arrBranches.filter({ (model) -> Bool in
                return (model.name ?? "").lowercased().contains((sender.text ?? "").lowercased())
            })
        }
        var arrName = [String]()
        for model in (isBranchSearchEnabled ? arrSearchBranches : arrBranches){
            arrName.append(model.name ?? "")
        }
        branchDropDown.dataSource = arrName
        if branchDropDown.isDropdownHidden{
            branchDropDown.show()
        }else{
            branchDropDown.reloadAllComponents()
        }
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        if customerData == nil{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_CUSTOMER, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else if branchData == nil{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_BRANCH, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else if (txtReamark.text?.trimmingCharacters(in: .whitespaces).isEmpty)!
        {
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg:  AppConstants.GlobalAlert.ALERT_ENTER_REMARK, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                _ = self.txtReamark.becomeFirstResponder()
            }
        }else{
            callAddSalesRepoAPI()
        }
    }
    
    func callGetCustomersAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "59"
        param["eid"] = "\(AppConstants.employeeData?.id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCustomerURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrCustomers.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Customers Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([UserModel].self, from: data){
                                self.arrCustomers = aData
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
    func callAddSalesRepoAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "75"
        param["emp_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["cuid"] = "\(customerData?.id ?? 0)"
        param["cbid"] = "\(branchData?.id ?? 0)"
        param["remarks"] = txtReamark.text ?? ""
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCustomerURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Add repo Data:",response!)
                    }
                    aMessage = response!.getStringValue("ack_msg") ?? ""
                    if response!.getIntValue("ack") ?? 0 == 1{
                        var setting = BottomAlertSettings()
                        setting.strMessage = aMessage
                        AppValidation.shared.showBottomAlertView(settings: setting)
                        self.navigationController?.popViewController(animated: true)
                        return
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
    func callGetBranchesAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "48"
        param["cuid"] = "\(customerData?.id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrBranches.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("branch Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([branchModel].self, from: data){
                                self.arrBranches = aData
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension AddSalesRepoVC:UITextFieldDelegate{
    func textFieldDidBeginEditing(_ textField: UITextField) {
        if textField != txtBranchSearch{
            isCustomerSearchEnabled = false
            txtCustomerSearch.text = ""
            customerDropDown.dismissMode = .manual
            customerDropDown.anchorView = txtCustomerSearch
            customerDropDown.bottomOffset = CGPoint(x: 0, y: txtCustomerSearch.bounds.height)
            customerDropDown.topOffset = CGPoint(x: 0, y: -txtCustomerSearch.bounds.height)
            var arrName = [String]()
            for model in (isCustomerSearchEnabled ? arrSearchCustomers : arrCustomers){
                arrName.append(model.name ?? "")
            }
            customerDropDown.dataSource = arrName
            // Action triggered on selection
            customerDropDown.selectionAction = { [weak self] (index, item) in
                if ((self?.isCustomerSearchEnabled ?? false) ? self?.arrSearchCustomers : self?.arrCustomers)?[index].id ?? 0 != self?.customerData?.id ?? 0{
                    self?.customerData = ((self?.isCustomerSearchEnabled ?? false) ? self?.arrSearchCustomers : self?.arrCustomers)?[index]
                    self?.txtBranchSearch.text = ""
                    self?.txtBranchSearch.isUserInteractionEnabled = true
                    self?.view.endEditing(true)
                    self?.callGetBranchesAPI()
                }
                self?.txtCustomerSearch.text = item
            }
            customerDropDown.show()
        }else{
            isBranchSearchEnabled = false
            txtBranchSearch.text = ""
            branchDropDown.dismissMode = .manual
            branchDropDown.anchorView = txtBranchSearch
            branchDropDown.bottomOffset = CGPoint(x: 0, y: txtBranchSearch.bounds.height)
            branchDropDown.topOffset = CGPoint(x: 0, y: -txtBranchSearch.bounds.height)
            var arrName = [String]()
            for model in (isBranchSearchEnabled ? arrSearchBranches : arrBranches){
                arrName.append(model.name ?? "")
            }
            branchDropDown.dataSource = arrName
            // Action triggered on selection
            branchDropDown.selectionAction = { [weak self] (index, item) in
                self?.branchData = ((self?.isBranchSearchEnabled ?? false) ? self?.arrSearchBranches : self?.arrBranches)?[index]
                self?.txtBranchSearch.text = item
                self?.view.endEditing(true)
            }
            branchDropDown.show()
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        txtCustomerSearch.text = customerData?.name ?? ""
        txtBranchSearch.text = branchData?.name ?? ""
    }
}

extension AddSalesRepoVC:UITextViewDelegate{
    func textViewDidChange(_ textView: UITextView) {
        lblRemarkCount.text = "\(textView.text!.count)/300"
        if textView.text! == ""{
            lblPlaceholder.isHidden = false
        }else{
            lblPlaceholder.isHidden = true
        }
    }
    
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        if text == ""{
            return true
        }
        if textView.text.count >= 300{
            return false
        }
        return true
    }
}
