//
//  ApproveOrderVC.swift
//  globalfoods
//
//  Created by Apple on 29/01/21.
//

import UIKit
import SDWebImage
import ActionSheetPicker_3_0

class ApproveOrderVC: UIViewController {

    @IBOutlet weak var txtSelectDate:UITextField!
    @IBOutlet weak var tblOrders:UITableView!
    @IBOutlet weak var lblPlaceholder:UILabel!
    @IBOutlet weak var lblOrderNo:UILabel!
    @IBOutlet weak var lblRemarkCount:UILabel!
    @IBOutlet weak var txtReamark:UITextView!
    @IBOutlet weak var lblCurrentPage:UILabel!
    @IBOutlet weak var scrollOrderInfo:UIScrollView!
    
    var customHeaderView:HeaderView?
    var popRecognizer: InteractivePopRecognizer?
    var orderDetail:OrderHistoryModel!
    var arrItems = [CartItemsModel]()
    var strStoreDate = ""
    var currentPage = 1
    var strRemark = ""
    var selectedRow = 0
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        setInteractiveRecognizer()
        setHeaderView()
        lblCurrentPage.text = "\(currentPage)/2"
        self.lblOrderNo.text = "Order No: #\(orderDetail.orderNo ?? "")"
        if currentPage == 1{
            callGetOrderDetailAPI()
            scrollOrderInfo.isHidden = false
            tblOrders.superview?.isHidden = true
        }else{
            self.arrItems = orderDetail.arrItems ?? [CartItemsModel]()
            self.arrItems.forEach({ (model) in
                model.approveQty = model.itemQty
                model.approvePrice = model.itemPrice
                model.itemTotalPrice = (model.itemQty ?? 0) * (model.itemPrice ?? 0)
                model.isSelected = true
            })
            
            scrollOrderInfo.isHidden = true
            tblOrders.superview?.isHidden = false
        }
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->ApproveOrderVC?{
        return UIStoryboard.init(name: "Employees", bundle: nil).instantiateViewController(withIdentifier: "ApproveOrderVC") as? ApproveOrderVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Approve Order", isBackEnbaled: true)
    }

    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func btnSelectAction(_ sender:UIButton){
        self.view.endEditing(true)
        arrItems[sender.tag].isSelected = !arrItems[sender.tag].isSelected
        if let cell = tblOrders.cellForRow(at: IndexPath.init(row: sender.tag, section: 0)) as? OrderDetailViewCell{
            cell.btnSelect.isSelected = arrItems[sender.tag].isSelected
            if arrItems[sender.tag].isSelected{
                cell.txtQty.isUserInteractionEnabled = true
                cell.txtPrice.isUserInteractionEnabled = true
                cell.txtQty.text = "\(arrItems[sender.tag].approveQty ?? 0)"
                cell.txtPrice.text = String.init(format: "%.2f", arrItems[sender.tag].approvePrice ?? 0)
            }else{
                cell.txtQty.isUserInteractionEnabled = false
                cell.txtPrice.isUserInteractionEnabled = false
                cell.txtQty.text = ""
                cell.txtPrice.text = ""
            }
        }
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        if currentPage == 1{
            if (txtSelectDate.text?.trimmingCharacters(in: .whitespaces).isEmpty)!
            {
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_DATE, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }else if (txtReamark.text?.trimmingCharacters(in: .whitespaces).isEmpty)!
            {
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_ENTER_REMARK, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }else{
                if let VC = ApproveOrderVC.instance(){
                    VC.currentPage = 2
                    VC.strStoreDate = strStoreDate
                    VC.strRemark = txtReamark.text ?? ""
                    VC.orderDetail = orderDetail
                    self.navigationController?.pushViewController(VC, animated: true)
                }
            }
        }else{
            callApproveOrderAPI()
        }
    }
    
    
    func callApproveOrderAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "113"
        param["id"] = "\(orderDetail.id ?? 0)"
        param["order_type"] = "approve"
        param["customer_id"] = "\(orderDetail.cid ?? 0)"
        param["customer_branch_id"] = "\(orderDetail.cbid ?? 0)"
        param["sales_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["order_date"] = strStoreDate
        param["remarks"] = strRemark
        var arrOrder = [NSMutableDictionary]()
        for item in arrItems{
            if item.isSelected{
                if item.approveQty ?? 0 == 0{
                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: "Please enter quantity for \(item.itemName ?? "")", withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                        
                    }
                    return
                }
                if item.approvePrice ?? 0 == 0{
                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: "Please enter price for \(item.itemName ?? "")", withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                        
                    }
                    return
                }
                let dictOrder = NSMutableDictionary()
                dictOrder["order_item_id"] = "\(item.itemId ?? 0)"
                dictOrder["qty"] = "\(item.approveQty ?? 0)"
                dictOrder["price"] = "\(item.approvePrice ?? 0)"
                arrOrder.append(dictOrder)
            }
        }
        if arrOrder.count == 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_NO_APPROVE_ITEM, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
            return
        }
        param["item"] = AppCommonMethods.convertJsontoString(json: arrOrder)
        if AppConstants.isInDevlopmentMode{
            print("Order Approve Param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Order Status Data:",response!)
                    }
                    aMessage = response!.getStringValue("ack_msg") ?? ""
                    if response!.getIntValue("ack") ?? 0 == 1{
                        var settings = AlertSettings()
                        settings.alertMsg = "Do you want to print this order?"
                        settings.alertTitle = "Print Order"
                        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
                        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
                        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
                            VC.delegate = self
                        }
                        var setting = BottomAlertSettings()
                        setting.strMessage = aMessage
                        AppValidation.shared.showBottomAlertView(settings: setting)
                        //self.navigationController?.popToRootViewController(animated: true)
                        return
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
    
    func callGetOrderDetailAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "7"
        param["order_id"] = "\(orderDetail.id ?? 0)"
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCartURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Order Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode(OrderHistoryModel.self, from: data){
                                self.orderDetail = aData
                                self.strStoreDate = self.orderDetail.date ?? ""
                                self.txtSelectDate.text = self.strStoreDate.changeDateFormat(fromFormat: "dd-MM-yyyy", toFormat: "dd/MM/yyyy")
                                self.txtReamark.text = self.orderDetail.remarks ?? ""
                                self.lblRemarkCount.text = "\(self.txtReamark.text!.count)/300"
                                if self.txtReamark.text! == ""{
                                    self.lblPlaceholder.isHidden = false
                                }else{
                                    self.lblPlaceholder.isHidden = true
                                }
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension ApproveOrderVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrItems.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderDetailViewCell", for: indexPath) as! OrderDetailViewCell
        let model = arrItems[indexPath.row]
        cell.lblName.text = model.itemName ?? ""
        cell.btnSelect.tag = indexPath.row
        cell.lblPrice.setLabelAttributedText(title: "Sub Total", value: String.init(format: "%@%.2f", AppConstants.currencySign ,model.itemTotalPrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.lblQty.setLabelAttributedText(title: "Quantity", value: String.init(format: "%.1f", model.itemQty ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.txtQty.accessibilityIdentifier = "101"
        cell.txtPrice.accessibilityIdentifier = "102"
        cell.btnSelect.isSelected = model.isSelected
        if model.isSelected{
            cell.txtQty.isUserInteractionEnabled = true
            cell.txtPrice.isUserInteractionEnabled = true
            cell.txtQty.text = "\(model.approveQty ?? 0)"
            cell.txtPrice.text = String.init(format: "%.2f", model.approvePrice ?? 0)
        }else{
            cell.txtQty.isUserInteractionEnabled = false
            cell.txtPrice.isUserInteractionEnabled = false
            cell.txtQty.text = ""
            cell.txtPrice.text = ""
        }
        return cell
    }
}

extension ApproveOrderVC:UITextFieldDelegate{
    
    func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
        if textField == txtSelectDate{
            let date = strStoreDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
            let datePicker = ActionSheetDatePicker(title: "Select date", datePickerMode: UIDatePicker.Mode.date, selectedDate: date, doneBlock: {
                picker, value, index in
                self.strStoreDate = (value as! Date).getDateInString(format: "dd-MM-yyyy")
                self.txtSelectDate.text = (value as! Date).getDateInString(format: "dd/MM/yyyy")
            }, cancel: { ActionStringCancelBlock in return }, origin: textField)
            datePicker?.minimumDate = Date()
            datePicker?.show()
            return false
        }
        return true
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
    
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        if textField == txtSelectDate{
            return true
        }
        let allowedCharacters = CharacterSet(charactersIn:"0123456789.")//Here change this characters based on your requirement
        let characterSet = CharacterSet(charactersIn: string)
        if textField.accessibilityIdentifier == "102"{
            if allowedCharacters.isSuperset(of: characterSet){
                var strText = (textField.text ?? "").replacingOccurrences(of: ".", with: "")
                if string == ""{
                    if strText != "000"{
                        if strText.count == 3{
                            strText.removeLast()
                            strText = "0\(strText)"
                        }else{
                            strText.removeLast()
                        }
                        strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                        textField.text = "\(strText)"
                    }
                }else{
                    if strText.first! == "0"{
                        strText.removeFirst()
                        strText = "\(strText)\(string)"
                    }else{
                        strText = "\(strText)\(string)"
                    }
                    strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                    textField.text = "\(strText)"
                }
            }
            return false
        }
        return allowedCharacters.isSuperset(of: characterSet)
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        if textField != txtSelectDate{
            let item = arrItems[textField.tag]
            var qty = item.approveQty ?? 0
            var price = item.approvePrice ?? 0
            if textField.accessibilityIdentifier == "101"{
                qty = Double(textField.text ?? "") ?? 0
                if qty == 0{
                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_ITEM_NOT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                        
                    }
                    if let cell = tblOrders.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                        cell.txtQty.text = "\(item.approveQty ?? 0)"
                    }
                    return
                }else if qty > item.itemQty ?? 0{
                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.QTY_BELOW_SELECTED_QTY, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                        
                    }
                    if let cell = tblOrders.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                        cell.txtQty.text = "\(item.approveQty ?? 0)"
                    }
                    return
                }
            }else{
                price = Double(textField.text ?? "") ?? 0
                if price == 0{
                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_AMOUNT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                        
                    }
                    if let cell = tblOrders.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                        cell.txtPrice.text = String.init(format: "%.2f", item.approvePrice ?? 0)
                    }
                    return
                }
            }
            if let cell = tblOrders.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                item.itemTotalPrice = qty*price
                cell.lblPrice.setLabelAttributedText(title: "Sub Total", value: String.init(format: "%@%.2f", AppConstants.currencySign ,item.itemTotalPrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
                cell.txtQty.text = "\(qty)"
                cell.txtPrice.text = String.init(format: "%.2f", price)
                
            }
            item.approveQty = qty
            item.approvePrice = price
        }
    }
}

extension ApproveOrderVC:UITextViewDelegate{
    func textViewDidChange(_ textView: UITextView) {
        lblRemarkCount.text = "\(textView.text!.count)/300"
        if textView.text! == ""{
            lblPlaceholder.isHidden = false
        }else{
            lblPlaceholder.isHidden = true
        }
    }
    
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        if text == ""{
            return true
        }
        if textView.text.count >= 300{
            return false
        }
        return true
    }
}

extension ApproveOrderVC:alertClickedDelegate{
    func alertButtonClicked(index: Int,value:String) {
        if index == 1{
            PrintManager.shared.connectToPrinterWithPrint(model: orderDetail!)
        }
        self.navigationController?.popToRootViewController(animated: true)
    }
}
