//
//  ChecklistVC.swift
//  globalfoods
//
//  Created by Apple on 27/01/21.
//

import UIKit
import SDWebImage

class ChecklistCell: UITableViewCell {
    
    @IBOutlet weak var imgItem:UIImageView!
    @IBOutlet weak var lblDescription:UILabel!
    @IBOutlet weak var lblItemName:UILabel!
    
    override class func awakeFromNib() {
        
    }
}

class ChecklistVC: UIViewController {
    
    @IBOutlet weak var tblChecklist:UITableView!
    @IBOutlet weak var btnBack:UIButton!
    
    var arrChecklist = [ChecklistModel]()
    var strToDate = ""
    var strFromDate = ""
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        self.strFromDate = Date().addingTimeInterval(-7*24*60*60).getDateInString(format: "dd-MM-yyyy")
        self.strToDate = Date().getDateInString(format: "dd-MM-yyyy")
        btnBack.isSelected = isBackEnabled
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        callGetChecklist()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->ChecklistVC?{
        return UIStoryboard.init(name: "Employees", bundle: nil).instantiateViewController(withIdentifier: "ChecklistVC") as? ChecklistVC
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func btnMenuAction(_ sender:UIButton) {
        if isBackEnabled{
            self.navigationController?.popViewController(animated: true)
        }else{
            self.sideMenuController?.revealMenu()
        }
    }
    
    @IBAction func btnFilterAction(_ sender:UIButton) {
        if let VC = DateRangeVC.showPopup(parentVC: self, toDate: strToDate, fromDate: strFromDate){
            VC.delegate = self
        }
    }
    
    @IBAction func btnDownloadAction(_ sender:UIButton) {
        downloadChecklistPDFAPI()
    }
    
    func callGetChecklist(){
        //http://global.smartmps.co.uk/rms/service/service_employee.php?key=1226&s=78&driver=2&from_date=01-01-2021&to_date=31-01-2021
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "78"
        param["driver"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["from_date"] = strFromDate
        param["to_date"] = strToDate
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrChecklist.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Checklist Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([ChecklistModel].self, from: data){
                                self.arrChecklist = aData
                                self.tblChecklist.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblChecklist.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
    func downloadChecklistPDFAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "101"
        param["from_date"] = strFromDate
        param["to_date"] = strToDate
        param["driver"] = "\(AppConstants.employeeData?.id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Checklist Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        if let VC = WebViewController.instance(){
                            VC.strURL = response!.getStringValue("pdf_path") ?? ""
                            self.navigationController?.pushViewController(VC, animated: true)
                        }
                        return
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
}

extension ChecklistVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrChecklist.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "ChecklistCell", for: indexPath) as! ChecklistCell
        let model = arrChecklist[indexPath.row]
        cell.lblItemName.text = model.itemName ?? ""
        let strDesc = "SKU: \(model.itemCode ?? "")\nWeight: \(String.init(format: "%.1f %@", model.weight ?? 0,model.unitName ?? ""))\nPackage type: \(model.packagingType ?? "")\nQuantity: \(String.init(format: "%.1f", model.qty ?? 0))"
        let attrDesc = NSMutableAttributedString.init(string: strDesc, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
        attrDesc.addFontAttribute(title: "SKU", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrDesc.addFontAttribute(title: "Weight", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrDesc.addFontAttribute(title: "Package type", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrDesc.addFontAttribute(title: "Quantity", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.lblDescription.attributedText = attrDesc
        cell.imgItem.sd_setImage(with: URL(string: model.mediaURL ?? ""), placeholderImage:UIImage(named: "placeholder"), options: .avoidDecodeImage)
        return cell
    }
}

extension ChecklistVC:dateRangeDelegate{
    func userselectedDate(toDate: String, fromDate: String) {
        strToDate = toDate
        strFromDate = fromDate
        callGetChecklist()
    }
}
