//
//  DeliveriesVC.swift
//  globalfoods
//
//  Created by Apple on 21/01/21.
//

import UIKit

class DeliveriesVC: UIViewController {

    @IBOutlet weak var tblDisptaches:UITableView!
    
    var customHeaderView:HeaderView?
    var arrDispatches = [DispatchModel]()
    var arrSelectedDispatch = [DispatchModel]()
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    var totalArrCount = 0
    var isAPICalling = false
    let limit = 10
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
        callGetReadyToDispatchAPI(isFromLoad:true)
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->DeliveriesVC?{
        return UIStoryboard.init(name: "Employees", bundle: nil).instantiateViewController(withIdentifier: "DeliveriesVC") as? DeliveriesVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Deliveries", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func btnNextAction(_ sender:UIButton){
        if arrSelectedDispatch.count == 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg:  AppConstants.GlobalAlert.ALERT_SELECT_DISPATCH, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else{
            if let VC = VehicleStorageVC.instance(){
                VC.arrSelectedDelivery = arrSelectedDispatch
                self.navigationController?.pushViewController(VC, animated: true)
            }
        }
    }
    
    func callGetReadyToDispatchAPI(isFromLoad:Bool){
        if isAPICalling{
            return
        }
        isAPICalling = true
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "69"
        
        if isFromLoad{
            param["offset"] = "0,\(limit)"
        }else{
            param["offset"] = "\(arrDispatches.count),\(limit)"
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceOrderURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                if isFromLoad{
                    self.arrDispatches.removeAll()
                }
                
                AppCommonMethods.stopProgressBar()
                self.isAPICalling = false
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Dispatches Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        self.totalArrCount = response!.getIntValue("ack_msg") ?? 0
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([DispatchModel].self, from: data){
                                if isFromLoad{
                                    self.arrDispatches = aData
                                }else{
                                    self.arrDispatches.append(contentsOf: aData)
                                }
                                self.tblDisptaches.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblDisptaches.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension DeliveriesVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrDispatches.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "DispatchesCell", for: indexPath) as! DispatchesCell
        let model = arrDispatches[indexPath.row]
        cell.lblDispatchNo.text = "Dispatch No: #\(model.dispatchNo ?? "")"
        cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign, model.dispatchTotal ?? 0)
        let index = arrSelectedDispatch.firstIndex(where: { (model) -> Bool in
            return model.id ?? 0 == arrDispatches[indexPath.row].id ?? 0
        })
        if index != nil{
            cell.imgSelect.image = UIImage.init(named: "ic_check_selected")
        }else{
            cell.imgSelect.image = UIImage.init(named: "ic_check_unselected")
        }
        cell.imgSelect.imageColor = AppConstants.Colors.kAppMainThemeColor
        let strDesc = "Billing person name: \(model.billingName ?? "")\nPhone: \(model.billingPhone ?? "")\nAddress: \(model.billingAddress ?? "")\nCity: \(model.billingCity ?? "")\nPincode: \(model.billingPincode ?? "")"
        let attrDesc = NSMutableAttributedString.init(string: strDesc, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
        attrDesc.addFontAttribute(title: "City", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrDesc.addFontAttribute(title: "Billing person name", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrDesc.addFontAttribute(title: "Phone", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrDesc.addFontAttribute(title: "Address", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrDesc.addFontAttribute(title: "Pincode", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.lblDescription.attributedText = attrDesc
        
        cell.lblProducts.attributedText = (model.items ?? "").htmlToAttributedString(font: nil)
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let index = arrSelectedDispatch.firstIndex(where: { (model) -> Bool in
            return model.id ?? 0 == arrDispatches[indexPath.row].id ?? 0
        })
        if let cell = tableView.cellForRow(at: indexPath) as? DispatchesCell{
            if index != nil{
                cell.imgSelect.image = UIImage.init(named: "ic_check_unselected")
                arrSelectedDispatch.remove(at: index!)
            }else{
                cell.imgSelect.image = UIImage.init(named: "ic_check_selected")
                arrSelectedDispatch.append(arrDispatches[indexPath.row])
            }
            cell.imgSelect.imageColor = AppConstants.Colors.kAppMainThemeColor
        }
    }
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if tblDisptaches.contentOffset.y >= (tblDisptaches.contentSize.height - tblDisptaches.frame.size.height) {
            if self.arrDispatches.count < self.totalArrCount{
                self.callGetReadyToDispatchAPI(isFromLoad: false)
            }
        }
    }
}
