//
//  PaymentListVC.swift
//  globalfoods
//
//  Created by Apple on 25/01/21.
//

import UIKit

class PaymentViewCell: UITableViewCell {
    
    @IBOutlet weak var btnApprove:UIButton!
    @IBOutlet weak var btnReject:UIButton!
    @IBOutlet weak var lblDescription:UILabel!
    @IBOutlet weak var lblTotalPrice:UILabel!
    @IBOutlet weak var viewForActions:UIView!
    
    override class func awakeFromNib() {
        
    }
}

class PaymentListVC: UIViewController {
    
    @IBOutlet weak var tblPaymentList:UITableView!
    @IBOutlet weak var btnAddPayment:UIButton!
    
    var customHeaderView:HeaderView?
    var arrPayments = [PaymentModel]()
    var selectedRow = 0
    var strStatus = ""
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
        callGetPayments()
        if AppConstants.employeeData?.rights?.payment?.insert ?? 0 == 1{
            btnAddPayment.superview?.isHidden = false
        }else{
            btnAddPayment.superview?.isHidden = true
        }
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->PaymentListVC?{
        return UIStoryboard.init(name: "Employees", bundle: nil).instantiateViewController(withIdentifier: "PaymentListVC") as? PaymentListVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Payment", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func btnApproveAction(_ sender:UIButton){
        selectedRow = sender.tag
        strStatus = "approve"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to approve this payment?"
        settings.alertTitle = "Approve Payment"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    @IBAction func btnRejectAction(_ sender:UIButton){
        selectedRow = sender.tag
        strStatus = "reject"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to reject this payment?"
        settings.alertTitle = "Reject Payment"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    @IBAction func btnCreatePaymentAction(_ sender:UIButton){
        if let VC = CreatePaymentVC.instance(){
            self.navigationController?.pushViewController(VC, animated: true)
        }
    }
    
    func callUpdatePaymentStatus(){
        //http://global.smartmps.co.uk/rms/service/service_employee.php?key=1226&s=106&emp_id=2&cbid=1&payment_type=approve
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "106"
        param["emp_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["cbid"] = "\(arrPayments[selectedRow].cbid ?? 0)"
        param["payment_type"] = strStatus
        param["payment_id"] = "\(arrPayments[selectedRow].id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Payment Status Data:",response!)
                    }
                    aMessage = response!.getStringValue("ack_msg") ?? ""
                    if response!.getIntValue("ack") ?? 0 == 1{
                        var setting = BottomAlertSettings()
                        setting.strMessage = aMessage
                        AppValidation.shared.showBottomAlertView(settings: setting)
                        self.callGetPayments()
                        return
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in

                }
            }

        }
    }
    
    func callGetPayments(){
        //http://global.smartmps.co.uk/rms/service/service_employee.php?key=1226&s=47&emp_id=2
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "47"
        param["emp_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrPayments.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Payment Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([PaymentModel].self, from: data){
                                self.arrPayments = aData
                                self.tblPaymentList.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblPaymentList.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension PaymentListVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrPayments.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "PaymentViewCell", for: indexPath) as! PaymentViewCell
        let model = arrPayments[indexPath.row]
        cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign, model.credit ?? 0)
        if let date = (model.date ?? "").getDateFromString(format: "dd-MM-yyyy"){
            let strDesc = "Customer name: \(model.branchName ?? "")\nAddress: \(model.address ?? "")\nPincode: \(model.pincode ?? "")\nStatus: \(model.statusSlug ?? "")\nDate: \(date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy"))"
            let attrDesc = NSMutableAttributedString.init(string: strDesc, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
            attrDesc.addFontAttribute(title: "Customer name", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Address", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Pincode", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Status", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Date", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            cell.lblDescription.attributedText = attrDesc
        }
        if model.status ?? 0 == 0{
            cell.viewForActions.isHidden = false
            cell.btnReject.tag = indexPath.row
            cell.btnApprove.tag = indexPath.row
        }else{
            cell.viewForActions.isHidden = true
        }
        return cell
    }
}

extension PaymentListVC:alertClickedDelegate{
    func alertButtonClicked(index: Int,value:String) {
        if index == 1{
            callUpdatePaymentStatus()
        }
    }
}
