//
//  SalesReportVC.swift
//  globalfoods
//
//  Created by Tushar Premal on 07/10/21.
//

import UIKit
import ActionSheetPicker_3_0

class SalesReportVC: UIViewController {

    @IBOutlet weak var lblNoOrder:UILabel!
    @IBOutlet weak var lblValueOrder:UILabel!
    @IBOutlet weak var lblNoCashOrder:UILabel!
    @IBOutlet weak var lblValueCashOrder:UILabel!
    @IBOutlet weak var lblNoCardOrder:UILabel!
    @IBOutlet weak var lblValueChequeOrder:UILabel!
    @IBOutlet weak var lblNoChequeOrder:UILabel!
    @IBOutlet weak var lblTotalRefundedAmount:UILabel!
    @IBOutlet weak var btnPrint:UIButton!
    @IBOutlet weak var lblValueCardOrder:UILabel!
    @IBOutlet weak var lblTotalItems:UILabel!
    @IBOutlet weak var lblFromDate:UILabel!
    @IBOutlet weak var lblToDate:UILabel!
    
    var customHeaderView:HeaderView?
    var strToDate = ""
    var strFromDate = ""
    var popRecognizer: InteractivePopRecognizer?
    var isBackEnabled = false
    var report:FullReportModel!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        self.strFromDate = Date().getDateInString(format: "dd-MM-yyyy")
        self.strToDate = Date().getDateInString(format: "dd-MM-yyyy")
        self.lblFromDate.text = "From - \(Date().getDateInString(format: "dd/MM/yyyy"))"
        self.lblToDate.text = "To - \(Date().getDateInString(format: "dd/MM/yyyy"))"
        if isBackEnabled{
            setInteractiveRecognizer()
        }
        if AppConstants.printerIP ?? "" == ""{
            btnPrint.isHidden = true
        }
        self.callGetReportAPI()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->SalesReportVC?{
        return UIStoryboard.init(name: "Employees", bundle: nil).instantiateViewController(withIdentifier: "SalesReportVC") as? SalesReportVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Sales Report", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    //MARK: Button Actions
    @IBAction func btnFromDateAction(_ sender:UIButton){
        let date = strFromDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        let datePicker = ActionSheetDatePicker(title: "Select from date", datePickerMode: UIDatePicker.Mode.date, selectedDate: date, doneBlock: {
            picker, value, index in
            self.strFromDate = (value as! Date).getDateInString(format: "dd-MM-yyyy")
            self.lblFromDate.text = "From - \((value as! Date).getDateInString(format: "dd/MM/yyyy"))"
            self.callGetReportAPI()
        }, cancel: { ActionStringCancelBlock in return }, origin: sender)
        datePicker?.maximumDate = strToDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        datePicker?.show()
    }
    
    @IBAction func btnToDateAction(_ sender:UIButton){
        let date = strToDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        let datePicker = ActionSheetDatePicker(title: "Select to date", datePickerMode: UIDatePicker.Mode.date, selectedDate: date, doneBlock: {
            picker, value, index in
            self.strToDate = (value as! Date).getDateInString(format: "dd-MM-yyyy")
            self.lblToDate.text = "To - \((value as! Date).getDateInString(format: "dd/MM/yyyy"))"
            self.callGetReportAPI()
        }, cancel: { ActionStringCancelBlock in return }, origin: sender)
        if let fromDate = strFromDate.getDateFromString(format: "dd-MM-yyyy", locale: nil){
            datePicker?.minimumDate = fromDate
        }
        datePicker?.show()
    }
    
    @IBAction func btnPrintReportAction(_ sender:UIButton){
        if report != nil{
            PrintManager.shared.fromDate = strFromDate
            PrintManager.shared.toDate = strToDate
            PrintManager.shared.connectToPrinterWithPrint(model: report!)
        }
    }
    
    func callGetReportAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["emp_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["from_date"] = strFromDate
        param["to_date"] = strToDate
        param["s"] = "129"
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var aMSG = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Report Data:",response!)
                    }
                    aMSG = response!.getStringValue("ack_msg") ?? ""
                    let jsonDecoder = JSONDecoder()
                    if response!.getIntValue("ack") ?? 0 == 1,let result = response!["report"] as? NSDictionary, let data = AppCommonMethods.convertToJson(object: result), let aData = try? jsonDecoder.decode(FullReportModel.self, from: data){
                        self.report = aData
                        self.lblNoCashOrder.text = "\(aData.totalCodOrderCount ?? 0)"
                        self.lblNoChequeOrder.text = "\(aData.totalChequeOrderCount ?? 0)"
                        self.lblNoCardOrder.text = "\(aData.totalCardOrderCount ?? 0)"
                        self.lblNoOrder.text = "\(aData.totalOrderCount ?? 0)"
                        self.lblValueOrder.text = String.init(format: "%@ %.2f", AppConstants.currencySign, aData.orderValue ?? 0)
                        self.lblTotalItems.text = String.init(format: "%.1f", aData.totalItemCount ?? 0)
                        self.lblValueCashOrder.text = String.init(format: "%@ %.2f", AppConstants.currencySign, aData.codOrderValue ?? 0)
                        self.lblValueChequeOrder.text = String.init(format: "%@ %.2f", AppConstants.currencySign, aData.chequeOrderValue ?? 0)
                        self.lblValueCardOrder.text = String.init(format: "%@ %.2f", AppConstants.currencySign,aData.cardOrderValue ?? 0)
                        self.lblTotalRefundedAmount.text = String.init(format: "%@ %.2f", AppConstants.currencySign,aData.totalRefundAmount ?? 0)
                        return
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMSG, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
}
