//
//  ApproveReturnVC.swift
//  globalfoods
//
//  Created by Apple on 28/01/21.
//

import UIKit
import DropDown

protocol ReturnStatusDelegate: AnyObject {
    func returnApproved()
}

class ApproveReturnVC: UIViewController {

    @IBOutlet weak var txtSelectStore:UITextField!
    @IBOutlet weak var tblReturnOrder:UITableView!
    
    var returnOrderDetail:ReturnOrderModel!
    var arrItems = [ReturnItemModel]()
    var arrStores = [StoreModel]()
    var storeData:StoreModel?
    var arrSearchStores = [StoreModel]()
    let storeDropDown = DropDown()
    var isStoreSearchEnabled = false
    weak var delegate:ReturnStatusDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        callGetReturnOrderDetailAPI()
        callGetStoreAPI()
    }
    
    //MARK: create viewcontroller instance
    static func instance(order:ReturnOrderModel)->SheetViewController?{
        guard let controller = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "ApproveReturnVC") as? ApproveReturnVC else { return nil }
        controller.returnOrderDetail = order
        let height:CGFloat = UIScreen.main.bounds.size.height - (AppConstants.ScreenSize.topPadding + 100)
        
        return SheetViewController(controller: controller, sizes: [.fixed(height)], options: SheetOptions(useInlineMode: false))
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        if storeData == nil{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_STORE, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else{
            callApproveReturnAPI()
        }
    }
    
    @IBAction func txtSelectStoreAction(_ sender:UITextField){
        if sender.text ?? "" == ""{
            isStoreSearchEnabled = false
        }else{
            isStoreSearchEnabled = true
            arrSearchStores = arrStores.filter({ (model) -> Bool in
                return (model.name ?? "").lowercased().contains((sender.text ?? "").lowercased())
            })
            
        }
        var arrName = [String]()
        for model in (isStoreSearchEnabled ? arrSearchStores : arrStores){
            arrName.append(model.name ?? "")
        }
        storeDropDown.dataSource = arrName
        if storeDropDown.isDropdownHidden{
            storeDropDown.show()
        }else{
            storeDropDown.reloadAllComponents()
        }
    }
    
    func callGetStoreAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "67"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceOrderURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrStores.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("store Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([StoreModel].self, from: data){
                                self.arrStores = aData
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }

    func callApproveReturnAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "110"
        param["return_type"] = "approve"
        param["store_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["return_id"] = "\(returnOrderDetail.id ?? 0)"
        var arrReturn = [NSMutableDictionary]()
        for item in arrItems{
            if item.approveQty ?? 0 == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: "Please enter quantity for \(item.itemName ?? "")", withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                return
            }
            let dictReturn = NSMutableDictionary()
            dictReturn["return_item_id"] = "\(item.id ?? 0)"
            dictReturn["qty"] = "\(item.approveQty ?? 0)"
            dictReturn["price"] = "\(item.returnPrice ?? 0)"
            arrReturn.append(dictReturn)
        }
        param["item"] = AppCommonMethods.convertJsontoString(json: arrReturn)
        if AppConstants.isInDevlopmentMode{
            print("Return Approve Param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Return Status Data:",response!)
                    }
                    aMessage = response!.getStringValue("ack_msg") ?? ""
                    if response!.getIntValue("ack") ?? 0 == 1{
                        var setting = BottomAlertSettings()
                        setting.strMessage = aMessage
                        AppValidation.shared.showBottomAlertView(settings: setting)
                        if self.delegate != nil{
                            self.delegate?.returnApproved()
                        }
                        self.dismiss(animated: true, completion: nil)
                        return
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
    
    func callGetReturnOrderDetailAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "52"
        param["rtid"] = "\(returnOrderDetail.id ?? 0)"
        param["vehical_id"] = "\(returnOrderDetail.vehicleId ?? 0)"
        param["eid"] = "\(returnOrderDetail.eid ?? 0)"
        AppCommonMethods.startProgressBar()
        if AppConstants.isInDevlopmentMode{
            print("Return Order Param:",param)
        }
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Return Order Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode(ReturnOrderModel.self, from: data){
                                self.returnOrderDetail = aData
                                self.arrItems = aData.arrItems ?? [ReturnItemModel]()
                                self.arrItems.forEach({ (model) in
                                    model.approveQty = model.qty
                                })
                                self.tblReturnOrder.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
}

extension ApproveReturnVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrItems.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderDetailViewCell", for: indexPath) as! OrderDetailViewCell
        let model = arrItems[indexPath.row]
        cell.lblName.text = model.itemName ?? ""
        cell.txtQty.text = "\(model.approveQty ?? 0)"
        
        cell.lblQty.setLabelAttributedText(title: "Quantity", value: String.init(format: "%.1f", model.qty ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.lblPrice.setLabelAttributedText(title: "Sub Total", value: String.init(format: "%@%.2f", AppConstants.currencySign ,model.returnSubTotal ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        return cell
    }
}

extension ApproveReturnVC:UITextFieldDelegate{
    func textFieldDidBeginEditing(_ textField: UITextField) {
        if textField == txtSelectStore{
            isStoreSearchEnabled = false
            txtSelectStore.text = ""
            storeDropDown.dismissMode = .manual
            storeDropDown.anchorView = txtSelectStore
            storeDropDown.bottomOffset = CGPoint(x: 0, y: txtSelectStore.bounds.height)
            storeDropDown.topOffset = CGPoint(x: 0, y: -txtSelectStore.bounds.height)
            var arrName = [String]()
            for model in (isStoreSearchEnabled ? arrSearchStores : arrStores){
                arrName.append(model.name ?? "")
            }
            storeDropDown.dataSource = arrName
            // Action triggered on selection
            storeDropDown.selectionAction = { [weak self] (index, item) in
                self?.storeData = ((self?.isStoreSearchEnabled ?? false) ? self?.arrSearchStores : self?.arrStores)?[index]
                self?.txtSelectStore.text = item
                self?.view.endEditing(true)
            }
            storeDropDown.show()
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
    
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        if textField == txtSelectStore{
            return true
        }
        let allowedCharacters = CharacterSet(charactersIn:"0123456789.")//Here change this characters based on your requirement
        let characterSet = CharacterSet(charactersIn: string)
        return allowedCharacters.isSuperset(of: characterSet)
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        if textField == txtSelectStore{
            txtSelectStore.text = storeData?.name ?? ""
        }else{
            let item = arrItems[textField.tag]
            let qty = Double(textField.text ?? "") ?? 0
            if qty == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_ITEM_NOT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblReturnOrder.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                    cell.txtQty.text = "\(item.approveQty ?? 0)"
                }
                return
            }else if qty > item.qty ?? 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.QTY_BELOW_RETURN_QTY, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblReturnOrder.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                    cell.txtQty.text = "\(item.approveQty ?? 0)"
                }
                return
            }
            
            if let cell = tblReturnOrder.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                item.returnSubTotal = qty*(item.returnPrice ?? 0)
                cell.lblPrice.setLabelAttributedText(title: "Sub Total", value: String.init(format: "%@%.2f", AppConstants.currencySign ,item.returnSubTotal ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
                cell.txtQty.text = "\(qty)"
            }
            item.approveQty = qty
        }
    }
}
