//
//  CreateReturnOrderVC.swift
//  globalfoods
//
//  Created by Apple on 08/01/21.
//

import UIKit

class CreateReturnOrderVC: UIViewController {

    @IBOutlet weak var tblOrders:UITableView!
    @IBOutlet weak var lblReturnAmount:UILabel!
    @IBOutlet weak var btnConfirm:UIButton!
    
    var selectedOrder:OrderHistoryModel!
    var arrItems = [CartItemsModel]()
    var totalAmount = 0.0
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        arrItems = selectedOrder.arrItems ?? [CartItemsModel]()
        getTotalReturnPrice()
    }
    
    //MARK: create viewcontroller instance
    static func instance(order:OrderHistoryModel!)->SheetViewController?{
        guard let controller = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "CreateReturnOrderVC") as? CreateReturnOrderVC else { return nil }
        controller.selectedOrder = order
        let height:CGFloat = UIScreen.main.bounds.size.height - (AppConstants.ScreenSize.topPadding + 100)
        
        return SheetViewController(controller: controller, sizes: [.fixed(height)], options: SheetOptions(useInlineMode: false))
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        callReturnOrderAPI()
    }
    
    @IBAction func btnSelectAction(_ sender:UIButton){
        self.view.endEditing(true)
        let item = arrItems[sender.tag]
        if item.isSelected{
            item.isSelected = false
            item.returnQty = nil
        }else{
            item.isSelected = true
            item.returnQty = 0.5
        }
        getTotalReturnPrice()
        if let cell = tblOrders.cellForRow(at: IndexPath.init(row: sender.tag, section: 0)) as? OrderDetailViewCell{
            if item.returnQty != nil{
                cell.txtQty.text = "\(item.returnQty ?? 0)"
            }else{
                cell.txtQty.text = ""
            }
            if item.isSelected{
                cell.txtQty.isUserInteractionEnabled = true
                cell.imgSelect.image = UIImage.init(named: "ic_radio_selected")
            }else{
                cell.imgSelect.image = UIImage.init(named: "ic_radio_unselected")
                cell.txtQty.isUserInteractionEnabled = false
            }
            cell.imgSelect.imageColor = AppConstants.Colors.kAppMainThemeColor
        }
    }
    
    func getTotalReturnPrice(){
        totalAmount = 0
        for item in arrItems{
            if item.isSelected{
                totalAmount += ((item.itemPrice ?? 0) * (item.returnQty ?? 0))
            }
        }
        if totalAmount == 0{
            btnConfirm.isUserInteractionEnabled = false
            self.lblReturnAmount.superview?.backgroundColor = AppConstants.Colors.kAppLightGrayColor
        }else{
            btnConfirm.isUserInteractionEnabled = true
            self.lblReturnAmount.superview?.backgroundColor = AppConstants.Colors.kAppMainThemeColor
        }
        self.lblReturnAmount.text = ""
        if let data = AppConstants.userData{
            if data.showPrice ?? 0 == 1{
                self.lblReturnAmount.text = String.init(format: "%@%.2f", AppConstants.currencySign ,totalAmount)
            }
        }else{
            self.lblReturnAmount.text = String.init(format: "%@%.2f", AppConstants.currencySign ,totalAmount)
        }
    }
    
    func callReturnOrderAPI(){
        var arrReturn = [NSMutableDictionary]()
        for item in arrItems{
            if item.isSelected{
                if item.returnQty ?? 0 == 0{
                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: "Please enter quantity for \(item.itemName ?? "")", withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                        
                    }
                    return
                }
                let dictReturn = NSMutableDictionary()
                dictReturn["pid"] = "\(item.itemId ?? 0)"
                dictReturn["qty"] = "\(item.returnQty ?? 0)"
                arrReturn.append(dictReturn)
            }
        }
        if arrReturn.count == 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_NO_RETURN_ITEM, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else{
            var param = [String:String]()
            param["key"] = AppConstants.commonConstant.kAPIKey
            param["s"] = "50"
            if AppConstants.userData == nil{
                param["eid"] = "\(AppConstants.employeeData?.id ?? 0)"
                param["cuid"] = "\(selectedOrder.cid ?? 0)"
                param["cbid"] = "\(selectedOrder.arrItems?.first?.cbid ?? 0)"
            }else{
                param["eid"] = "0"
                param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
                param["cbid"] = "\(AppConstants.selectedBranch?.id ?? 0)"
            }
            
            param["items"] = AppCommonMethods.convertJsontoString(json: arrReturn) 
            AppCommonMethods.startProgressBar()
            if AppConstants.isInDevlopmentMode{
                print("Return Order Param:",param)
            }
            WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
                DispatchQueue.main.async {
                    AppCommonMethods.stopProgressBar()
                    var setting = BottomAlertSettings()
                    setting.strMessage = message
                    if response != nil{
                        if AppConstants.isInDevlopmentMode{
                            print("Return Order Data:",response!)
                        }
                        if response!.getIntValue("ack") ?? 0 == 1{
                            self.dismiss(animated: true, completion: nil)
                        }
                        setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                    AppValidation.shared.showBottomAlertView(settings: setting)
                }
            }
        }
    }
}

extension CreateReturnOrderVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrItems.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderDetailViewCell", for: indexPath) as! OrderDetailViewCell
        let model = arrItems[indexPath.row]
        cell.lblName.text = model.itemName ?? ""
        if model.returnQty != nil{
            cell.txtQty.text = "\(model.returnQty ?? 0)"
        }else{
            cell.txtQty.text = ""
        }
        if model.isSelected{
            cell.txtQty.isUserInteractionEnabled = true
            cell.imgSelect.image = UIImage.init(named: "ic_radio_selected")
        }else{
            cell.imgSelect.image = UIImage.init(named: "ic_radio_unselected")
            cell.txtQty.isUserInteractionEnabled = false
        }
        cell.btnSelect.tag = indexPath.row
        cell.txtQty.tag = indexPath.row
        cell.imgSelect.imageColor = AppConstants.Colors.kAppMainThemeColor
        cell.lblPrice.text = ""
        if let data = AppConstants.userData{
            if data.showPrice ?? 0 == 1{
                cell.lblPrice.setLabelAttributedText(title: "Item price", value: String.init(format: "%@%.2f", AppConstants.currencySign, model.itemPrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            }
        }else{
            cell.lblPrice.setLabelAttributedText(title: "Item price", value: String.init(format: "%@%.2f", AppConstants.currencySign, model.itemPrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        }
        
        cell.lblQty.setLabelAttributedText(title: "Purchased quantity", value: String.init(format: "%.1f", model.itemQty ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        return cell
    }
}

extension CreateReturnOrderVC:UITextFieldDelegate{
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        let allowedCharacters = CharacterSet(charactersIn:"0123456789.")//Here change this characters based on your requirement
        let characterSet = CharacterSet(charactersIn: string)
        return allowedCharacters.isSuperset(of: characterSet)
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        let item = arrItems[textField.tag]
        let qty = Double(textField.text ?? "") ?? 0
        if qty == 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_ITEM_NOT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
            if let cell = tblOrders.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                if item.returnQty != nil{
                    cell.txtQty.text = "\(item.returnQty ?? 0)"
                }else{
                    cell.txtQty.text = ""
                }
            }
            return
        }else if qty > item.itemQty ?? 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.QTY_BELOW_SELECTED_QTY, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
            if let cell = tblOrders.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                if item.returnQty != nil{
                    cell.txtQty.text = "\(item.returnQty ?? 0)"
                }else{
                    cell.txtQty.text = ""
                }
            }
            return
        }
        
        if let cell = tblOrders.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
            cell.txtQty.text = "\(qty)"
        }
        item.returnQty = qty
        
        getTotalReturnPrice()
    }
}
