//
//  CustomeAlertVC.swift
//  globalfoods
//
//  Created by Apple on 05/01/21.
//

import UIKit

protocol alertClickedDelegate: AnyObject {
    func alertButtonClicked(index: Int,value:String)
}

struct AlertSettings {
    var alertTitle = ""
    var alertMsg = ""
    var alertPlaceholder = ""
    var alertTextValue = ""
    var confirmButtonTitle = "Confirm"
    var cancelButtonTitle = "Cancel"
    
    var confirmButtonFirstColor = AppConstants.Colors.kAppMainThemeColor
    var cancelButtonFirstColor = AppConstants.Colors.kAppLightRedColor
    var confirmButtonSecondColor = AppConstants.Colors.kAppDarkThemeColor
    var cancelButtonSecondColor = AppConstants.Colors.kAppRedColor
}

class CustomeAlertVC: UIViewController {

    @IBOutlet weak var lblTitle:UILabel!
    @IBOutlet weak var lblMessage:UILabel!
    @IBOutlet weak var txtAlertField:UITextField!
    @IBOutlet weak var btnConfirm:UIButtonX!
    @IBOutlet weak var btnCancel:UIButtonX!
    @IBOutlet weak var constViewYCenter:NSLayoutConstraint!
    
    var settings:AlertSettings!
    weak var delegate:alertClickedDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        lblTitle.text = settings.alertTitle
        lblMessage.text = settings.alertMsg
        txtAlertField.placeholder = "Enter \(settings.alertPlaceholder)..."
        txtAlertField.text = settings.alertTextValue
        btnConfirm.firstColor = settings.confirmButtonFirstColor
        btnConfirm.secondColor = settings.confirmButtonSecondColor
        btnCancel.firstColor = settings.cancelButtonFirstColor
        btnCancel.secondColor = settings.cancelButtonSecondColor
        btnCancel.setTitle(settings.cancelButtonTitle, for: .normal)
        btnConfirm.setTitle(settings.confirmButtonTitle, for: .normal)
        if settings.alertPlaceholder != ""{
            txtAlertField.superview?.isHidden = false
            txtAlertField.delegate = self
            txtAlertField.keyboardType = .decimalPad
        }
        if settings.cancelButtonTitle == ""{
            btnCancel.isHidden = true
        }
        if settings.alertMsg != ""{
            lblMessage.isHidden = false
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillShow(_:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillHide(_:)), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self)
    }
    
    //MARK: create viewcontroller instance
    class func showPopup(parentVC:UIViewController,settings:AlertSettings)->CustomeAlertVC?{
        if let container = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "CustomeAlertVC") as? CustomeAlertVC{
            container.settings = settings
            parentVC.addChild(container)
            parentVC.view.addSubview(container.view)
            container.view.frame = CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.size.width, height: UIScreen.main.bounds.size.height)
            container.view.alpha = 0.0
            UIView.animate(withDuration: 0.25) {
                container.view.alpha = 1.0
            }
            return container
        }
        return nil
    }
    
    func closePopup(){
        UIView.animate(withDuration: 0.25) {
            self.view.alpha = 0.0
        } completion: { (status) in
            self.view.removeFromSuperview()
            self.removeFromParent()
        }
    }
    
    @objc func keyboardWillShow(_ notification: Notification) {
        constViewYCenter.constant = 200
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @objc func keyboardWillHide(_ notification: Notification) {
        constViewYCenter.constant = 0
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        if settings.alertPlaceholder != ""{
            if (txtAlertField.text ?? "").trimmingCharacters(in: .whitespacesAndNewlines).isEmpty{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: "Please enter \(settings.alertPlaceholder)", withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    _ = self.txtAlertField.becomeFirstResponder()
                }
                return
            }
        }
        self.closePopup()
        if delegate != nil{
            self.delegate?.alertButtonClicked(index: sender.tag, value: txtAlertField.text ?? "")
        }
    }
    
    @IBAction func btnCancelAction(_ sender:UIButton){
        self.closePopup()
        if delegate != nil{
            self.delegate?.alertButtonClicked(index: sender.tag, value: "")
        }
    }

}

extension CustomeAlertVC: UITextFieldDelegate {
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        let allowedCharacters = CharacterSet(charactersIn:"0123456789.")//Here change this characters based on your requirement
        let characterSet = CharacterSet(charactersIn: string)
        return allowedCharacters.isSuperset(of: characterSet)
    }
}
