//
//  ModifyProductsVC.swift
//  globalfoods
//
//  Created by Apple on 01/02/21.
//

import UIKit

protocol ModifyProductsDelegate: AnyObject {
    func productUpdated(arrSelecteditems:[InvoiceItemModel])
}

class ModifyProductsVC: UIViewController {
    
    @IBOutlet weak var tblItems:UITableView!
    @IBOutlet weak var txtSearch:UITextField!
    
    var arrItems = [ProductsModel]()
    var arrSearchItems = [ProductsModel]()
    weak var delegate:ModifyProductsDelegate?
    var isSearchEnabled = false
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
    }

    @IBAction func txtSearchChanged(_ sender:UITextField){
        if sender.text == ""{
            isSearchEnabled = false
        }else{
            isSearchEnabled = true
            arrSearchItems = arrItems.filter({ (model) -> Bool in
                return (model.itemName ?? "").lowercased().contains(self.txtSearch.text!.lowercased())
            })
        }
        tblItems.reloadData()
    }
    
    @IBAction func btnSelectAction(_ sender:UIButton){
        self.view.endEditing(true)
        let item = isSearchEnabled ? arrSearchItems[sender.tag] : arrItems[sender.tag]
        item.isSelected = !item.isSelected
        if isSearchEnabled{
            let index = arrItems.firstIndex { (model) -> Bool in
                return item.id ?? 0 == model.id ?? 0
            }
            arrItems[index!].isSelected = !item.isSelected
        }
        
        if let cell = tblItems.cellForRow(at: IndexPath.init(row: sender.tag, section: 0)) as? OrderDetailViewCell{
            cell.btnSelect.isSelected = item.isSelected
            if item.isSelected{
                cell.txtQty.isUserInteractionEnabled = true
                cell.txtPrice.isUserInteractionEnabled = true
            }else{
                cell.txtQty.isUserInteractionEnabled = false
                cell.txtPrice.isUserInteractionEnabled = false
            }
        }
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        var arrSelectedItem = [InvoiceItemModel]()
        let arr = arrItems.filter({ (model) -> Bool in
            return model.isSelected
        })
        
        if arr.count == 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_NO_ITEM, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
            return
        }
        for item in arr{
            if item.itemPrice ?? 0 == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: "Please enter price for \(item.itemName ?? "")", withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                return
            }
            if item.selectedQty == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: "Please enter quantity for \(item.itemName ?? "")", withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                return
            }
            let invoiceItem = InvoiceItemModel()
            invoiceItem.itemName = item.itemName
            invoiceItem.itemId = item.id
            invoiceItem.itemCode = item.itemCode
            invoiceItem.price = item.itemPrice
            invoiceItem.qty = item.selectedQty
            invoiceItem.subTotal = (item.itemPrice ?? 0) * (item.selectedQty)
            arrSelectedItem.append(invoiceItem)
        }
        if self.delegate != nil{
            self.delegate?.productUpdated(arrSelecteditems: arrSelectedItem)
        }
        self.dismiss(animated: true, completion: nil)
    }
    
    //MARK: create viewcontroller instance
    static func instance(items:[ProductsModel])->SheetViewController?{
        guard let controller = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "ModifyProductsVC") as? ModifyProductsVC else { return nil }
        controller.arrItems = items
        let height:CGFloat = UIScreen.main.bounds.size.height - (AppConstants.ScreenSize.topPadding + 100)
        
        return SheetViewController(controller: controller, sizes: [.fixed(height)], options: SheetOptions(useInlineMode: false))
    }
}

extension ModifyProductsVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return isSearchEnabled ? arrSearchItems.count : arrItems.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderDetailViewCell", for: indexPath) as! OrderDetailViewCell
        let model = isSearchEnabled ? arrSearchItems[indexPath.row] :arrItems[indexPath.row]
        cell.lblName.text = model.itemName ?? ""
        cell.btnSelect.tag = indexPath.row
        cell.lblPrice.setLabelAttributedText(title: "Sub Total", value: String.init(format: "%@%.2f", AppConstants.currencySign ,model.itemTotalPrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.lblQty.setLabelAttributedText(title: "Product Code", value: model.itemCode ?? "", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.txtQty.accessibilityIdentifier = "101"
        cell.txtPrice.accessibilityIdentifier = "102"
        cell.btnSelect.isSelected = model.isSelected
        cell.txtQty.text = "\(model.selectedQty)"
        cell.txtPrice.text = String.init(format: "%.2f", model.itemPrice ?? 0)
        cell.txtQty.tag = indexPath.row
        cell.txtPrice.tag = indexPath.row
        if model.isSelected{
            cell.txtQty.isUserInteractionEnabled = true
            cell.txtPrice.isUserInteractionEnabled = true
        }else{
            cell.txtQty.isUserInteractionEnabled = false
            cell.txtPrice.isUserInteractionEnabled = false
        }
        return cell
    }
}

extension ModifyProductsVC:UITextFieldDelegate{
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
    
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        let allowedCharacters = CharacterSet(charactersIn:"0123456789.")//Here change this characters based on your requirement
        let characterSet = CharacterSet(charactersIn: string)
        if textField.accessibilityIdentifier == "102"{
            if allowedCharacters.isSuperset(of: characterSet){
                var strText = (textField.text ?? "").replacingOccurrences(of: ".", with: "")
                if string == ""{
                    if strText != "000"{
                        if strText.count == 3{
                            strText.removeLast()
                            strText = "0\(strText)"
                        }else{
                            strText.removeLast()
                        }
                        strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                        textField.text = "\(strText)"
                    }
                }else{
                    if strText.first! == "0"{
                        strText.removeFirst()
                        strText = "\(strText)\(string)"
                    }else{
                        strText = "\(strText)\(string)"
                    }
                    strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                    textField.text = "\(strText)"
                }
            }
            return false
        }
        return allowedCharacters.isSuperset(of: characterSet)
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        let item = isSearchEnabled ? arrSearchItems[textField.tag] : arrItems[textField.tag]
        var qty = item.selectedQty
        var price = item.itemPrice ?? 0
        if textField.accessibilityIdentifier == "101"{
            qty = Double(textField.text ?? "") ?? 0
            if qty == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_ITEM_NOT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblItems.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                    cell.txtQty.text = "\(item.selectedQty)"
                }
                return
            }else if qty > item.stockQty ?? 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: String.init(format: "Selected item has %.1f quantity in stock, Please enter below stock quantity to proceed", item.stockQty ?? 0), withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblItems.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                    cell.txtQty.text = "\(item.selectedQty)"
                }
                return
            }
        }else{
            price = Double(textField.text ?? "") ?? 0
            if price == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_AMOUNT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblItems.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                    cell.txtPrice.text = String.init(format: "%.2f", item.itemPrice ?? 0)
                }
                return
            }
        }
        if let cell = tblItems.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
            cell.lblPrice.setLabelAttributedText(title: "Sub Total", value: String.init(format: "%@%.2f", AppConstants.currencySign ,item.itemTotalPrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            cell.txtQty.text = "\(qty)"
            cell.txtPrice.text = String.init(format: "%.2f", price)
        }
        item.selectedQty = qty
        item.itemPrice = price
        item.itemTotalPrice = qty*price
        if isSearchEnabled{
            let index = arrItems.firstIndex { (model) -> Bool in
                return item.id ?? 0 == model.id ?? 0
            }
            arrItems[index!].selectedQty = qty
            arrItems[index!].itemPrice = price
            arrItems[index!].itemTotalPrice = qty*price
        }
        
    }
}
