//
//  PaymentMethodVC.swift
//  globalfoods
//
//  Created by Apple on 17/11/21.
//

import UIKit

class PaymentMethodVC: UIViewController {

    @IBOutlet weak var imgCard:UIImageViewX!
    @IBOutlet weak var imgCash:UIImageViewX!
    @IBOutlet weak var imgCheque:UIImageViewX!
    @IBOutlet weak var txtAmount:UITextField!
    @IBOutlet weak var txtChequeNo:UITextField!
    @IBOutlet weak var lblTotalAmount:UILabel!
    @IBOutlet weak var lblChange:UILabel!
    @IBOutlet weak var constViewYCenter:NSLayoutConstraint!
    
    var parentVC:UIViewController!
    var paymentMethod = "cod"
    var totalAmount = 0.0
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        setPaymentModeView()
        if AppConstants.DeviceType.IS_IPAD{
            txtAmount.isUserInteractionEnabled = false
        }
        lblTotalAmount.text = String.init(format: "Total: %@%.2f", AppConstants.currencySign, totalAmount)
        txtAmount.text = String.init(format: "%.2f", 0)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillShow(_:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillHide(_:)), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self)
    }

    //MARK: create viewcontroller instance
    class func showPopup(parentVC:UIViewController, totalAmount:Double)->PaymentMethodVC?{
        if let container = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "PaymentMethodVC") as? PaymentMethodVC{
            container.parentVC = parentVC
            container.totalAmount = totalAmount
            parentVC.addChild(container)
            parentVC.view.addSubview(container.view)
            container.view.frame = CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.size.width, height: UIScreen.main.bounds.size.height)
            container.view.alpha = 0.0
            UIView.animate(withDuration: 0.25) {
                container.view.alpha = 1.0
            }
            return container
        }
        return nil
    }
    
    func closePopup(){
        UIView.animate(withDuration: 0.25) {
            self.view.alpha = 0.0
        } completion: { (status) in
            self.view.removeFromSuperview()
            self.removeFromParent()
        }
    }
    
    @objc func keyboardWillShow(_ notification: Notification) {
        constViewYCenter.constant = 200
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @objc func keyboardWillHide(_ notification: Notification) {
        constViewYCenter.constant = 0
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    func setPaymentModeView(){
        lblChange.isHidden = true
        txtChequeNo.superview?.superview?.isHidden = true
        if paymentMethod == "cod"{
            lblChange.isHidden = false
            imgCash.image = UIImage.init(named: "ic_radio_selected")
            imgCard.image = UIImage.init(named: "ic_radio_unselected")
            imgCheque.image = UIImage.init(named: "ic_radio_unselected")
            calculateChange()
        }else if paymentMethod == "card"{
            imgCash.image = UIImage.init(named: "ic_radio_unselected")
            imgCard.image = UIImage.init(named: "ic_radio_selected")
            imgCheque.image = UIImage.init(named: "ic_radio_unselected")
        }else{
            if let _ = self.parentVC as? UserCartVC{
                txtChequeNo.superview?.superview?.isHidden = false
            }
            imgCash.image = UIImage.init(named: "ic_radio_unselected")
            imgCard.image = UIImage.init(named: "ic_radio_unselected")
            imgCheque.image = UIImage.init(named: "ic_radio_selected")
        }
        imgCash.imageColor = AppConstants.Colors.kAppMainThemeColor
        imgCard.imageColor = AppConstants.Colors.kAppMainThemeColor
        imgCheque.imageColor = AppConstants.Colors.kAppMainThemeColor
    }
    
    func calculateChange(){
        let amount = Double(txtAmount.text ?? "") ?? 0
        let diff = amount - self.totalAmount
        if diff < 0{
            lblChange.text = String.init(format: "Change: %@%.2f", AppConstants.currencySign, 0)
        }else{
            lblChange.text = String.init(format: "Change: %@%.2f", AppConstants.currencySign, amount - self.totalAmount)
        }
        
    }
    
    @IBAction func btnPaymentMethod(_ sender:UIButton){
        if sender.tag == 0{
            paymentMethod = "cod"
        }else if sender.tag == 1{
            paymentMethod = "card"
        }else{
            paymentMethod = "cheque"
        }
        setPaymentModeView()
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        let amount = Double(txtAmount.text ?? "") ?? 0
        if amount == 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_AMOUNT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                _ = self.txtAmount.becomeFirstResponder()
            }
            return
        }
        if paymentMethod == "card"{
            if amount > totalAmount{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_AMOUNT_GREATER_TOTAL, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    _ = self.txtAmount.becomeFirstResponder()
                }
                return
            }
        }
        if let VC = self.parentVC as? UserCartVC{
            VC.callPlaceOrderAPI(paymentMode: paymentMethod, amount: amount, chequeNo: txtChequeNo.text)
        }else if let VC = self.parentVC as? OrderDetailVC{
            VC.callUpdatePaymentAPI(paymentMode: paymentMethod, amount: amount)
        }
        self.closePopup()
    }
    
    @IBAction func btnCancelAction(_ sender:UIButton){
        self.closePopup()
    }
    
    @IBAction func btnNumberPadAction(_ sender:UIButton){
        var strQuantity = txtAmount.text ?? ""
        var strText = (strQuantity).replacingOccurrences(of: ".", with: "")
        if sender.tag == 10{
            if strText != "000"{
                if strText.count == 3{
                    strText.removeLast()
                    strText = "0\(strText)"
                }else{
                    strText.removeLast()
                }
                strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                strQuantity = "\(strText)"
            }
        }else{
            let string = "\(sender.tag)"
            if strText.first! == "0"{
                strText.removeFirst()
                strText = "\(strText)\(string)"
            }else{
                strText = "\(strText)\(string)"
            }
            strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
            strQuantity = "\(strText)"
        }
        txtAmount.text = strQuantity
        calculateChange()
    }
}

extension PaymentMethodVC:UITextFieldDelegate{
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        if textField == txtAmount{
            let allowedCharacters = CharacterSet(charactersIn:"0123456789")
            let characterSet = CharacterSet(charactersIn: string)
            if allowedCharacters.isSuperset(of: characterSet){
                var strText = (textField.text!).replacingOccurrences(of: ".", with: "")
                if string == ""{
                    if strText != "000"{
                        if strText.count == 3{
                            strText.removeLast()
                            strText = "0\(strText)"
                        }else{
                            strText.removeLast()
                        }
                        strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                        textField.text = "\(strText)"
                    }
                }else{
                    if strText.first! == "0"{
                        strText.removeFirst()
                        strText = "\(strText)\(string)"
                    }else{
                        strText = "\(strText)\(string)"
                    }
                    strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                    textField.text = "\(strText)"
                }
                calculateChange()
            }
            return false
        }
        return true
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
}
