//
//  RefundSalesOrderVC.swift
//  globalfoods
//
//  Created by Apple on 25/11/21.
//

import UIKit

protocol RefundSalesOrderDelegate: AnyObject {
    func refundSubmitted()
}

class RefundSalesOrderVC: UIViewController {

    @IBOutlet weak var txtCashAmount:UITextField!
    @IBOutlet weak var txtCardAmount:UITextField!
    @IBOutlet weak var txtChequeAmount:UITextField!
    @IBOutlet weak var lblTotalRefundedAmount:UILabel!
    @IBOutlet weak var tblReturnOrder:UITableView!
    
    var orderDetail:OrderHistoryModel!
    var arrItems = [CartItemsModel]()
    weak var delegate:RefundSalesOrderDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        txtCashAmount.text = "0.00"
        txtCardAmount.text = "0.00"
        txtChequeAmount.text = "0.00"
        arrItems = orderDetail.arrItems ?? [CartItemsModel]()
        setTotalRefundedAmount()
    }
    
    //MARK: create viewcontroller instance
    static func instance(order:OrderHistoryModel)->SheetViewController?{
        guard let controller = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "RefundSalesOrderVC") as? RefundSalesOrderVC else { return nil }
        controller.orderDetail = order
        let height:CGFloat = UIScreen.main.bounds.size.height - (AppConstants.ScreenSize.topPadding + 100)
        
        return SheetViewController(controller: controller, sizes: [.fixed(height)], options: SheetOptions(useInlineMode: false))
    }
    
    func setTotalRefundedAmount(){
        var totalRefundedAmount = 0.0
        arrItems.forEach { model in
            totalRefundedAmount += (model.approvePrice ?? 0)
        }
        lblTotalRefundedAmount.text = String.init(format: "Refund Amount: %@%.2f", AppConstants.currencySign, totalRefundedAmount)
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        let cashAmount = Double(txtCashAmount.text ?? "") ?? 0
        let cardAmount = Double(txtCardAmount.text ?? "") ?? 0
        let chequeAmount = Double(txtChequeAmount.text ?? "") ?? 0
        var totalRefundedAmount = 0.0
        arrItems.forEach { model in
            totalRefundedAmount += (model.approvePrice ?? 0)
        }
        totalRefundedAmount = Double(String.init(format: "%.2f", totalRefundedAmount)) ?? 0
        if cashAmount > orderDetail.cashAmount ?? 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_CASHAMOUNT_NOT_GREATER, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else if cardAmount > orderDetail.cardAmount ?? 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_CARDAMOUNT_NOT_GREATER, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else if chequeAmount > orderDetail.chequeAmount ?? 0{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_CHEQUEAMOUNT_NOT_GREATER, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else if totalRefundedAmount != (cashAmount + cardAmount + chequeAmount){
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_AMOUNT_EQUAL_RETURN, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else{
            callRefundOrderAPI()
        }
    }
    
    func callRefundOrderAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "133"
        param["order_id"] = "\(orderDetail.id ?? 0)"
        var arrI = [NSMutableDictionary]()
        for model in arrItems{
            let dict = NSMutableDictionary()
            dict["qty"] = model.approveQty ?? 0
            dict["order_item_id"] = model.itemId ?? 0
            arrI.append(dict)
        }
        let totalPiad = (Double(txtCashAmount.text ?? "") ?? 0) + (Double(txtCardAmount.text ?? "") ?? 0)
        let totalAmount = (orderDetail.refundAmount ?? 0) + (Double(txtChequeAmount.text ?? "") ?? 0) + totalPiad
        let cashAmount = (orderDetail.cashAmount ?? 0) - (Double(txtCashAmount.text ?? "") ?? 0)
        let cardAmount = (orderDetail.cardAmount ?? 0) - (Double(txtCardAmount.text ?? "") ?? 0)
        let chequeAmount = (orderDetail.chequeAmount ?? 0) - (Double(txtChequeAmount.text ?? "") ?? 0)
        param["cash_amount"] = "\(cashAmount)"
        param["refund_amount"] = "\(totalAmount)"
        param["card_amount"] = "\(cardAmount)"
        param["cheque_amount"] = "\(chequeAmount)"
        param["item"] = AppCommonMethods.convertJsontoString(json: arrI)
        if AppConstants.isInDevlopmentMode{
            print("Refund Order Param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCustomerURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Refund Order Data:",response!)
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                    if response!.getIntValue("ack") ?? 0 == 1{
                        if self.delegate != nil{
                            self.delegate!.refundSubmitted()
                        }
                        self.dismiss(animated: true) {
                            AppValidation.shared.showBottomAlertView(settings: setting)
                        }
                        return
                    }
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
}

extension RefundSalesOrderVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrItems.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderDetailViewCell", for: indexPath) as! OrderDetailViewCell
        let model = arrItems[indexPath.row]
        cell.lblName.text = model.itemName ?? ""
        cell.txtQty.text = "\(model.approveQty ?? 0)"
        cell.txtQty.tag = indexPath.row
        cell.lblQty.setLabelAttributedText(title: "Quantity", value: String.init(format: "%.1f", model.itemQty ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.lblPrice.setLabelAttributedText(title: "Refund Amount", value: String.init(format: "%@%.2f", AppConstants.currencySign ,model.approvePrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        return cell
    }
}

extension RefundSalesOrderVC:UITextFieldDelegate{
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
    
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        if textField == txtCashAmount || textField == txtCardAmount || textField == txtChequeAmount{
            let allowedCharacters = CharacterSet(charactersIn:"0123456789")
            let characterSet = CharacterSet(charactersIn: string)
            if allowedCharacters.isSuperset(of: characterSet){
                var strText = (textField.text ?? "").replacingOccurrences(of: ".", with: "")
                if string == ""{
                    if strText != "000"{
                        if strText.count == 3{
                            strText.removeLast()
                            strText = "0\(strText)"
                        }else{
                            strText.removeLast()
                        }
                        strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                        textField.text = "\(strText)"
                    }
                }else{
                    if strText.first! == "0"{
                        strText.removeFirst()
                        strText = "\(strText)\(string)"
                    }else{
                        strText = "\(strText)\(string)"
                    }
                    strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                    textField.text = "\(strText)"
                }
            }
            return false
        }else{
            let allowedCharacters = CharacterSet(charactersIn:"0123456789.")
            let characterSet = CharacterSet(charactersIn: string)
            return allowedCharacters.isSuperset(of: characterSet)
        }
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        if textField != txtCashAmount && textField != txtCardAmount && textField != txtChequeAmount{
            let item = arrItems[textField.tag]
            let qty = Double(textField.text ?? "") ?? 0
            if qty == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_ITEM_NOT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblReturnOrder.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                    cell.txtQty.text = "\(item.approveQty ?? 0)"
                }
                return
            }else if qty > item.itemQty ?? 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.QTY_BELOW_SELECTED_QTY, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblReturnOrder.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                    cell.txtQty.text = "\(item.approveQty ?? 0)"
                }
                return
            }
            
            if let cell = tblReturnOrder.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? OrderDetailViewCell{
                item.approvePrice = qty*(item.itemPrice ?? 0)
                cell.lblPrice.setLabelAttributedText(title: "Refund Amount", value: String.init(format: "%@%.2f", AppConstants.currencySign ,item.approvePrice ?? 0), fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
                cell.txtQty.text = "\(qty)"
            }
            item.approveQty = qty
            setTotalRefundedAmount()
        }
    }
}
