//
//  SelectCustomerVC.swift
//  globalfoods
//
//  Created by Apple on 18/01/21.
//

import UIKit
import DropDown

protocol customerSelctionDelegate: AnyObject {
    func customerSelected(customerData:UserModel?,branchData:branchModel?,customerName:String?)
}

class SelectCustomerVC: UIViewController {

    @IBOutlet weak var txtCustomerSearch:UITextField!
    @IBOutlet weak var txtBranchSearch:UITextField!
    @IBOutlet weak var constViewYCenter:NSLayoutConstraint!
    
    var customerData:UserModel?
    var branchData:branchModel?
    weak var delegate:customerSelctionDelegate?
    var arrCustomers = [UserModel]()
    var arrSearchCustomers = [UserModel]()
    var isCustomerSearchEnabled = false
    var isBranchSearchEnabled = false
    var arrBranches = [branchModel]()
    var arrSearchBranches = [branchModel]()
    let customerDropDown = DropDown()
    let branchDropDown = DropDown()
    var isSalesOrder = false
    var customerName = ""
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        callGetCustomersAPI()
        txtCustomerSearch.text = customerData?.name ?? ""
        txtBranchSearch.text = branchData?.name ?? ""
        if isSalesOrder{
            txtCustomerSearch.text = customerName
        }
        
        if branchData == nil{
            self.txtBranchSearch.isUserInteractionEnabled = false
        }else{
            callGetBranchesAPI()
        }
        
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillShow(_:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillHide(_:)), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self)
    }
    
    //MARK: create viewcontroller instance
    class func showPopup(parentVC:UIViewController, branchData:branchModel?, customerData:UserModel?, isSalesOrder:Bool = false, customerName:String = "")->SelectCustomerVC?{
        if let container = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "SelectCustomerVC") as? SelectCustomerVC{
            container.customerData = customerData
            container.branchData = branchData
            container.customerName = customerName
            container.isSalesOrder = isSalesOrder
            parentVC.addChild(container)
            parentVC.view.addSubview(container.view)
            container.view.frame = CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.size.width, height: UIScreen.main.bounds.size.height)
            container.view.alpha = 0.0
            UIView.animate(withDuration: 0.25) {
                container.view.alpha = 1.0
            }
            return container
        }
        return nil
    }
    
    func closePopup(){
        UIView.animate(withDuration: 0.25) {
            self.view.alpha = 0.0
        } completion: { (status) in
            self.view.removeFromSuperview()
            self.removeFromParent()
        }
    }
    
    @objc func keyboardWillShow(_ notification: Notification) {
        constViewYCenter.constant = 200
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @objc func keyboardWillHide(_ notification: Notification) {
        constViewYCenter.constant = 0
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @IBAction func txtCustomerSearchChanged(_ sender:UITextField){
        if sender.text ?? "" == ""{
            isCustomerSearchEnabled = false
        }else{
            isCustomerSearchEnabled = true
            arrSearchCustomers = arrCustomers.filter({ (model) -> Bool in
                return (model.name ?? "").lowercased().contains((sender.text ?? "").lowercased())
            })
        }
        var arrName = [String]()
        for model in (isCustomerSearchEnabled ? arrSearchCustomers : arrCustomers){
            arrName.append(model.name ?? "")
        }
        customerDropDown.dataSource = arrName
        if customerDropDown.isDropdownHidden{
            customerDropDown.show()
        }else{
            customerDropDown.reloadAllComponents()
        }
    }
    
    @IBAction func txtBranchSearchChanged(_ sender:UITextField){
        if sender.text ?? "" == ""{
            isBranchSearchEnabled = false
        }else{
            isBranchSearchEnabled = true
            arrSearchBranches = arrBranches.filter({ (model) -> Bool in
                return (model.name ?? "").lowercased().contains((sender.text ?? "").lowercased())
            })
        }
        var arrName = [String]()
        for model in (isBranchSearchEnabled ? arrSearchBranches : arrBranches){
            arrName.append(model.name ?? "")
        }
        branchDropDown.dataSource = arrName
        if branchDropDown.isDropdownHidden{
            branchDropDown.show()
        }else{
            branchDropDown.reloadAllComponents()
        }
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        if !isSalesOrder{
            if customerData == nil{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_CUSTOMER, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                return
            }
            if branchData == nil{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_BRANCH, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                return
            }
        }else{
            if (txtCustomerSearch.text ?? "").trimmingCharacters(in: .whitespacesAndNewlines).isEmpty{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_EMPTY_CUSTOMERNAME, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                return
            }
        }
        self.closePopup()
        if delegate != nil{
            self.delegate?.customerSelected(customerData: customerData, branchData: branchData, customerName: self.txtCustomerSearch.text)
        }
    }
    
    @IBAction func btnCancelAction(_ sender:UIButton){
        self.closePopup()
    }
    
    func callGetCustomersAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "59"
        param["eid"] = "\(AppConstants.employeeData?.id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCustomerURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrCustomers.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Customers Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([UserModel].self, from: data){
                                self.arrCustomers = aData
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
    func callGetBranchesAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "48"
        param["cuid"] = "\(customerData?.id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrBranches.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("branch Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([branchModel].self, from: data){
                                self.arrBranches = aData
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension SelectCustomerVC:UITextFieldDelegate{
    func textFieldDidBeginEditing(_ textField: UITextField) {
        if textField != txtBranchSearch{
            isCustomerSearchEnabled = false
            txtCustomerSearch.text = ""
            customerDropDown.dismissMode = .manual
            customerDropDown.anchorView = txtCustomerSearch
            customerDropDown.bottomOffset = CGPoint(x: 0, y: txtCustomerSearch.bounds.height)
            customerDropDown.topOffset = CGPoint(x: 0, y: -txtCustomerSearch.bounds.height)
            var arrName = [String]()
            for model in (isCustomerSearchEnabled ? arrSearchCustomers : arrCustomers){
                arrName.append(model.name ?? "")
            }
            customerDropDown.dataSource = arrName
            // Action triggered on selection
            customerDropDown.selectionAction = { [weak self] (index, item) in
                if ((self?.isCustomerSearchEnabled ?? false) ? self?.arrSearchCustomers : self?.arrCustomers)?[index].id ?? 0 != self?.customerData?.id ?? 0{
                    self?.customerData = ((self?.isCustomerSearchEnabled ?? false) ? self?.arrSearchCustomers : self?.arrCustomers)?[index]
                    self?.txtBranchSearch.text = ""
                    self?.branchData = nil
                    self?.txtBranchSearch.isUserInteractionEnabled = true
                    self?.callGetBranchesAPI()
                    self?.view.endEditing(true)
                }
                self?.txtCustomerSearch.text = item
            }
            customerDropDown.show()
        }else{
            isBranchSearchEnabled = false
            txtBranchSearch.text = ""
            branchDropDown.dismissMode = .manual
            branchDropDown.anchorView = txtBranchSearch
            branchDropDown.bottomOffset = CGPoint(x: 0, y: txtBranchSearch.bounds.height)
            branchDropDown.topOffset = CGPoint(x: 0, y: -txtBranchSearch.bounds.height)
            var arrName = [String]()
            for model in (isBranchSearchEnabled ? arrSearchBranches : arrBranches){
                arrName.append(model.name ?? "")
            }
            branchDropDown.dataSource = arrName
            // Action triggered on selection
            branchDropDown.selectionAction = { [weak self] (index, item) in
                self?.branchData = ((self?.isBranchSearchEnabled ?? false) ? self?.arrSearchBranches : self?.arrBranches)?[index]
                self?.txtBranchSearch.text = item
                self?.view.endEditing(true)
            }
            branchDropDown.show()
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        if !isSalesOrder{
            txtCustomerSearch.text = customerData?.name ?? ""
            txtBranchSearch.text = branchData?.name ?? ""
        }
    }
}
