//
//  SelectDriverVC.swift
//  globalfoods
//
//  Created by Apple on 18/01/21.
//

import UIKit
import DropDown

protocol driverSelctionDelegate: AnyObject {
    func driverSelected(driverData:EmployeeModel?,vehicleData:vehicleModel?)
}

class SelectDriverVC: UIViewController {

    @IBOutlet weak var txtDriverSearch:UITextField!
    @IBOutlet weak var txtVehicleSearch:UITextField!
    @IBOutlet weak var constViewYCenter:NSLayoutConstraint!
    
    var driverData:EmployeeModel?
    var vehicleData:vehicleModel?
    weak var delegate:driverSelctionDelegate?
    var arrDrivers = [EmployeeModel]()
    var arrSearchDrivers = [EmployeeModel]()
    var isDriverSearchEnabled = false
    var isVehicleSearchEnabled = false
    var arrVehiclees = [vehicleModel]()
    var arrSearchVehiclees = [vehicleModel]()
    let driverDropDown = DropDown()
    let vehicleDropDown = DropDown()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        callGetDriversAPI()
        txtDriverSearch.text = driverData?.name ?? ""
        txtVehicleSearch.text = vehicleData?.name ?? ""
        
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillShow(_:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillHide(_:)), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self)
    }
    
    //MARK: create viewcontroller instance
    class func showPopup(parentVC:UIViewController, vehicleData:vehicleModel?, driverData:EmployeeModel?)->SelectDriverVC?{
        if let container = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "SelectDriverVC") as? SelectDriverVC{
            container.driverData = driverData
            container.vehicleData = vehicleData
            parentVC.addChild(container)
            parentVC.view.addSubview(container.view)
            container.view.frame = CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.size.width, height: UIScreen.main.bounds.size.height)
            container.view.alpha = 0.0
            UIView.animate(withDuration: 0.25) {
                container.view.alpha = 1.0
            }
            return container
        }
        return nil
    }
    
    func closePopup(){
        UIView.animate(withDuration: 0.25) {
            self.view.alpha = 0.0
        } completion: { (status) in
            self.view.removeFromSuperview()
            self.removeFromParent()
        }
    }
    
    @objc func keyboardWillShow(_ notification: Notification) {
        constViewYCenter.constant = 200
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @objc func keyboardWillHide(_ notification: Notification) {
        constViewYCenter.constant = 0
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @IBAction func txtDriverSearchChanged(_ sender:UITextField){
        if sender.text ?? "" == ""{
            isDriverSearchEnabled = false
        }else{
            isDriverSearchEnabled = true
            arrSearchDrivers = arrDrivers.filter({ (model) -> Bool in
                return (model.name ?? "").lowercased().contains((sender.text ?? "").lowercased())
            })
        }
        var arrName = [String]()
        for model in (isDriverSearchEnabled ? arrSearchDrivers : arrDrivers){
            arrName.append(model.name ?? "")
        }
        driverDropDown.dataSource = arrName
        if driverDropDown.isDropdownHidden{
            driverDropDown.show()
        }else{
            driverDropDown.reloadAllComponents()
        }
    }
    
    @IBAction func txtVehicleSearchChanged(_ sender:UITextField){
        if sender.text ?? "" == ""{
            isVehicleSearchEnabled = false
        }else{
            isVehicleSearchEnabled = true
            arrSearchVehiclees = arrVehiclees.filter({ (model) -> Bool in
                return (model.name ?? "").lowercased().contains((sender.text ?? "").lowercased())
            })
        }
        var arrName = [String]()
        for model in (isVehicleSearchEnabled ? arrSearchVehiclees : arrVehiclees){
            arrName.append("\(model.name ?? "") - \(model.vehicleNo ?? "")")
        }
        vehicleDropDown.dataSource = arrName
        if vehicleDropDown.isDropdownHidden{
            vehicleDropDown.show()
        }else{
            vehicleDropDown.reloadAllComponents()
        }
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        self.view.endEditing(true)
        if driverData == nil{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_DRIVER, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else if vehicleData == nil{
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_SELECT_VEHICLE, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                
            }
        }else{
            self.closePopup()
            if delegate != nil{
                self.delegate?.driverSelected(driverData: driverData, vehicleData: vehicleData)
            }
        }
    }
    
    @IBAction func btnCancelAction(_ sender:UIButton){
        self.closePopup()
    }
    
    func callGetDriversAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "70"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceOrderURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrDrivers.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Drivers Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if let driver = result["employee"] as? [NSDictionary], let data = AppCommonMethods.convertToJson(object: driver){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([EmployeeModel].self, from: data){
                                self.arrDrivers = aData
                            }
                        }
                        if let vehicle = result["vehical"] as? [NSDictionary], let data = AppCommonMethods.convertToJson(object: vehicle){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([vehicleModel].self, from: data){
                                self.arrVehiclees = aData
                            }
                        }
                        return
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension SelectDriverVC:UITextFieldDelegate{
    func textFieldDidBeginEditing(_ textField: UITextField) {
        if textField != txtVehicleSearch{
            isDriverSearchEnabled = false
            txtDriverSearch.text = ""
            driverDropDown.dismissMode = .manual
            driverDropDown.anchorView = txtDriverSearch
            driverDropDown.bottomOffset = CGPoint(x: 0, y: txtDriverSearch.bounds.height)
            driverDropDown.topOffset = CGPoint(x: 0, y: -txtDriverSearch.bounds.height)
            var arrName = [String]()
            for model in (isDriverSearchEnabled ? arrSearchDrivers : arrDrivers){
                arrName.append(model.name ?? "")
            }
            driverDropDown.dataSource = arrName
            // Action triggered on selection
            driverDropDown.selectionAction = { [weak self] (index, item) in
                if ((self?.isDriverSearchEnabled ?? false) ? self?.arrSearchDrivers : self?.arrDrivers)?[index].id ?? 0 != self?.driverData?.id ?? 0{
                    self?.driverData = ((self?.isDriverSearchEnabled ?? false) ? self?.arrSearchDrivers : self?.arrDrivers)?[index]
                    self?.view.endEditing(true)
                }
                self?.txtDriverSearch.text = item
            }
            driverDropDown.show()
        }else{
            isVehicleSearchEnabled = false
            txtVehicleSearch.text = ""
            vehicleDropDown.dismissMode = .manual
            vehicleDropDown.anchorView = txtVehicleSearch
            vehicleDropDown.bottomOffset = CGPoint(x: 0, y: txtVehicleSearch.bounds.height)
            vehicleDropDown.topOffset = CGPoint(x: 0, y: -txtVehicleSearch.bounds.height)
            var arrName = [String]()
            for model in (isVehicleSearchEnabled ? arrSearchVehiclees : arrVehiclees){
                arrName.append("\(model.name ?? "") - \(model.vehicleNo ?? "")")
            }
            vehicleDropDown.dataSource = arrName
            // Action triggered on selection
            vehicleDropDown.selectionAction = { [weak self] (index, item) in
                self?.vehicleData = ((self?.isVehicleSearchEnabled ?? false) ? self?.arrSearchVehiclees : self?.arrVehiclees)?[index]
                self?.txtVehicleSearch.text = item
                self?.view.endEditing(true)
            }
            vehicleDropDown.show()
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        return textField.resignFirstResponder()
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        txtDriverSearch.text = driverData?.name ?? ""
        txtVehicleSearch.text = vehicleData?.name ?? ""
    }
}
