//
//  UserTextViewVC.swift
//  globalfoods
//
//  Created by Apple on 31/12/20.
//

import UIKit

protocol UserTextViewDelegate: AnyObject {
    func userAddedText(string:String)
}

enum TextViewType {
    case kAddRemark
    case kCancelReason
    case kSupportSubject
}

class UserTextViewVC: UIViewController {
    
    @IBOutlet weak var lblTitle:UILabel!
    @IBOutlet weak var lblPlaceholder:UILabel!
    @IBOutlet weak var lblTxtCount:UILabel!
    @IBOutlet weak var txtView:UITextView!
    @IBOutlet weak var constViewYCenter:NSLayoutConstraint!
    var textViewString = ""
    weak var delegate:UserTextViewDelegate?
    var viewType = TextViewType.kAddRemark
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        self.txtView.text = textViewString
        lblTxtCount.text = "\(txtView.text!.count)/300"
        if viewType == .kCancelReason{
            lblPlaceholder.text = "Enter reason for cancellation"
            lblTitle.text = "Reason for cancellation"
        }else if viewType == .kSupportSubject{
            lblPlaceholder.text = "Enter Subject"
            lblTitle.text = "Subject"
        }else{
            if textViewString != ""{
                lblTitle.text = "Edit Comment"
            }
        }
        if txtView.text! == ""{
            lblPlaceholder.isHidden = false
        }else{
            lblPlaceholder.isHidden = true
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillShow(_:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.keyboardWillHide(_:)), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self)
    }
    
    //MARK: create viewcontroller instance
    class func showPopup(parentVC:UIViewController,string:String,viewType:TextViewType = .kAddRemark)->UserTextViewVC?{
        if let container = UIStoryboard.init(name: "PopUps", bundle: nil).instantiateViewController(withIdentifier: "UserTextViewVC") as? UserTextViewVC{
            container.textViewString = string
            container.viewType = viewType
            parentVC.addChild(container)
            parentVC.view.addSubview(container.view)
            container.view.frame = CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.size.width, height: UIScreen.main.bounds.size.height)
            container.view.alpha = 0.0
            UIView.animate(withDuration: 0.25) {
                container.view.alpha = 1.0
            }
            return container
        }
        return nil
    }
    
    func closePopup(){
        UIView.animate(withDuration: 0.25) {
            self.view.alpha = 0.0
        } completion: { (status) in
            self.view.removeFromSuperview()
            self.removeFromParent()
        }
    }
    
    @IBAction func btnConfirmAction(_ sender:UIButton){
        if (txtView.text?.trimmingCharacters(in: .whitespaces).isEmpty)!
        {
            var strMsg = AppConstants.GlobalAlert.ALERT_ENTER_REMARK
            if viewType == .kCancelReason{
                strMsg = AppConstants.GlobalAlert.ALERT_ENTER_CANCELREASON
            }else if viewType == .kCancelReason{
                strMsg = AppConstants.GlobalAlert.ALERT_ENTER_SUBJECT
            }
            AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg:  strMsg, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                _ = self.txtView.becomeFirstResponder()
            }
        }else{
            if delegate != nil{
                self.delegate?.userAddedText(string: txtView.text ?? "")
                self.closePopup()
            }
        }
    }
    
    @IBAction func btnCancelAction(_ sender:UIButton){
        self.view.endEditing(true)
        self.closePopup()
    }

    @objc func keyboardWillShow(_ notification: Notification) {
        constViewYCenter.constant = 200
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
    
    @objc func keyboardWillHide(_ notification: Notification) {
        constViewYCenter.constant = 0
        UIView.animate(withDuration: 0.2) {
            self.view.layoutIfNeeded()
        }
    }
}

extension UserTextViewVC:UITextViewDelegate{
    func textViewDidChange(_ textView: UITextView) {
        lblTxtCount.text = "\(textView.text!.count)/300"
        if textView.text! == ""{
            lblPlaceholder.isHidden = false
        }else{
            lblPlaceholder.isHidden = true
        }
    }
    
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        if text == ""{
            return true
        }
        if textView.text.count >= 300{
            return false
        }
        return true
    }
}
