//
//  InvoiceVC.swift
//  globalfoods
//
//  Created by Apple on 29/12/20.
//

import UIKit
import ActionSheetPicker_3_0

class InvoicesCell: UITableViewCell {
    
    @IBOutlet weak var lblInvoiceNo:UILabel!
    @IBOutlet weak var btnDownload:UIButton!
    @IBOutlet weak var btnEdit:UIButton!
    @IBOutlet weak var lblDescription:UILabel!
    @IBOutlet weak var lblTotalPrice:UILabel!
    
    override class func awakeFromNib() {
        
    }
}

class InvoiceVC: UIViewController {

    @IBOutlet weak var lblFromDate:UILabel!
    @IBOutlet weak var lblToDate:UILabel!
    @IBOutlet weak var tblInvoices:UITableView!
    
    var customHeaderView:HeaderView?
    var strToDate = ""
    var strFromDate = ""
    var arrInvoice = [InvoiceModel]()
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        self.strFromDate = Date().addingTimeInterval(-7*24*60*60).getDateInString(format: "dd-MM-yyyy")
        self.strToDate = Date().getDateInString(format: "dd-MM-yyyy")
        self.lblFromDate.text = "From - \(Date().addingTimeInterval(-7*24*60*60).getDateInString(format: "dd/MM/yyyy"))"
        self.lblToDate.text = "To - \(Date().getDateInString(format: "dd/MM/yyyy"))"
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
        callGetInvoiceAPI()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->InvoiceVC?{
        return UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "InvoiceVC") as? InvoiceVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Invoice", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    func checkPDFExist(id:Int)->Bool{
        let fileName = "Invoices_\(id).pdf"
        let documentsUrl =  FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
        let destinationFileUrl = documentsUrl.appendingPathComponent("\(fileName)")
        return FileManager.default.fileExists(atPath: destinationFileUrl.path)
    }
    
    func downloadPDFFile(id:Int,path:String){
        let fileName = "Invoices_\(id).pdf"
        let documentsUrl =  FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
        let destinationFileUrl = documentsUrl.appendingPathComponent("\(fileName)")
        
        if let fileURL = URL(string: path){
            AppCommonMethods.startProgressBar()
            DispatchQueue.global().async {
                do{
                    let data = try Data.init(contentsOf: fileURL)
                    try data.write(to: destinationFileUrl)
                    DispatchQueue.main.async {
                        AppCommonMethods.stopProgressBar()
                        if let VC = WebViewController.instance(){
                            VC.strURL = destinationFileUrl.absoluteString
                            self.navigationController?.pushViewController(VC, animated: true)
                        }
                    }
                }catch (let err) {
                    DispatchQueue.main.async {
                        AppCommonMethods.stopProgressBar()
                    }
                    print("error: \(err)")
                }
            }
        }
    }
    
    //MARK: Button Actions
    @IBAction func btnFromDateAction(_ sender:UIButton){
        let date = strFromDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        let datePicker = ActionSheetDatePicker(title: "Select from date", datePickerMode: UIDatePicker.Mode.date, selectedDate: date, doneBlock: {
            picker, value, index in
            self.strFromDate = (value as! Date).getDateInString(format: "dd-MM-yyyy")
            self.lblFromDate.text = "From - \((value as! Date).getDateInString(format: "dd/MM/yyyy"))"
            self.callGetInvoiceAPI()
        }, cancel: { ActionStringCancelBlock in return }, origin: sender)
        datePicker?.maximumDate = strToDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        datePicker?.show()
    }
    
    @IBAction func btnToDateAction(_ sender:UIButton){
        let date = strToDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        let datePicker = ActionSheetDatePicker(title: "Select to date", datePickerMode: UIDatePicker.Mode.date, selectedDate: date, doneBlock: {
            picker, value, index in
            self.strToDate = (value as! Date).getDateInString(format: "dd-MM-yyyy")
            self.lblToDate.text = "To - \((value as! Date).getDateInString(format: "dd/MM/yyyy"))"
            self.callGetInvoiceAPI()
        }, cancel: { ActionStringCancelBlock in return }, origin: sender)
        if let fromDate = strFromDate.getDateFromString(format: "dd-MM-yyyy", locale: nil){
            datePicker?.minimumDate = fromDate
        }
        datePicker?.show()
    }
    
    @IBAction func btnEditAction(_ sender:UIButton){
        if let VC = UpdateInvoiceVC.instance(){
            VC.invoiceDetail = arrInvoice[sender.tag]
            self.navigationController?.pushViewController(VC, animated: true)
        }
    }
    
    @IBAction func btnDownloadAction(_ sender:UIButton){
        if sender.isSelected{
            if let VC = WebViewController.instance(){
                let fileName = "Invoices_\(arrInvoice[sender.tag].id ?? 0).pdf"
                let documentsUrl =  FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
                let destinationFileUrl = documentsUrl.appendingPathComponent("\(fileName)")
                VC.strURL = destinationFileUrl.absoluteString
                self.navigationController?.pushViewController(VC, animated: true)
            }
        }else{
            downloadPDFFile(id: arrInvoice[sender.tag].id ?? 0, path: arrInvoice[sender.tag].path ?? "")
        }
    }
    
    func callGetInvoiceAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        var strURL = AppConstants.APIURLs.kServiceCustomerURL
        if AppConstants.userData == nil{
            param["s"] = "89"
            strURL = AppConstants.APIURLs.kServiceEmployeeURL
        }else{
            param["s"] = "55"
            param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
            param["cbid"] = "\(AppConstants.selectedBranch?.id ?? 0)"
        }
        param["from_date"] = strFromDate
        param["to_date"] = strToDate
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: strURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrInvoice.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Invoices Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([InvoiceModel].self, from: data){
                                self.arrInvoice = aData
                                self.tblInvoices.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblInvoices.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
}

extension InvoiceVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrInvoice.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "InvoicesCell", for: indexPath) as! InvoicesCell
        let model = arrInvoice[indexPath.row]
        cell.lblInvoiceNo.text = "Sales Invoice No: #\(model.invoiceNo ?? "")"
        cell.lblTotalPrice.text = ""
        if let data = AppConstants.userData{
            if data.showPrice ?? 0 == 1{
                cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign, model.grandTotal ?? 0)
            }
        }else{
            cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign, model.grandTotal ?? 0)
        }
        if checkPDFExist(id: model.id ?? 0){
            cell.btnDownload.isSelected = true
        }else{
            cell.btnDownload.isSelected = false
        }
        if AppConstants.employeeData?.rights?.invoice?.update ?? 0 == 1{
            cell.btnEdit.isHidden = false
        }else{
            cell.btnEdit.isHidden = true
        }
        cell.btnEdit.tag = indexPath.row
        cell.btnDownload.tag = indexPath.row
        if let invoiceDate = model.invoiceDate?.getDateFromString(format: "dd-MM-yy"){
            let strDesc = "Customer name: \(model.customerName ?? "")\nInvoice date: \(invoiceDate.getDateInString(format: "dd'\(invoiceDate.getDateSuffix())' MMM, yyyy"))"
            let attrDesc = NSMutableAttributedString.init(string: strDesc, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
            
            attrDesc.addFontAttribute(title: "Invoice date", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Customer name", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            cell.lblDescription.attributedText = attrDesc
        }
        return cell
    }
}
