//
//  NotificationsVC.swift
//  globalfoods
//
//  Created by Apple on 05/01/21.
//

import UIKit

class NotificationCell: UITableViewCell {
    
    @IBOutlet weak var lblTitle:UILabel!
    @IBOutlet weak var lblMessage:UILabel!
    @IBOutlet weak var lblDate:UILabel!
    
    override class func awakeFromNib() {
        
    }
}

class NotificationsVC: UIViewController {

    @IBOutlet weak var tblNotifications:UITableView!
    
    var customHeaderView:HeaderView?
    var arrNotification = [NotificationModel]()
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
        callGetNotificationAPI()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->NotificationsVC?{
        return UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "NotificationsVC") as? NotificationsVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Notifications", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    func callGetNotificationAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        if AppConstants.userData == nil{
            param["s"] = "73"
            param["emp_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        }else{
            param["s"] = "24"
            param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
        }
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceGeneralURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrNotification.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Notification Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([NotificationModel].self, from: data){
                                self.arrNotification = aData
                                self.tblNotifications.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblNotifications.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
}

extension NotificationsVC:UITableViewDelegate,UITableViewDataSource{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrNotification.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "NotificationCell", for: indexPath) as! NotificationCell
        let model = arrNotification[indexPath.row]
        cell.lblTitle.text = model.title ?? ""
        cell.lblMessage.text = model.description ?? ""
        if let date = model.date?.getDateFromString(format: "dd-MM-yyyy"){
            cell.lblDate.text = date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy")
        }
        return cell
    }
}
