//
//  OrderHistoryVC.swift
//  globalfoods
//
//  Created by Apple on 29/12/20.
//

import UIKit

class OrderHistoryCell: UITableViewCell {
    
    @IBOutlet weak var lblOrderNo:UILabel!
    @IBOutlet weak var btnDelete:UIButton!
    @IBOutlet weak var btnApprove:UIButton!
    @IBOutlet weak var btnReject:UIButton!
    @IBOutlet weak var btnReprint:UIButton!
    @IBOutlet weak var btnOrders:UIButton!
    @IBOutlet weak var viewForAction:UIView!
    @IBOutlet weak var lblDescription:UILabel!
    @IBOutlet weak var lblOrderPrice:UILabel!
    
    override class func awakeFromNib() {
        
    }
}

class OrderHistoryVC: UIViewController {

    @IBOutlet weak var tblOrderHistory:UITableView!
    
    var customHeaderView:HeaderView?
    var arrOrderHistory = [OrderHistoryModel]()
    var selectedRow = 0
    var strStatus = ""
    var isBackEnabled = false
    var isSalesOrder = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
        callGetOrderHistoryAPI()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->OrderHistoryVC?{
        return UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "OrderHistoryVC") as? OrderHistoryVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: isSalesOrder ? "Sales Order History" : "Order History", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func btnDeleteAction(_ sender:UIButton){
        selectedRow = sender.tag
        if let VC = UserTextViewVC.showPopup(parentVC: self, string: "", viewType: .kCancelReason){
            VC.delegate = self
        }
    }
    
    @IBAction func btnReprintAction(_ sender:UIButton){
        self.callGetOrderDetailAPI(orderDetail: arrOrderHistory[sender.tag])
    }
    
    @IBAction func btnApproveAction(_ sender:UIButton){
        selectedRow = sender.tag
        strStatus = "approve"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to approve this order?"
        settings.alertTitle = "Approve Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    @IBAction func btnRejectAction(_ sender:UIButton){
        selectedRow = sender.tag
        strStatus = "reject"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to reject this order?"
        settings.alertTitle = "Reject Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    func callGetOrderHistoryAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        
        AppCommonMethods.startProgressBar()
        if AppConstants.employeeData != nil{
            param["s"] = isSalesOrder ? "130" : "88"
        }else{
            param["s"] = "6"
            if AppConstants.subCustomerData != nil{
                param["cuid"] = "\(AppConstants.subCustomerData?.cid ?? 0)"
                param["cbid"] = "\(AppConstants.subCustomerData?.cbid ?? 0)"
            }else{
                param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
                param["cbid"] = "\(AppConstants.selectedBranch?.id ?? 0)"
            }
        }
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: (AppConstants.employeeData != nil) ? AppConstants.APIURLs.kServiceEmployeeURL : AppConstants.APIURLs.kServiceCustomerURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrOrderHistory.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Order History Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([OrderHistoryModel].self, from: data){
                                self.arrOrderHistory = aData
                                self.tblOrderHistory.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblOrderHistory.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
    
    func callGetOrderDetailAPI(orderDetail:OrderHistoryModel){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = isSalesOrder ? "131": "7"
        param["order_id"] = "\(orderDetail.id ?? 0)"
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCartURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Order Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode(OrderHistoryModel.self, from: data){
                                PrintManager.shared.isSalesOrder = self.isSalesOrder
                                PrintManager.shared.connectToPrinterWithPrint(model: aData)
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
    func callDeleteOrderAPI(strReason:String){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "31"
        param["order_id"] = "\(arrOrderHistory[selectedRow].id ?? 0)"
        param["reason"] = strReason
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceOrderURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Delete Order Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        self.callGetOrderHistoryAPI()
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
    
    func callRejectOrderAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "113"
        param["id"] = "\(arrOrderHistory[selectedRow].id ?? 0)"
        param["order_type"] = strStatus
        param["customer_id"] = "\(arrOrderHistory[selectedRow].cid ?? 0)"
        param["customer_branch_id"] = "\(arrOrderHistory[selectedRow].cbid ?? 0)"
        param["sales_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["order_date"] = arrOrderHistory[selectedRow].date ?? ""
        if AppConstants.isInDevlopmentMode{
            print("Reject Order Param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Reject Order Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        self.callGetOrderHistoryAPI()
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
}

extension OrderHistoryVC:UITableViewDelegate,UITableViewDataSource{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrOrderHistory.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderHistoryCell", for: indexPath) as! OrderHistoryCell
        let model = arrOrderHistory[indexPath.row]
        cell.lblOrderPrice.text = ""
        if let data = AppConstants.userData{
            if data.showPrice ?? 0 == 1{
                cell.lblOrderPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign,model.orderTotal ?? 0)
            }
        }else{
            cell.lblOrderPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign,model.orderTotal ?? 0)
        }
        cell.lblOrderPrice.text = ""
        cell.btnDelete.tag = indexPath.row
        cell.btnReject.tag = indexPath.row
        cell.btnApprove.tag = indexPath.row
        cell.btnReprint.tag = indexPath.row
        if model.status ?? 0 == 1 && AppConstants.subCustomerData == nil{
            cell.viewForAction.isHidden = false
            if AppConstants.employeeData == nil{
                if AppConstants.userData?.allowCancel ?? 0 == 0{
                    cell.viewForAction.isHidden = true
                }else{
                    cell.btnDelete.isHidden = false
                }
            }else{
                cell.btnReject.isHidden = false
                cell.btnApprove.isHidden = false
                cell.btnReprint.isHidden = true
            }
        }else{
            if AppConstants.employeeData != nil{
                if model.status ?? 0 == -1{
                    cell.viewForAction.isHidden = true
                }else{
                    if AppConstants.printerIP ?? "" != ""{
                        cell.viewForAction.isHidden = false
                    }else{
                        cell.viewForAction.isHidden = true
                    }
                }
                cell.btnReprint.isHidden = false
                cell.btnReject.isHidden = true
                cell.btnApprove.isHidden = true
            }else{
                cell.viewForAction.isHidden = true
            }
        }
        if AppConstants.employeeData?.rights?.order?.update ?? 0 == 0{
            cell.viewForAction.isHidden = true
        }
        cell.lblOrderNo.text = "Order No: #\(model.orderNo ?? "")"
        if let date = model.date?.getDateFromString(format: "dd-MM-yyyy"){
            var strDesc = "Ordered on: \(date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy"))\nOrder status: \(model.statusText ?? "")"
            if AppConstants.employeeData != nil{
                strDesc = "Customer name: \(model.customerName ?? "")\nOrdered on: \(date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy"))\nOrder status: \(model.statusText ?? "")"
            }
            let attrDesc = NSMutableAttributedString.init(string: strDesc, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
            attrDesc.addFontAttribute(title: "Ordered on", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Order status", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Status", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            if AppConstants.employeeData != nil{
                attrDesc.addFontAttribute(title: "Customer name", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            }
            
            cell.lblDescription.attributedText = attrDesc
        }
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let VC = OrderDetailVC.instance(){
            VC.isSalesOrder = self.isSalesOrder
            VC.orderDetail = arrOrderHistory[indexPath.row]
            self.navigationController?.pushViewController(VC, animated: true)
        }
    }
}

extension OrderHistoryVC:UserTextViewDelegate{
    func userAddedText(string:String) {
        callDeleteOrderAPI(strReason: string)
    }
}

extension OrderHistoryVC:alertClickedDelegate{
    func alertButtonClicked(index: Int,value:String) {
        if index == 1{
            if strStatus == "approve"{
                if let VC = ApproveOrderVC.instance(){
                    VC.orderDetail = arrOrderHistory[selectedRow]
                    self.navigationController?.pushViewController(VC, animated: true)
                }
            }else{
                callRejectOrderAPI()
            }
        }
    }
}
