//
//  ReturnOrderVC.swift
//  globalfoods
//
//  Created by Apple on 07/01/21.
//

import UIKit

class ReturnOrderCell: UITableViewCell {
    
    @IBOutlet weak var lblReturnNo:UILabel!
    @IBOutlet weak var btnDownload:UIButton!
    @IBOutlet weak var btnApprove:UIButton!
    @IBOutlet weak var btnReject:UIButton!
    @IBOutlet weak var lblDescription:UILabel!
    @IBOutlet weak var lblTotalPrice:UILabel!
    
    override class func awakeFromNib() {
        
    }
}

class ReturnOrderVC: UIViewController {

    @IBOutlet weak var tblReturnList:UITableView!
    
    var customHeaderView:HeaderView?
    var arrReturnOrders = [ReturnOrderModel]()
    var selectedRow = 0
    var strStatus = ""
    var refreshControl = UIRefreshControl()
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        callGetReturnOrderAPI()
        refreshControl.addTarget(self, action: #selector(callGetReturnOrderAPI), for: .valueChanged)
        self.tblReturnList.addSubview(refreshControl)
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->ReturnOrderVC?{
        return UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "ReturnOrderVC") as? ReturnOrderVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Return Order", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func btnDownloadAction(_ sender:UIButton){
        if sender.isSelected{
            if let VC = WebViewController.instance(){
                let fileName = "ReturnOrder_\(arrReturnOrders[sender.tag].id ?? 0).pdf"
                let documentsUrl =  FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
                let destinationFileUrl = documentsUrl.appendingPathComponent("\(fileName)")
                VC.strURL = destinationFileUrl.absoluteString
                self.navigationController?.pushViewController(VC, animated: true)
            }
        }else{
            downloadPDFFile(id: arrReturnOrders[sender.tag].id ?? 0, path: arrReturnOrders[sender.tag].path ?? "")
        }
    }
    
    @IBAction func btnApproveAction(_ sender:UIButton){
        selectedRow = sender.tag
        strStatus = "approve"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to approve this return order?"
        settings.alertTitle = "Approve Return Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    @IBAction func btnRejectAction(_ sender:UIButton){
        selectedRow = sender.tag
        strStatus = "reject"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to reject this return order?"
        settings.alertTitle = "Reject Return Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    func checkPDFExist(id:Int)->Bool{
        let fileName = "ReturnOrder_\(id).pdf"
        let documentsUrl =  FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
        let destinationFileUrl = documentsUrl.appendingPathComponent("\(fileName)")
        //print(destinationFileUrl)
        return FileManager.default.fileExists(atPath: destinationFileUrl.path)
    }
    
    func downloadPDFFile(id:Int,path:String){
        let fileName = "ReturnOrder_\(id).pdf"
        let documentsUrl =  FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
        let destinationFileUrl = documentsUrl.appendingPathComponent("\(fileName)")
        
        if let fileURL = URL(string: path){
            AppCommonMethods.startProgressBar()
            DispatchQueue.global().async {
                do{
                    let data = try Data.init(contentsOf: fileURL)
                    try data.write(to: destinationFileUrl)
                    DispatchQueue.main.async {
                        AppCommonMethods.stopProgressBar()
                        if let VC = WebViewController.instance(){
                            VC.strURL = destinationFileUrl.absoluteString
                            self.navigationController?.pushViewController(VC, animated: true)
                        }
                    }
                }catch (let err) {
                    DispatchQueue.main.async {
                        AppCommonMethods.stopProgressBar()
                    }
                    print("error: \(err)")
                }
            }
        }
    }
    
    func callRejectReturnAPI(){
        //http://global.smartmps.co.uk/rms/service/service_employee.php?key=1226&s=110&return_id=&return_type=approve&store_id=1&item=[{%22qty%22:%222%22,%22price%22:%2212%22,%22return_item_id%22:12}]
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "110"
        param["return_type"] = strStatus
        param["store_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["return_id"] = "\(arrReturnOrders[selectedRow].id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Return Status Data:",response!)
                    }
                    aMessage = response!.getStringValue("ack_msg") ?? ""
                    if response!.getIntValue("ack") ?? 0 == 1{
                        var setting = BottomAlertSettings()
                        setting.strMessage = aMessage
                        AppValidation.shared.showBottomAlertView(settings: setting)
                        self.callGetReturnOrderAPI()
                        return
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
    
    @objc func callGetReturnOrderAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "51"
        
        if AppConstants.userData != nil{
            param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
            param["cbid"] = "\(AppConstants.selectedBranch?.id ?? 0)"
            param["eid"] = "0"
        }
        
        AppCommonMethods.startProgressBar(view: self.view)
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrReturnOrders.removeAll()
                self.refreshControl.endRefreshing()
                AppCommonMethods.stopProgressBar(view: self.view)
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Return Order Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([ReturnOrderModel].self, from: data){
                                self.arrReturnOrders = aData
                                self.tblReturnList.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblReturnList.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension ReturnOrderVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrReturnOrders.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "ReturnOrderCell", for: indexPath) as! ReturnOrderCell
        let model = arrReturnOrders[indexPath.row]
        cell.lblReturnNo.text = "Return No: #\(model.returnNo ?? "")"
        cell.lblTotalPrice.text = ""
        if let data = AppConstants.userData{
            if data.showPrice ?? 0 == 1{
                cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign,model.grandTotal ?? 0)
            }
        }else{
            cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign,model.grandTotal ?? 0)
        }
        cell.btnDownload.tag = indexPath.row
        if checkPDFExist(id: model.id ?? 0){
            cell.btnDownload.isSelected = true
        }else{
            cell.btnDownload.isSelected = false
        }
        cell.btnApprove.isHidden = true
        cell.btnReject.isHidden = true
        if AppConstants.employeeData?.rights?.returnOrder?.update ?? 0 == 1 && model.status ?? 0 == 0{
            cell.btnApprove.isHidden = false
            cell.btnReject.isHidden = false
            cell.btnApprove.tag = indexPath.row
            cell.btnReject.tag = indexPath.row
        }
        if let date = model.date?.getDateFromString(format: "dd-MM-yyyy"){
            let strDesc = "Created on: \(date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy"))\nStatus: \(model.statusSlug ?? "")"
            let attrDesc = NSMutableAttributedString.init(string: strDesc, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
            attrDesc.addFontAttribute(title: "Created on", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Status", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            cell.lblDescription.attributedText = attrDesc
        }
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let VC = ReturnOrderDetailVC.instance(){
            VC.returnOrderDetail = arrReturnOrders[indexPath.row]
            VC.delegate = self
            self.navigationController?.pushViewController(VC, animated: true)
        }
    }
}

extension ReturnOrderVC:alertClickedDelegate{
    func alertButtonClicked(index: Int,value:String) {
        if index == 1{
            if strStatus == "approve"{
                if let VC = ApproveReturnVC.instance(order: arrReturnOrders[selectedRow]){
                    if let topVC = VC.childViewController as? ApproveReturnVC{
                        topVC.delegate = self
                    }
                    self.present(VC, animated: true, completion: nil)
                }
            }else{
                callRejectReturnAPI()
            }
        }
    }
}

extension ReturnOrderVC:ReturnStatusDelegate{
    func returnApproved() {
        callGetReturnOrderAPI()
    }
}
