//
//  SideMenuVC.swift
//  globalfoods
//
//  Created by Apple on 28/12/20.
//

import UIKit
import SideMenuSwift

struct SideMenuCellType {
    static let kDashboard = 0
    static let kCreateOrder = 1
    static let kOrderHistory = 2
    static let kInvoice = 3
    static let kCreateNote = 4
    static let kDispatches = 5
    static let kRepoVisits = 6
    static let kAccount = 7
    static let kSupport = 8
    static let kProfile = 9
    static let kTermCondition = 10
    static let kLogout = 11
    static let kNotification = 12
    static let kReturnOrder = 13
    static let kEmployeeDashboard = 14
    static let kDeliveries = 15
    static let kPayment = 16
    static let kChecklist = 17
    static let kCashDrawer = 18
    static let kCreateSalesOrder = 19
    static let kSalesReport = 20
    static let kSalesOrderHistory = 21
    static let kLayAway = 22
}

class SideMenuCellData{
    
    var title = ""
    var imgName = ""
    var type = SideMenuCellType.kCreateOrder
    
    init(_ title:String,_ imgName:String,_ type:Int) {
        self.title = title
        self.imgName = imgName
        self.type = type
    }
}

class SideMenuViewCell: UITableViewCell {
    
    @IBOutlet weak var lblTitle:UILabel!
    @IBOutlet weak var imgView:UIImageViewX!
    
    override class func awakeFromNib() {
        
    }
}

class SideMenuVC: UIViewController {

    @IBOutlet weak var lblBranchName:UILabel!
    @IBOutlet weak var lblVersionInfo:UILabel!
    @IBOutlet weak var lblClosingBalance:UILabel!
    @IBOutlet weak var lblClosingBalanceTitle:UILabel!
    @IBOutlet weak var btnClose:UIButton!
    @IBOutlet weak var tblSideMenu:UITableView!
    @IBOutlet weak var constBtnChangeWidth:NSLayoutConstraint!
    
    var arrSideMenu:[SideMenuCellData] = [.init("Dashboard", "home", SideMenuCellType.kDashboard),.init("Create Order", "createorder", SideMenuCellType.kCreateOrder),.init("Order History", "orderhistory", SideMenuCellType.kOrderHistory),.init("Return Order", "return_order", SideMenuCellType.kReturnOrder),.init("Invoice", "invoice", SideMenuCellType.kInvoice),.init("Credit Note", "createorder", SideMenuCellType.kCreateNote),.init("Dispatches", "dispath", SideMenuCellType.kDispatches),.init("Repo Visits", "repovisit", SideMenuCellType.kRepoVisits)]
    
    var arrSetings:[SideMenuCellData] = [.init("Account", "account", SideMenuCellType.kAccount),.init("Profile", "user", SideMenuCellType.kProfile),.init("Support", "support", SideMenuCellType.kSupport),.init("Notifications", "notificationios", SideMenuCellType.kNotification),.init("Terms & Condition", "termsandcondition", SideMenuCellType.kTermCondition),.init("Logout", "logout", SideMenuCellType.kLogout)]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
    }
    
    override func viewWillAppear(_ animated: Bool) {
        lblVersionInfo.text = "Version \(Bundle.main.infoDictionary?["CFBundleShortVersionString"] as? String ?? "1.0")"
        if AppConstants.userData != nil{
            lblBranchName.setLabelAttributedText(title: "Branch", value: AppConstants.selectedBranch?.name ?? "", fontSize: AppConstants.DeviceType.IS_IPAD ? 19 : 17)
            if AppConstants.selectedBranch?.closingBalance ?? 0 < 0{
                self.lblClosingBalance.text = String.init(format: "-%@%.2f", AppConstants.currencySign,-(AppConstants.selectedBranch?.closingBalance ?? 0))
            }else{
                self.lblClosingBalance.text = String.init(format: "%@%.2f", AppConstants.currencySign,AppConstants.selectedBranch?.closingBalance ?? 0)
            }
        }else if AppConstants.employeeData != nil{
            constBtnChangeWidth.constant = 0
            btnClose.isHidden = true
            lblBranchName.setLabelAttributedText(title: "Name", value: AppConstants.employeeData?.name ?? "", fontSize: AppConstants.DeviceType.IS_IPAD ? 19 : 17)
            if AppConstants.employeeData?.closingBalance ?? 0 < 0{
                self.lblClosingBalance.text = String.init(format: "-%@%.2f", AppConstants.currencySign,-(AppConstants.employeeData?.closingBalance ?? 0))
            }else{
                self.lblClosingBalance.text = String.init(format: "%@%.2f", AppConstants.currencySign,AppConstants.employeeData?.closingBalance ?? 0)
            }
            arrSideMenu = [.init("Dashboard", "home", SideMenuCellType.kEmployeeDashboard)]
            arrSetings = [.init("Support", "support", SideMenuCellType.kSupport),.init("Notifications", "notificationios", SideMenuCellType.kNotification),.init("Terms & Condition", "termsandcondition", SideMenuCellType.kTermCondition),.init("Logout", "logout", SideMenuCellType.kLogout)]
            if AppConstants.employeeData?.rights?.order?.insert ?? 0 == 1{
                arrSideMenu.append(.init("Create Order", "createorder", SideMenuCellType.kCreateOrder))
            }
            if AppConstants.employeeData?.rights?.salesOrder?.insert ?? 0 == 1{
                arrSideMenu.append(.init("Create Sales Order", "createorder", SideMenuCellType.kCreateSalesOrder))
            }
            if AppConstants.employeeData?.rights?.order?.view ?? 0 == 1{
                arrSideMenu.append(.init("Order History", "orderhistory", SideMenuCellType.kOrderHistory))
            }
            if AppConstants.employeeData?.rights?.salesOrder?.view ?? 0 == 1{
                arrSideMenu.append(.init("Sales Order History", "orderhistory", SideMenuCellType.kSalesOrderHistory))
            }
            if AppConstants.employeeData?.rights?.salesOrder?.insert ?? 0 == 1{
                arrSideMenu.append(.init("Order Layaway", "lawaway", SideMenuCellType.kLayAway))
            }
            if AppConstants.employeeData?.rights?.returnOrder?.view ?? 0 == 1{
                arrSideMenu.append(.init("Return Order", "return_order", SideMenuCellType.kReturnOrder))
            }
            if AppConstants.employeeData?.rights?.invoice?.view ?? 0 == 1{
                arrSideMenu.append(.init("Invoice", "invoice", SideMenuCellType.kInvoice))
            }
            if AppConstants.employeeData?.rights?.creditNote?.view ?? 0 == 1{
                arrSideMenu.append(.init("Credit Note", "createorder", SideMenuCellType.kCreateNote))
            }
            if AppConstants.employeeData?.rights?.payment?.view ?? 0 == 1{
                arrSideMenu.append(.init("Payment", "pay", SideMenuCellType.kPayment))
            }
            if AppConstants.employeeData?.rights?.dispatch?.view ?? 0 == 1{
                arrSideMenu.append(.init("Dispatches", "dispath", SideMenuCellType.kDispatches))
            }
            if AppConstants.employeeData?.rights?.delivery?.view ?? 0 == 1{
                arrSideMenu.append(.init("Deliveries", "dispath", SideMenuCellType.kDeliveries))
            }
            if AppConstants.employeeData?.rights?.visit?.view ?? 0 == 1{
                arrSideMenu.append(.init("Repo Visits", "repovisit", SideMenuCellType.kRepoVisits))
            }
            if AppConstants.employeeData?.rights?.picklist?.view ?? 0 == 1{
                arrSideMenu.append(.init("Checklist", "checklist", SideMenuCellType.kChecklist))
            }
            if let drawer = AppConstants.employeeData?.rights?.cashDrawer{
                if drawer.view ?? 0 == 1{
                    arrSideMenu.append(.init("Cash Drawer", "ic_cash", SideMenuCellType.kCashDrawer))
                }
            }else{
                arrSideMenu.append(.init("Cash Drawer", "ic_cash", SideMenuCellType.kCashDrawer))
            }
            if AppConstants.employeeData?.rights?.account?.view ?? 0 == 1{
                arrSetings.insert(.init("Account", "account", SideMenuCellType.kAccount), at: 0)
            }
            if AppConstants.employeeData?.rights?.salesOrder?.view ?? 0 == 1{
                arrSideMenu.append(.init("Sales Report", "createorder", SideMenuCellType.kSalesReport))
            }
        }else{
            constBtnChangeWidth.constant = 0
            btnClose.isHidden = true
            lblClosingBalanceTitle.text = ""
            lblBranchName.setLabelAttributedText(title: "Branch", value: AppConstants.subCustomerData?.branchName ?? "", fontSize: AppConstants.DeviceType.IS_IPAD ? 19 : 17)
            arrSetings = [.init("Terms & Condition", "termsandcondition", SideMenuCellType.kTermCondition),.init("Logout", "logout", SideMenuCellType.kLogout)]
            arrSideMenu = [.init("Dashboard", "home", SideMenuCellType.kDashboard),.init("Create Order", "home", SideMenuCellType.kCreateOrder),.init("Order History", "orderhistory", SideMenuCellType.kOrderHistory)]
            lblClosingBalance.text = ""
        }
        tblSideMenu.reloadData()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->SideMenuVC?{
        return UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "SideMenuVC") as? SideMenuVC
    }
    
    @IBAction func btnChangeBranch(_ sender:UIButton){
        if let VC = SelectBranchVC.instance(){
            VC.isBackEnabled = true
            self.sideMenuController?.hideMenu()
            if self.sideMenuController!.contentViewController is UINavigationController{
                (self.sideMenuController!.contentViewController as! UINavigationController).pushViewController(VC, animated: true)
            }else{
                self.sideMenuController!.contentViewController.navigationController!.pushViewController(VC, animated: true)
            }
        }
    }
}

extension SideMenuVC:UITableViewDataSource,UITableViewDelegate{
    func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        if section == 1{
            let cell = tableView.dequeueReusableCell(withIdentifier: "SideMenuHeaderCell") as! SideMenuViewCell
            return cell
        }
        return nil
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if section == 1{
            return 35
        }
        return 0
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section == 1{
            return arrSetings.count
        }
        return arrSideMenu.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "SideMenuViewCell", for: indexPath) as! SideMenuViewCell
        let model = (indexPath.section == 0) ? arrSideMenu[indexPath.row] : arrSetings[indexPath.row]
        cell.lblTitle.text = model.title
        cell.imgView.image = UIImage.init(named: model.imgName)
        cell.imgView.imageColor = AppConstants.Colors.kAppMainThemeColor
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let model = (indexPath.section == 0) ? arrSideMenu[indexPath.row] : arrSetings[indexPath.row]
        let type = model.type
        switch model.type {
        case SideMenuCellType.kLogout:
            sideMenuController?.hideMenu(animated: true, completion: { (status) in
                var settings = AlertSettings()
                settings.alertTitle = "Logout!!"
                settings.alertMsg = "Are you sure you want to logout?"
                settings.confirmButtonFirstColor = AppConstants.Colors.kAppLightRedColor
                settings.confirmButtonSecondColor = AppConstants.Colors.kAppRedColor
                settings.confirmButtonTitle = "Logout"
                settings.cancelButtonFirstColor = AppConstants.Colors.kAppMainThemeColor
                settings.cancelButtonSecondColor = AppConstants.Colors.kAppDarkThemeColor
                if let VC = CustomeAlertVC.showPopup(parentVC: self.sideMenuController!.contentViewController, settings: settings){
                    VC.delegate = self
                }
            })
        case SideMenuCellType.kCashDrawer:
            PrintManager.shared.connectToPrinterWithPrint(model: AppConstants.employeeData!)
        case SideMenuCellType.kDashboard:
            if let VC = DashboardVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kAccount:
            if let VC = AccountVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kSupport:
            if let VC = SupportVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kChecklist:
            if let VC = ChecklistVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kRepoVisits:
            if let VC = RepoVisitVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kDispatches:
            if let VC = DispatchesVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kPayment:
            if let VC = PaymentListVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kDeliveries:
            if let VC = DashboardVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kCreateNote:
            if let VC = CreditNoteVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kInvoice:
            if let VC = InvoiceVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kSalesOrderHistory:
            if let VC = OrderHistoryVC.instance(){
                VC.isSalesOrder = true
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kOrderHistory:
            if let VC = OrderHistoryVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kSalesReport:
            if let VC = SalesReportVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kCreateSalesOrder:
            if let VC = CreateOrderVC.instance(){
                VC.isSalesOrder = true
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kCreateOrder:
            if let VC = CreateOrderVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kEmployeeDashboard:
            if let VC = EmployeeDashboardVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kLayAway:
            if let VC = OrderLayawayVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kReturnOrder:
            if let VC = ReturnOrderVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kTermCondition:
            if let VC = TermsConditionVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kProfile:
            if let VC = ProfileVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        case SideMenuCellType.kNotification:
            if let VC = NotificationsVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                sideMenuController?.setContentViewController(to: navc)
            }
        default:
            break
        }
    }
}

extension SideMenuVC:alertClickedDelegate{
    func alertButtonClicked(index: Int,value:String) {
        if index == 1{
            AppCommonMethods.removefromUserDefault(withKey: AppConstants.userDefaultKey.kPrinterIPAddress)
            AppCommonMethods.removefromUserDefault(withKey: AppConstants.userDefaultKey.kUserData)
            AppCommonMethods.removefromUserDefault(withKey: AppConstants.userDefaultKey.kSelectedBranch)
            AppCommonMethods.removefromUserDefault(withKey: AppConstants.userDefaultKey.kEmployeeData)
            AppCommonMethods.removefromUserDefault(withKey: AppConstants.userDefaultKey.kSubCustomerData)
            if let VC = LoginVC.instance(){
                let navc = UINavigationController.init(rootViewController: VC)
                navc.isNavigationBarHidden = true
                AppConstants.appDelegate.window?.rootViewController = navc
            }
        }
    }
}
