//
//  SupportVC.swift
//  globalfoods
//
//  Created by Apple on 29/12/20.
//

import UIKit
import ActionSheetPicker_3_0

class SupportViewCell: UITableViewCell {
    
    @IBOutlet weak var lblDescription:UILabel!
    @IBOutlet weak var lblTicket:UILabel!
    @IBOutlet weak var lblMsgCount:UILabel!
    @IBOutlet weak var constMsgWidth:NSLayoutConstraint!
    
    override class func awakeFromNib() {
        
    }
}

class SupportVC: UIViewController {

    @IBOutlet weak var lblFromDate:UILabel!
    @IBOutlet weak var lblToDate:UILabel!
    @IBOutlet weak var btnAddTicket:UIButton!
    @IBOutlet weak var tblSupport:UITableView!
    
    var customHeaderView:HeaderView?
    var strToDate = ""
    var strFromDate = ""
    var arrSupport = [supportModel]()
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        self.strFromDate = Date().addingTimeInterval(-7*24*60*60).getDateInString(format: "dd-MM-yyyy")
        self.strToDate = Date().getDateInString(format: "dd-MM-yyyy")
        self.lblFromDate.text = "From - \(Date().addingTimeInterval(-7*24*60*60).getDateInString(format: "dd/MM/yyyy"))"
        self.lblToDate.text = "To - \(Date().getDateInString(format: "dd/MM/yyyy"))"
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        if AppConstants.userData == nil{
            btnAddTicket.superview?.isHidden = true
        }else{
            btnAddTicket.superview?.isHidden = false
        }
        setHeaderView()
        callGetSupportsAPI()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->SupportVC?{
        return UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "SupportVC") as? SupportVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Support", isBackEnbaled: isBackEnabled)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    //MARK: Button Actions
    @IBAction func btnFromDateAction(_ sender:UIButton){
        let date = strFromDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        let datePicker = ActionSheetDatePicker(title: "Select from date", datePickerMode: UIDatePicker.Mode.date, selectedDate: date, doneBlock: {
            picker, value, index in
            self.strFromDate = (value as! Date).getDateInString(format: "dd-MM-yyyy")
            self.lblFromDate.text = "From - \((value as! Date).getDateInString(format: "dd/MM/yyyy"))"
            self.callGetSupportsAPI()
        }, cancel: { ActionStringCancelBlock in return }, origin: sender)
        datePicker?.maximumDate = strToDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        datePicker?.show()
    }
    
    @IBAction func btnToDateAction(_ sender:UIButton){
        let date = strToDate.getDateFromString(format: "dd-MM-yyyy", locale: nil) ?? Date()
        let datePicker = ActionSheetDatePicker(title: "Select to date", datePickerMode: UIDatePicker.Mode.date, selectedDate: date, doneBlock: {
            picker, value, index in
            self.strToDate = (value as! Date).getDateInString(format: "dd-MM-yyyy")
            self.lblToDate.text = "To - \((value as! Date).getDateInString(format: "dd/MM/yyyy"))"
            self.callGetSupportsAPI()
        }, cancel: { ActionStringCancelBlock in return }, origin: sender)
        if let fromDate = strFromDate.getDateFromString(format: "dd-MM-yyyy", locale: nil){
            datePicker?.minimumDate = fromDate
        }
        datePicker?.show()
    }
    
    @IBAction func btnAddSupportAction(_ sender:UIButton){
        if let VC = UserTextViewVC.showPopup(parentVC: self, string: "", viewType: .kSupportSubject){
            VC.delegate = self
        }
    }
    
    func callGetSupportsAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "62"
        if AppConstants.userData != nil{
            param["cid"] = "\(AppConstants.userData?.id ?? 0)"
        }
        
        param["from_date"] = strFromDate
        param["to_date"] = strToDate
        
        if AppConstants.isInDevlopmentMode{
            print("Support Param:",param)
        }
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceChatURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                self.arrSupport.removeAll()
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Support Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode([supportModel].self, from: data){
                                self.arrSupport = aData
                                self.tblSupport.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblSupport.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
    
    func callCreateSupportsAPI(subject:String){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "61"
        param["eid"] = "\(AppConstants.userData?.id ?? 0)"
        param["subject"] = subject
        
        if AppConstants.isInDevlopmentMode{
            print("Add Support Param:",param)
        }
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceChatURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Add Support Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        var setting = BottomAlertSettings()
                        setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                        AppValidation.shared.showBottomAlertView(settings: setting)
                        if let result = response!["result"] as? NSDictionary, let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode(supportModel.self, from: data){
                                self.arrSupport.insert(aData, at: 0)
                                self.tblSupport.reloadData()
                                return
                            }
                        }
                        self.callGetSupportsAPI()
                        return
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblSupport.reloadData()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
}

extension SupportVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrSupport.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "SupportViewCell", for: indexPath) as! SupportViewCell
        let model = arrSupport[indexPath.row]
        cell.lblTicket.text = "TICKET #\(model.id ?? 0)"
        if model.newMsgCount ?? 0 == 0{
            cell.constMsgWidth.constant = 0
        }else{
            cell.constMsgWidth.constant = 26
            cell.lblMsgCount.text = "\(model.newMsgCount ?? 0)"
        }
        if let date = model.date?.getDateFromString(format: "yyyy-MM-dd HH:mm:ss"){
            let strDesc = "Subject: \(model.subject ?? "")\nLast message: \(model.lastMsg ?? "")\nDate: \(date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy 'at' hh:mm a"))"
            let attrDesc = NSMutableAttributedString.init(string: strDesc, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
            attrDesc.addFontAttribute(title: "Subject", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Last message", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            attrDesc.addFontAttribute(title: "Date", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
            cell.lblDescription.attributedText = attrDesc
        }
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let VC = SupportChatVC.instance(){
            VC.selectedSupport = arrSupport[indexPath.row]
            self.navigationController?.pushViewController(VC, animated: true)
        }
    }
}

extension SupportVC:UserTextViewDelegate{
    func userAddedText(string:String) {
        callCreateSupportsAPI(subject:string)
    }
}
