//
//  TermsConditionVC.swift
//  globalfoods
//
//  Created by Apple on 05/01/21.
//

import UIKit
import WebKit

class TermsConditionVC: UIViewController {

    @IBOutlet weak var webView: WKWebView?
    var customHeaderView:HeaderView?
    var isBackEnabled = false
    var popRecognizer: InteractivePopRecognizer?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        
        if webView != nil{
            webView!.navigationDelegate = self
        }
        if isBackEnabled{
            setInteractiveRecognizer()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        setHeaderView()
        callGetTermsConditionAPI()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->TermsConditionVC?{
        return UIStoryboard.init(name: "SideMenu", bundle: nil).instantiateViewController(withIdentifier: "TermsConditionVC") as? TermsConditionVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Terms & Condition", isBackEnbaled: isBackEnabled)
    }

    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    func callGetTermsConditionAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "58"
        
        AppCommonMethods.startProgressBar(isInteractive: true)
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCustomerURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Terms Condition Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if self.webView != nil{
                            self.webView!.loadHTMLString(result.getStringValue("customer_term_condition") ?? "", baseURL: nil)
                            return
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppCommonMethods.stopProgressBar()
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
}

//MARK: WebView Methods
extension TermsConditionVC:WKNavigationDelegate{
    func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        AppCommonMethods.stopProgressBar()
    }
    
    func webView(_ webView: WKWebView, didFail navigation: WKNavigation!, withError error: Error) {
        print(error.localizedDescription)
        AppCommonMethods.stopProgressBar()
    }
}
