//
//  OrderDetailVC.swift
//  globalfoods
//
//  Created by Apple on 01/01/21.
//

import UIKit

class OrderDetailViewCell: UITableViewCell {
    
    @IBOutlet weak var lblName:UILabel!
    @IBOutlet weak var lblQty:UILabel!
    @IBOutlet weak var lblStockQty:UILabel!
    @IBOutlet weak var txtQty:UITextField!
    @IBOutlet weak var txtPrice:UITextField!
    @IBOutlet weak var lblPrice:UILabel!
    @IBOutlet weak var btnSelect:UIButton!
    @IBOutlet weak var imgSelect:UIImageViewX!
    
    override class func awakeFromNib() {
        
    }
}

class OrderPriceDetailCell: UITableViewCell {
    
    @IBOutlet weak var lblTotalPrice:UILabel!
    @IBOutlet weak var lblVATPrice:UILabel!
    @IBOutlet weak var lblDiscount:UILabel!
    @IBOutlet weak var lblGrandTotal:UILabel!
    @IBOutlet weak var lblRemark:UILabel!
    @IBOutlet weak var lblCashPayment:UILabel!
    @IBOutlet weak var lblCardPayment:UILabel!
    @IBOutlet weak var lblChequePayment:UILabel!
    @IBOutlet weak var lblRefundedAmount:UILabel!
    @IBOutlet weak var viewForRemark:UIView!
    @IBOutlet weak var btnDelete:UIButton!
    @IBOutlet weak var btnDispatch:UIButton!
    @IBOutlet weak var btnReprint:UIButton!
    @IBOutlet weak var btnReturn:UIButton!
    @IBOutlet weak var btnApprove:UIButton!
    @IBOutlet weak var btnPay:UIButton!
    @IBOutlet weak var btnRefund:UIButton!
    @IBOutlet weak var btnReject:UIButton!
    @IBOutlet weak var btnCreateDispatch:UIButton!
    
    override class func awakeFromNib() {
        
    }
}

class OrderDetailVC: UIViewController {
    
    @IBOutlet weak var lblOrderNo:UILabel!
    @IBOutlet weak var lblOrderDate:UILabel!
    @IBOutlet weak var lblOrderStatus:UILabel!
    @IBOutlet weak var tblOrder:UITableView!
    @IBOutlet weak var viewForOrderInfo:UIView!
    @IBOutlet weak var lblCustomerName:UILabel!
    @IBOutlet weak var constOrderNoBottomSpace:NSLayoutConstraint!
    
    var customHeaderView:HeaderView?
    var popRecognizer: InteractivePopRecognizer?
    var orderDetail:OrderHistoryModel!
    var arrItems = [CartItemsModel]()
    var strStatus = ""
    var isSalesOrder = false
    var isChangeMessage = false
    var vatAmount = 0.0
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        setHeaderView()
        setInteractiveRecognizer()
        setOrderData()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        callGetOrderDetailAPI()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->OrderDetailVC?{
        return UIStoryboard.init(name: "SubViews", bundle: nil).instantiateViewController(withIdentifier: "OrderDetailVC") as? OrderDetailVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Order Details", isBackEnbaled: true)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    func setOrderData(){
        self.lblOrderNo.text = "Order No: #\(orderDetail.orderNo ?? "")"
        if let date = orderDetail.date?.getDateFromString(format: "dd-MM-yyyy"){
            lblOrderDate.setLabelAttributedText(title: "Ordered on", value: date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy"), fontSize: AppConstants.DeviceType.IS_IPAD ? 18 : 16)
        }
        lblOrderStatus.setLabelAttributedText(title: "Order status", value: orderDetail.statusText ?? "", fontSize: AppConstants.DeviceType.IS_IPAD ? 18 : 16)
        if AppConstants.employeeData != nil{
            constOrderNoBottomSpace.constant = 40
            lblCustomerName.setLabelAttributedText(title: "Customer name", value: orderDetail.customerName ?? "", fontSize: AppConstants.DeviceType.IS_IPAD ? 18 : 16)
            viewForOrderInfo.frame = CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.size.width, height: 150)
        }
    }
    
    @IBAction func btnDispatchesAction(_ sender:UIButton){
        if let VC = DispatchesVC.instance(){
            VC.orderDetail = orderDetail
            self.navigationController?.pushViewController(VC, animated: true)
        }
    }
    
    @IBAction func btnDeleteAction(_ sender:UIButton){
        if let VC = UserTextViewVC.showPopup(parentVC: self, string: "", viewType: .kCancelReason){
            VC.delegate = self
        }
    }
    
    @IBAction func btnRejectAction(_ sender:UIButton){
        strStatus = "reject"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to reject this order?"
        settings.alertTitle = "Reject Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    @IBAction func btnApproveAction(_ sender:UIButton){
        strStatus = "approve"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to approve this order?"
        settings.alertTitle = "Approve Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    @IBAction func btnReturnOrderAction(_ sender:UIButton){
        if let VC = CreateReturnOrderVC.instance(order: orderDetail){
            self.present(VC, animated: true, completion: nil)
        }
    }
    
    @IBAction func btnRefundOrderAction(_ sender:UIButton){
        if let VC = RefundSalesOrderVC.instance(order: orderDetail){
            if let topVC = VC.childViewController as? RefundSalesOrderVC{
                topVC.delegate = self
            }
            self.present(VC, animated: true, completion: nil)
        }
    }
    
    @IBAction func btnReprintAction(_ sender:UIButton){
        PrintManager.shared.isSalesOrder = isSalesOrder
        PrintManager.shared.connectToPrinterWithPrint(model: orderDetail!)
    }
    
    @IBAction func btnPayAction(_ sender:UIButton){
        let paid = (orderDetail.cardAmount ?? 0) + (orderDetail.cashAmount ?? 0) + (orderDetail.chequeAmount ?? 0)
        var amount = (orderDetail.orderTotal ?? 0) - paid
        amount = Double(String.init(format: "%.2f", amount)) ?? 0
        if let _ = PaymentMethodVC.showPopup(parentVC: self, totalAmount: amount){
            
        }
    }
    
    @IBAction func btnCreateDispatchAction(_ sender:UIButton){
        if let VC = CreateDispatchesVC.instance(){
            VC.selectedOrder = orderDetail
            self.navigationController?.pushViewController(VC, animated: true)
        }
    }
    
    func callGetOrderDetailAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = isSalesOrder ? "131" : "7"
        param["order_id"] = "\(orderDetail.id ?? 0)"
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCartURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Order Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode(OrderHistoryModel.self, from: data){
                                self.orderDetail = aData
                                self.arrItems = aData.arrItems ?? [CartItemsModel]()
                                self.vatAmount = 0.0
                                self.arrItems.forEach({ model in
                                    self.vatAmount += (model.taxAmount ?? 0)
                                })
                                self.tblOrder.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
    func callUpdatePaymentAPI(paymentMode:String, amount:Double){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "132"
        param["order_id"] = "\(orderDetail.id ?? 0)"
        if paymentMode == "cod"{
            let totalCashAmount = (self.orderDetail.cashAmount ?? 0) + amount
            let total = totalCashAmount + (self.orderDetail.cardAmount ?? 0) + (self.orderDetail.chequeAmount ?? 0)
            if total > orderDetail.orderTotal ?? 0{
                param["cash_amount"] = String.init(format: "%.2f", (orderDetail.orderTotal ?? 0) - (orderDetail.cardAmount ?? 0) - (orderDetail.chequeAmount ?? 0))
            }else{
                param["cash_amount"] = String.init(format: "%.2f", totalCashAmount)
            }
            param["card_amount"] = String.init(format: "%.2f", orderDetail.cardAmount ?? 0)
            param["cheque_amount"] = String.init(format: "%.2f", orderDetail.chequeAmount ?? 0)
        }else if paymentMode == "card"{
            param["cash_amount"] = String.init(format: "%.2f", orderDetail.cashAmount ?? 0)
            let total = (orderDetail.cardAmount ?? 0) + amount
            param["card_amount"] = String.init(format: "%.2f", total)
            param["cheque_amount"] = String.init(format: "%.2f", orderDetail.chequeAmount ?? 0)
        }else{
            param["cash_amount"] = String.init(format: "%.2f", orderDetail.cashAmount ?? 0)
            let total = (orderDetail.cardAmount ?? 0) + amount
            param["cheque_amount"] = String.init(format: "%.2f", total)
            param["card_amount"] = String.init(format: "%.2f", orderDetail.cardAmount ?? 0)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCustomerURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Payment Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        if paymentMode == "cod"{
                            let totalCashAmount = (self.orderDetail.cashAmount ?? 0) + amount
                            let total = totalCashAmount + (self.orderDetail.cardAmount ?? 0) + (self.orderDetail.chequeAmount ?? 0)
                            if total > self.orderDetail.orderTotal ?? 0{
                                self.orderDetail.cashAmount = (self.orderDetail.orderTotal ?? 0) - (self.orderDetail.cardAmount ?? 0) - (self.orderDetail.chequeAmount ?? 0)
                            }else{
                                self.orderDetail.cashAmount = totalCashAmount
                            }
                            if total > self.orderDetail.orderTotal ?? 0{
                                self.strStatus = "change"
                                var settings = AlertSettings()
                                settings.alertTitle = String.init(format: "Change: %@%.2f", AppConstants.currencySign, total - (self.orderDetail.orderTotal ?? 0))
                                settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_OK
                                settings.cancelButtonTitle = ""
                                if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
                                    VC.delegate = self
                                }
                            }else{
                                self.strStatus = "cashdrawer"
                                var settings = AlertSettings()
                                settings.alertMsg = "Do you want to open cash drawer?"
                                settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
                                settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
                                if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
                                    VC.delegate = self
                                }
                            }
                        }else if paymentMode == "card"{
                            self.orderDetail.cardAmount = (self.orderDetail.cardAmount ?? 0) + amount
                        }else{
                            self.orderDetail.chequeAmount = (self.orderDetail.chequeAmount ?? 0) + amount
                        }
                        self.tblOrder.reloadData()
                        return
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
    
    func callRejectOrderAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "113"
        param["id"] = "\(orderDetail.id ?? 0)"
        param["order_type"] = strStatus
        param["customer_id"] = "\(orderDetail.cid ?? 0)"
        param["customer_branch_id"] = "\(orderDetail.cbid ?? 0)"
        param["sales_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["order_date"] = orderDetail.date ?? ""
        if AppConstants.isInDevlopmentMode{
            print("Reject Order Param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Reject Order Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        self.callGetOrderDetailAPI()
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
    
    func callDeleteOrderAPI(strReason:String){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "31"
        param["order_id"] = "\(orderDetail.id ?? 0)"
        param["reason"] = strReason
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceOrderURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Delete Order Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        self.callGetOrderDetailAPI()
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
}

extension OrderDetailVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrItems.count+1
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if indexPath.row == arrItems.count{
            let cell = tableView.dequeueReusableCell(withIdentifier: "OrderPriceDetailCell", for: indexPath) as! OrderPriceDetailCell
            cell.lblDiscount.superview?.isHidden = true
            cell.lblDiscount.superview?.isHidden = true
            cell.lblGrandTotal.superview?.isHidden = true
            cell.lblCashPayment.superview?.isHidden = true
            cell.lblCardPayment.superview?.isHidden = true
            cell.lblChequePayment.superview?.isHidden = true
            cell.lblRefundedAmount.superview?.isHidden = true
            cell.btnReject.isHidden = true
            cell.btnApprove.isHidden = true
            cell.btnDelete.isHidden = true
            cell.btnCreateDispatch.isHidden = true
            cell.btnReturn.isHidden = true
            cell.btnDispatch.isHidden = true
            cell.btnReprint.isHidden = true
            cell.btnPay.isHidden = true
            cell.viewForRemark.isHidden = true
            cell.btnRefund.isHidden = true

            if let data = AppConstants.userData{
                if data.showPrice ?? 0 == 1{
                    cell.lblDiscount.superview?.isHidden = false
                    cell.lblGrandTotal.superview?.isHidden = false
                    cell.lblVATPrice.superview?.isHidden = false
                    cell.lblTotalPrice.superview?.isHidden = false
                    cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,orderDetail.subTotal ?? 0)
                    cell.lblVATPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,vatAmount)
                    cell.lblGrandTotal.text = String.init(format: "%@%.2f", AppConstants.currencySign ,orderDetail.orderTotal ?? 0)
                    if orderDetail.discount ?? 0 != 0{
                        cell.lblDiscount.superview?.isHidden = false
                        cell.lblDiscount.text = String.init(format: "%@%.2f", AppConstants.currencySign ,orderDetail.discount ?? 0)
                    }
                }
            }else{
                cell.lblDiscount.superview?.isHidden = false
                cell.lblGrandTotal.superview?.isHidden = false
                cell.lblVATPrice.superview?.isHidden = false
                cell.lblTotalPrice.superview?.isHidden = false
                cell.lblVATPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,vatAmount)
                cell.lblTotalPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,orderDetail.subTotal ?? 0)
                cell.lblGrandTotal.text = String.init(format: "%@%.2f", AppConstants.currencySign ,orderDetail.orderTotal ?? 0)
                if orderDetail.discount ?? 0 != 0{
                    cell.lblDiscount.superview?.isHidden = false
                    cell.lblDiscount.text = String.init(format: "%@%.2f", AppConstants.currencySign ,orderDetail.discount ?? 0)
                }
            }
            
            if orderDetail.remarks ?? "" != ""{
                cell.viewForRemark.isHidden = false
                cell.lblRemark.text = orderDetail.remarks ?? ""
            }
            if AppConstants.subCustomerData != nil{
                
            }else if isSalesOrder{
                cell.btnReprint.isHidden = false
                var total = (orderDetail.cardAmount ?? 0) + (orderDetail.cashAmount ?? 0) + (orderDetail.chequeAmount ?? 0)
                total = Double(String.init(format: "%.2f", total)) ?? 0
                if (orderDetail.orderTotal ?? 0) > total{
                    cell.btnPay.isHidden = false
                }else{
                    if orderDetail.orderTotal ?? 0 != 0{
                        cell.btnRefund.isHidden = false
                    }
                }
                if orderDetail.refundAmount ?? 0 != 0{
                    cell.lblRefundedAmount.superview?.isHidden = false
                    cell.lblRefundedAmount.text = String.init(format: "%@ %.2f", AppConstants.currencySign, orderDetail.refundAmount ?? 0)
                }
                cell.lblCashPayment.text = String.init(format: "%@ %.2f", AppConstants.currencySign, orderDetail.cashAmount ?? 0)
                cell.lblCardPayment.text = String.init(format: "%@ %.2f", AppConstants.currencySign, orderDetail.cardAmount ?? 0)
                cell.lblChequePayment.text = String.init(format: "%@ %.2f", AppConstants.currencySign, orderDetail.chequeAmount ?? 0)
                cell.lblCashPayment.superview?.isHidden = false
                cell.lblCardPayment.superview?.isHidden = false
                cell.lblChequePayment.superview?.isHidden = false
            }else{
                if orderDetail.status ?? 0 == 1{
                    if AppConstants.userData == nil{
                        if AppConstants.employeeData?.rights?.order?.update ?? 0 == 1{
                            cell.btnReject.isHidden = false
                            cell.btnApprove.isHidden = false
                        }
                    }else{
                        if AppConstants.userData?.allowCancel ?? 0 == 1{
                            cell.btnDelete.isHidden = false
                        }
                    }
                }
                if orderDetail.status ?? 0 == 2 && AppConstants.employeeData != nil{
                    if AppConstants.employeeData?.rights?.dispatch?.insert ?? 0 == 1{
                        cell.btnCreateDispatch.isHidden = false
                    }
                }
                if AppConstants.employeeData != nil{
                    if AppConstants.employeeData?.rights?.dispatch?.view ?? 0 == 1 && orderDetail.status ?? 0 >= 3{
                        cell.btnDispatch.isHidden = false
                    }
                    if orderDetail.status ?? 0 != -1 && orderDetail.status ?? 0 >= 2 && AppConstants.printerIP ?? "" != ""{
                        cell.btnReprint.isHidden = false
                    }
                }else{
                    if orderDetail.status ?? 0 >= 3{
                        cell.btnDispatch.isHidden = false
                    }
                }
                if orderDetail.status ?? 0 == 5{
                    if let emp = AppConstants.employeeData{
                        if emp.rights?.returnOrder?.insert ?? 0 == 1{
                            cell.btnReturn.isHidden = false
                        }
                    }else{
                        cell.btnReturn.isHidden = false
                    }
                }
            }
            
            return cell
        }
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderDetailViewCell", for: indexPath) as! OrderDetailViewCell
        let model = arrItems[indexPath.row]
        cell.lblName.text = model.itemName ?? ""
        cell.lblQty.text = String.init(format: "%.1f", model.itemQty ?? 0)
        if isSalesOrder{
            cell.lblPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign, model.itemTotalPrice ?? 0)
        }else{
            cell.lblPrice.text = ""
        }
        
        return cell
    }
}

extension OrderDetailVC:UserTextViewDelegate{
    func userAddedText(string:String) {
        callDeleteOrderAPI(strReason: string)
    }
}

extension OrderDetailVC:RefundSalesOrderDelegate{
    func refundSubmitted() {
        self.callGetOrderDetailAPI()
    }
}

extension OrderDetailVC:alertClickedDelegate{
    func alertButtonClicked(index: Int,value:String) {
        if index == 1{
            if strStatus == "approve"{
                if let VC = ApproveOrderVC.instance(){
                    VC.orderDetail = orderDetail
                    self.navigationController?.pushViewController(VC, animated: true)
                }
            }else if strStatus == "reject"{
                callRejectOrderAPI()
            }else if strStatus == "change"{
                strStatus = "cashdrawer"
                var settings = AlertSettings()
                settings.alertMsg = "Do you want to open cash drawer?"
                settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
                settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
                if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
                    VC.delegate = self
                }
            }else{
                PrintManager.shared.connectToPrinterWithPrint(model: AppConstants.employeeData!)
            }
        }
    }
}
