//
//  ReturnOrderDetailVC.swift
//  globalfoods
//
//  Created by Apple on 08/01/21.
//

import UIKit

class ReturnOrderDetailVC: UIViewController {

    @IBOutlet weak var lblReturnNo:UILabel!
    @IBOutlet weak var lblReturnDate:UILabel!
    @IBOutlet weak var lblReturnStatus:UILabel!
    @IBOutlet weak var tblReturnOrder:UITableView!

    var customHeaderView:HeaderView?
    var popRecognizer: InteractivePopRecognizer?
    var returnOrderDetail:ReturnOrderModel!
    var arrItems = [ReturnItemModel]()
    var strStatus = ""
    var delegate:ReturnStatusDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        setHeaderView()
        setInteractiveRecognizer()
        callGetReturnOrderDetailAPI()
        setReturnData()
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->ReturnOrderDetailVC?{
        return UIStoryboard.init(name: "SubViews", bundle: nil).instantiateViewController(withIdentifier: "ReturnOrderDetailVC") as? ReturnOrderDetailVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Return Details", isBackEnbaled: true)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func btnApproveAction(_ sender:UIButton){
        strStatus = "approve"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to approve this return order?"
        settings.alertTitle = "Approve Return Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    @IBAction func btnRejectAction(_ sender:UIButton){
        strStatus = "reject"
        var settings = AlertSettings()
        settings.alertMsg = "Are you sure you want to reject this return order?"
        settings.alertTitle = "Reject Return Order"
        settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
        settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
        if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
            VC.delegate = self
        }
    }
    
    func setReturnData(){
        if let date = returnOrderDetail.date?.getDateFromString(format: "dd-MM-yyyy"){
            lblReturnDate.setLabelAttributedText(title: "Created on", value: date.getDateInString(format: "dd'\(date.getDateSuffix())' MMM, yyyy"), fontSize: 16)
        }
        lblReturnNo.setLabelAttributedText(title: "Return No", value: "#\(returnOrderDetail.returnNo ?? "")", fontSize: 16)
        lblReturnStatus.setLabelAttributedText(title: "Status", value: returnOrderDetail.statusSlug ?? "", fontSize: 16)
        
    }
    
    func callRejectReturnAPI(){
        //http://global.smartmps.co.uk/rms/service/service_employee.php?key=1226&s=110&return_id=&return_type=approve&store_id=1&item=[{%22qty%22:%222%22,%22price%22:%2212%22,%22return_item_id%22:12}]
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "110"
        param["return_type"] = strStatus
        param["store_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        param["return_id"] = "\(returnOrderDetail.id ?? 0)"
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Return Status Data:",response!)
                    }
                    aMessage = response!.getStringValue("ack_msg") ?? ""
                    if response!.getIntValue("ack") ?? 0 == 1{
                        var setting = BottomAlertSettings()
                        setting.strMessage = aMessage
                        AppValidation.shared.showBottomAlertView(settings: setting)
                        if self.delegate != nil{
                            self.delegate?.returnApproved()
                        }
                        self.callGetReturnOrderDetailAPI()
                        return
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
        }
    }
    
    func callGetReturnOrderDetailAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "52"
        param["rtid"] = "\(returnOrderDetail.id ?? 0)"
        param["vehical_id"] = "\(returnOrderDetail.vehicleId ?? 0)"
        param["eid"] = "\(returnOrderDetail.eid ?? 0)"
        AppCommonMethods.startProgressBar()
        if AppConstants.isInDevlopmentMode{
            print("Return Order Param:",param)
        }
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Return Order Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode(ReturnOrderModel.self, from: data){
                                self.returnOrderDetail = aData
                                self.arrItems = aData.arrItems ?? [ReturnItemModel]()
                                self.tblReturnOrder.reloadData()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension ReturnOrderDetailVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrItems.count+1
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if indexPath.row == arrItems.count{
            let cell = tableView.dequeueReusableCell(withIdentifier: "OrderPriceDetailCell", for: indexPath) as! OrderPriceDetailCell
            
            cell.lblGrandTotal.superview?.isHidden = true
            if let data = AppConstants.userData{
                if data.showPrice ?? 0 == 1{
                    cell.lblGrandTotal.superview?.isHidden = false
                    cell.lblGrandTotal.text = String.init(format: "%@%.2f", AppConstants.currencySign ,returnOrderDetail.grandTotal ?? 0)
                }
            }else{
                cell.lblGrandTotal.superview?.isHidden = false
                cell.lblGrandTotal.text = String.init(format: "%@%.2f", AppConstants.currencySign ,returnOrderDetail.grandTotal ?? 0)
            }
            cell.btnApprove.isHidden = true
            cell.btnReject.isHidden = true
            if AppConstants.employeeData?.rights?.returnOrder?.update ?? 0 == 1 && returnOrderDetail.status ?? 0 == 0{
                cell.btnApprove.isHidden = false
                cell.btnReject.isHidden = false
            }
            return cell
        }
        let cell = tableView.dequeueReusableCell(withIdentifier: "OrderDetailViewCell", for: indexPath) as! OrderDetailViewCell
        let model = arrItems[indexPath.row]
        cell.lblName.text = model.itemName ?? ""
        var strQty = ""
        cell.lblPrice.text = ""
        
        if model.returnSubTotal == nil{
            strQty = "Selected quantity: \(String.init(format: "%.1f", model.qty ?? 0))"
        }else{
            strQty = "Selected quantity: \(String.init(format: "%.1f", model.qty ?? 0))\nApproved quantity: \(String.init(format: "%.1f", model.returnQty ?? 0))"
        }
        
        if let data = AppConstants.userData{
            if data.showPrice ?? 0 == 1{
                if model.returnSubTotal == nil{
                    strQty = "Selected quantity: \(String.init(format: "%.1f", model.qty ?? 0))\nPrice: \(String.init(format: "%@%.2f", AppConstants.currencySign, model.returnPrice ?? 0))"
                    cell.lblPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,model.subtotal ?? 0)
                }else{
                    strQty = "Selected quantity: \(String.init(format: "%.1f", model.qty ?? 0))\nApproved quantity: \(String.init(format: "%.1f", model.returnQty ?? 0))\nPrice: \(String.init(format: "%@%.2f", AppConstants.currencySign, model.returnPrice ?? 0))"
                    cell.lblPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,model.returnSubTotal ?? 0)
                }
            }
        }else{
            if model.returnSubTotal == nil{
                strQty = "Selected quantity: \(String.init(format: "%.1f", model.qty ?? 0))\nPrice: \(String.init(format: "%@%.2f", AppConstants.currencySign, model.returnPrice ?? 0))"
                cell.lblPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,model.subtotal ?? 0)
            }else{
                strQty = "Selected quantity: \(String.init(format: "%.1f", model.qty ?? 0))\nApproved quantity: \(String.init(format: "%.1f", model.returnQty ?? 0))\nPrice: \(String.init(format: "%@%.2f", AppConstants.currencySign, model.returnPrice ?? 0))"
                cell.lblPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign ,model.returnSubTotal ?? 0)
            }
        }
        let attrQty = NSMutableAttributedString.init(string: strQty, attributes: [NSAttributedString.Key.foregroundColor : AppConstants.Colors.kAppGrayThemeColor,NSAttributedString.Key.font:AppConstants.GlobalFontConstants.kRegularFont(size: AppConstants.DeviceType.IS_IPAD ? 16 : 14)])
        attrQty.addFontAttribute(title: "Selected quantity", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrQty.addFontAttribute(title: "Approved quantity", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        attrQty.addFontAttribute(title: "Price", fontSize: AppConstants.DeviceType.IS_IPAD ? 16 : 14)
        cell.lblQty.attributedText = attrQty
        return cell
    }
}

extension ReturnOrderDetailVC:alertClickedDelegate{
    func alertButtonClicked(index: Int,value:String) {
        if index == 1{
            if strStatus == "approve"{
                if let VC = ApproveReturnVC.instance(order: returnOrderDetail){
                    if let topVC = VC.childViewController as? ApproveReturnVC{
                        topVC.delegate = self
                    }
                    self.present(VC, animated: true, completion: nil)
                }
            }else{
                callRejectReturnAPI()
            }
        }
    }
}

extension ReturnOrderDetailVC:ReturnStatusDelegate{
    func returnApproved() {
        callGetReturnOrderDetailAPI()
        if self.delegate != nil{
            self.delegate?.returnApproved()
        }
    }
}
