//
//  SupportChatVC.swift
//  globalfoods
//
//  Created by Apple on 06/01/21.
//

import UIKit
import RSKGrowingTextView

//MARK: TableView Cell
class ChatHeaderViewCell: UITableViewCell {
    @IBOutlet weak var lblDate:UILabel!
    
    override class func awakeFromNib() {
        
    }
}

class ChatViewCell: UITableViewCell {

    @IBOutlet weak var lblMessage:UILabel!
    @IBOutlet weak var lblDate:UILabel!
    @IBOutlet weak var lblSenderName:UILabel!
    @IBOutlet weak var btnFailed:UIButton!
    @IBOutlet weak var constFailedWidth:NSLayoutConstraint!
    @IBOutlet weak var viewForBG:UIViewX!
    
    override func awakeFromNib() {
        super.awakeFromNib()
        // Initialization code
    }
    
    override func setSelected(_ selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)

        // Configure the view for the selected state
    }
}

class MessagesCellData{
    
    var strMessage = ""
    var sender = ""
    var date = ""
    var isOwner = 1
    var isFailed = false
    var id = 0
    
}

class SupportChatVC: UIViewController {

    @IBOutlet weak var txtMessage:RSKGrowingTextView!
    @IBOutlet weak var viewForNewCount:UIView!
    @IBOutlet weak var lblNewMessageCount:UILabel!
    @IBOutlet weak var tblChat:UITableView!
    @IBOutlet weak var btnSend:UIButton!
    
    var customHeaderView:HeaderView?
    var popRecognizer: InteractivePopRecognizer?
    var selectedSupport:supportModel!
    
    var arrMessages = [MessagesCellData]()
    var arrNewMessages = [MessagesCellData]()
    var chatTimer:Timer?
    var isSendApiRunning = false
    var lastMessageId:Int?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        setHeaderView()
        setInteractiveRecognizer()
        callGetSupportChatAPI(isFromLoad: true)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        AppEnterinForeGround()
        NotificationCenter.default.addObserver(self, selector: #selector(AppEnterinForeGround), name: UIApplication.willEnterForegroundNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(AppEnterinBackGround), name: UIApplication.didEnterBackgroundNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        AppEnterinBackGround()
        NotificationCenter.default.removeObserver(self)
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->SupportChatVC?{
        return UIStoryboard.init(name: "SubViews", bundle: nil).instantiateViewController(withIdentifier: "SupportChatVC") as? SupportChatVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "TICKET #\(selectedSupport.id ?? 0)", isBackEnbaled: true)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }
    
    @IBAction func AppEnterinForeGround(){
        if chatTimer == nil{
            self.chatTimer = Timer.scheduledTimer(timeInterval: 10, target: self, selector: #selector(CallAPIAfterInterval), userInfo: nil, repeats: true)
        }
    }
    
    @IBAction func AppEnterinBackGround(){
        if chatTimer != nil{
            self.chatTimer!.invalidate()
            self.chatTimer = nil
        }
    }
    
    @IBAction func CallAPIAfterInterval(){
        if !isSendApiRunning{
            callGetSupportChatAPI(isFromLoad: false)
        }
    }
    
    //MARK: Button Action
    @IBAction func btnFailedAction(_ sender:UIButton){
        AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.FailedChat, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_NO,AppConstants.GlobalAlert.ALERT_BTN_YES], withActionsType: [.default,.default]) { (index) in
            if index == 1{
                self.arrMessages[sender.tag].isFailed = false
                self.callSendMesaageAPI(message: self.arrMessages[sender.tag].strMessage,date: self.arrMessages[sender.tag].date, index: sender.tag)
            }
            
        }
    }
    
    @IBAction func btnSendMessageAction(_ sender:UIButton){
        let message = MessagesCellData()
        message.strMessage = txtMessage.text ?? ""
        message.date = Date().getDateInString(format: "dd-MM-yy hh:mm a")
        arrMessages.append(message)
        callSendMesaageAPI(message: txtMessage.text ?? "", date: Date().getDateInString(format: "yyyy-MM-yyyy+HH:mm:ss"), index: arrMessages.count-1)
        txtMessage.text = ""
        btnSend.isEnabled = false
        tblChat.reloadData()
        self.tblChat.scrollToRow(at: IndexPath.init(row: self.arrMessages.count-1, section: 0), at: .bottom, animated: false)
    }
    
    @IBAction func btnNewMessageAction(_ sender:UIButton){
        self.arrNewMessages.removeAll()
        viewForNewCount.isHidden = true
        self.tblChat.scrollToRow(at: IndexPath.init(row: self.arrMessages.count-1, section: 0), at: .bottom, animated: false)
    }
    
    func callSendMesaageAPI(message:String,date:String, index:Int){
        var dictParam = [String:String]()
        dictParam["key"] = AppConstants.commonConstant.kAPIKey
        dictParam["s"] = "63"
        if AppConstants.userData?.id ?? 0 != 0{
            dictParam["etype"] = "0"
            dictParam["eid"] = "\(AppConstants.userData?.id ?? 0)"
        }else{
            dictParam["etype"] = "\(AppConstants.employeeData?.adminType ?? 0)"
            dictParam["eid"] = "\(AppConstants.employeeData?.id ?? 0)"
        }
        dictParam["message_type"] = "0"
        dictParam["chat_room_id"] = "\(selectedSupport.id ?? 0)"
        dictParam["message"] = message
        dictParam["check_time"] = date
        
        if AppConstants.isInDevlopmentMode{
            print("Send Message Parameter: ",dictParam)
        }
        
        WebServiceManager.shared.requestAPIwithURlEncoding(params: dictParam, urlString: AppConstants.APIURLs.kServiceChatURL) { (message, response) in
            DispatchQueue.main.async {
                self.isSendApiRunning = false
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Send Message Response: ",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        self.lastMessageId = result.getIntValue("id") ?? 0
                        let message = MessagesCellData()
                        message.strMessage = result.getStringValue("message") ?? ""
                        message.isOwner = 1
                        message.date = result.getStringValue("datetime") ?? ""
                        message.sender = result.getStringValue("sender_name") ?? ""
                        self.arrMessages[index] = message
                    }else{
                        self.arrMessages[index].isFailed = true
                    }
                    self.tblChat.reloadData()
                }
            }
        }
    }
    
    func callGetSupportChatAPI(isFromLoad:Bool){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "64"
        if AppConstants.userData?.id ?? 0 != 0{
            param["eid"] = "\(AppConstants.userData?.id ?? 0)"
            param["etype"] = "0"
        }else{
            param["etype"] = "\(AppConstants.employeeData?.adminType ?? 0)"
            param["eid"] = "\(AppConstants.employeeData?.id ?? 0)"
        }
        param["chat_room_id"] = "\(selectedSupport.id ?? 0)"
        if lastMessageId != nil{
            param["last_message_id"] = "\(lastMessageId!)"
        }
        
        
        if AppConstants.isInDevlopmentMode{
            print("Chat Param:",param)
        }
        if isFromLoad{
            AppCommonMethods.startProgressBar()
        }
        
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceChatURL) { (message, response) in
            DispatchQueue.main.async {
                var aMessage = message
                if isFromLoad{
                    AppCommonMethods.stopProgressBar()
                }
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Chat Data:",response!)
                    }
                    if let result = response!["result"] as? [NSDictionary], response!.getIntValue("ack") ?? 0 == 1{
                        for i in 0..<result.count{
                            let chat = result[i]
                            let message = MessagesCellData()
                            message.strMessage = chat.getStringValue("message") ?? ""
                            message.isOwner = chat.getIntValue("isOwnMessage") ?? 0
                            message.date = chat.getStringValue("datetime") ?? ""
                            message.sender = chat.getStringValue("sender_name") ?? ""
                            self.arrNewMessages.append(message)
                            self.arrMessages.append(message)
                            if i == result.count-1{
                                self.lastMessageId = chat.getIntValue("id") ?? 0
                            }
                        }
                        self.tblChat.reloadData()
                        if isFromLoad{
                            self.tblChat.scrollToRow(at: IndexPath.init(row: self.arrMessages.count-1, section: 0), at: .bottom, animated: false)
                        }else{
                            if self.tblChat.contentOffset.y >= (self.tblChat.contentSize.height - self.tblChat.frame.size.height) {
                                self.tblChat.scrollToRow(at: IndexPath.init(row: self.arrMessages.count-1, section: 0), at: .bottom, animated: false)
                            }else{
                                if self.arrNewMessages.count != 0{
                                    self.lblNewMessageCount.text = "\(self.arrNewMessages.count)"
                                    self.viewForNewCount.isHidden = false
                                }
                            }
                        }
                        return
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                self.tblChat.reloadData()
                if isFromLoad{
                    AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    }
                }
            }
        }
    }
}

extension SupportChatVC:UITableViewDataSource,UITableViewDelegate{
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if self.tblChat.contentOffset.y >= (self.tblChat.contentSize.height - self.tblChat.frame.size.height-50) {
            self.arrNewMessages.removeAll()
            self.viewForNewCount.isHidden = true
        }
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrMessages.count
    }
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let message = arrMessages[indexPath.row]
        if message.isOwner == 1{
            let cell = tableView.dequeueReusableCell(withIdentifier: "SenderTextCell", for: indexPath) as! ChatViewCell
            cell.lblMessage.text = message.strMessage
            cell.lblDate.text = message.date
            if message.isFailed{
                cell.constFailedWidth.constant = 40
                cell.btnFailed.superview?.isHidden = false
                cell.btnFailed.tag = indexPath.row
            }else{
                cell.constFailedWidth.constant = 10
                cell.btnFailed.superview?.isHidden = true
            }
            return cell
        }else{
            let cell = tableView.dequeueReusableCell(withIdentifier: "ReceiverTextCell", for: indexPath) as! ChatViewCell
            cell.lblMessage.text = message.strMessage
            cell.lblDate.text = message.date
            return cell
        }
    }
}

extension SupportChatVC:UITextViewDelegate{
    func textViewDidChange(_ textView: UITextView) {
        if textView.text ?? "" == ""{
            btnSend.isEnabled = false
        }else{
            btnSend.isEnabled = true
        }
    }
}
