//
//  UserCartVC.swift
//  globalfoods
//
//  Created by Apple on 29/12/20.
//

import UIKit

protocol CheckoutDelegate: AnyObject {
    func userPlacedOrder()
    func userChangePrice(item:CartItemsModel)
}

class UserCartVC: UIViewController {

    @IBOutlet weak var tblCart:UITableView!
    @IBOutlet weak var lblGrandTotal:UILabel!
    @IBOutlet weak var btnAddRemark:UIButton!
    @IBOutlet weak var btnLayAway:UIButton!
    @IBOutlet weak var constTblBottomSpace:NSLayoutConstraint!
    
    var arrUserCart = [CartItemsModel]()
    var customHeaderView:HeaderView?
    var popRecognizer: InteractivePopRecognizer?
    var userCart:UserCartModel!
    var cuid:Int?
    var cbid:Int?
    weak var delegate:CheckoutDelegate?
    var customerName = ""
    var isSalesOrder = false
    var totalPrice = 0.0
    var isChangeMessage = false
    var isFromLayAway = false
    var orderDetail:OrderHistoryModel!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        setHeaderView()
        if AppConstants.employeeData == nil{
            callGetCartAPI()
        }else{
            self.arrUserCart = AppConstants.employeeCart?.cartitems ?? [CartItemsModel]()
            self.calculatePrice()
            if AppConstants.employeeCart?.remark ?? "" == ""{
                btnAddRemark.setTitle("ADD COMMENT", for: .normal)
            }else{
                btnAddRemark.setTitle("EDIT COMMENT", for: .normal)
            }
        }
        setInteractiveRecognizer()
        if isSalesOrder{
            btnLayAway.isHidden = false
            constTblBottomSpace.constant = AppConstants.DeviceType.IS_IPAD ? 130: 110
        }
    }
    
    //MARK: Create Viewcontrlloer instance
    static func instance()->UserCartVC?{
        return UIStoryboard.init(name: "SubViews", bundle: nil).instantiateViewController(withIdentifier: "UserCartVC") as? UserCartVC
    }
    
    //MARK: General methods
    func setHeaderView(){
        if customHeaderView != nil{
            customHeaderView!.removeFromSuperview()
            customHeaderView = nil
        }
        customHeaderView = HeaderView.init(controller: self, title: "Checkout", isBackEnbaled: true)
    }
    
    private func setInteractiveRecognizer() {
        guard let controller = navigationController else { return }
        popRecognizer = InteractivePopRecognizer(controller: controller)
        controller.interactivePopGestureRecognizer?.delegate = popRecognizer
    }

    func calculatePrice(){
        totalPrice = 0.0
        arrUserCart.forEach { model in
            totalPrice += ((model.itemPrice ?? 0) * (model.itemQty ?? 0))
        }
        AppConstants.employeeCart?.finalTotal = totalPrice
        self.totalPrice = Double(String.init(format: "%.2f", self.totalPrice)) ?? 0
        lblGrandTotal.text = String.init(format: "%@%.2f", AppConstants.currencySign,totalPrice)
    }
    
    @IBAction func btnIncrementAction(_ sender:UIButton){
        self.view.endEditing(true)
        let item = arrUserCart[sender.tag]
        var qty = item.itemQty ?? 0
        if qty == 0.5{
            qty = 1
        }else if qty < item.stockQty ?? 0{
            qty += 1
        }else{
            return
        }
        callAddCartAPI(row: sender.tag, quantity: qty)
        calculatePrice()
    }
    
    @IBAction func btnDecrementAction(_ sender:UIButton){
        self.view.endEditing(true)
        let item = arrUserCart[sender.tag]
        var qty = item.itemQty ?? 0
        if qty == 1.0{
            qty = 0.5
        }else{
            qty -= 1
        }
        if qty <= 0{
            callRemoveCartAPI(index: sender.tag)
        }else{
            callAddCartAPI(row: sender.tag, quantity: qty)
        }
        calculatePrice()
    }
    
    @IBAction func btnAddRemarkAction(_ sender:UIButton){
        self.view.endEditing(true)
        if let VC = UserTextViewVC.showPopup(parentVC: self, string: ((AppConstants.employeeData != nil) ? AppConstants.employeeCart : userCart)?.remark ?? ""){
            VC.delegate = self
        }
    }
    
    @IBAction func btnCheckOutAction(_ sender:UIButton){
        self.view.endEditing(true)
        if isSalesOrder{
            if let _ = PaymentMethodVC.showPopup(parentVC: self, totalAmount: totalPrice){
                
            }
        }else{
            callPlaceOrderAPI()
        }
    }
    
    @IBAction func btnLayWayAction(_ sender:UIButton){
        self.view.endEditing(true)
        callOrderLayawayAPI()
    }
    
    func callAddCartAPI(row:Int,quantity:Double){
        var param = [String:String]()
        let item = arrUserCart[row]
        if AppConstants.employeeData != nil{
            item.itemQty = quantity
            self.tblCart.reloadData()
            return
        }
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "35"
        if AppConstants.subCustomerData != nil{
            param["cuid"] = "\(AppConstants.subCustomerData?.cid ?? 0)"
            param["cbid"] = "\(AppConstants.subCustomerData?.cbid ?? 0)"
        }else{
            param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
            param["cbid"] = "\(AppConstants.selectedBranch?.id ?? 0)"
        }
        
        param["qty"] = "\(quantity)"
        param["pid"] = "\(item.itemId ?? 0)"
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCartURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Add cart Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        item.itemQty = quantity
                        self.tblCart.reloadData()
                    }
                }
            }
        }
    }
    
    func callRemoveCartAPI(index:Int){
        var param = [String:String]()
        let item = arrUserCart[index]
        if AppConstants.employeeData != nil{
            self.arrUserCart.remove(at: index)
            AppConstants.employeeCart?.cartitems?.remove(at: index)
            if  self.arrUserCart.count == 0{
                AppConstants.employeeCart = UserCartModel()
                self.navigationController?.popViewController(animated: true)
            }else{
                self.tblCart.reloadData()
            }
            return
        }
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "36"
        if AppConstants.subCustomerData != nil{
            param["cuid"] = "\(AppConstants.subCustomerData?.cid ?? 0)"
            param["cbid"] = "\(AppConstants.subCustomerData?.cbid ?? 0)"
        }else{
            param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
            param["cbid"] = "\(AppConstants.selectedBranch?.id ?? 0)"
        }
        
        param["ci_id"] = "\(item.id ?? 0)"
        param["cart_id"] = "\(item.cartId ?? 0)"
        if AppConstants.isInDevlopmentMode{
            print("Remove cart param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCartURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Remove cart Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        self.arrUserCart.remove(at: index)
                        if  self.arrUserCart.count == 0{
                            self.navigationController?.popViewController(animated: true)
                        }else{
                            self.tblCart.reloadData()
                        }
                    }else{
                        self.arrUserCart[index].itemQty = 0.5
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
    
    func callPlaceOrderAPI(paymentMode:String = "", amount:Double? = nil, chequeNo:String? = nil){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        if AppConstants.employeeData != nil{
            if isSalesOrder{
                param["s"] = "128"
                param["customer_name"] = customerName
                if cuid != nil{
                    param["cuid"] = "\(cuid ?? 0)"
                }
                if cbid != nil{
                    param["cbid"] = "\(cbid ?? 0)"
                }
                if isFromLayAway{
                    param["order_layaway_id"] = "\(AppConstants.employeeCart?.orderLayAwayId ?? 0)"
                }
                param["cash_amount"] = "0"
                param["cheque_amount"] = "0"
                param["card_amount"] = "0"
                if paymentMode == "card"{
                    param["card_amount"] = String.init(format: "%.2f", amount ?? 0)
                }else if paymentMode == "cod"{
                    if amount ?? 0 > totalPrice{
                        param["cash_amount"] = String.init(format: "%.2f", totalPrice)
                    }else{
                        param["cash_amount"] = String.init(format: "%.2f", amount ?? 0)
                    }
                }else{
                    param["cheque_number"] = chequeNo ?? ""
                    param["cheque_amount"] = String.init(format: "%.2f", amount ?? 0)
                }
                param["payment_mode"] = paymentMode
            }else{
                param["s"] = "60"
                param["cbid"] = "\(cbid ?? 0)"
                param["cuid"] = "\(cuid ?? 0)"
            }
            param["emp_id"] = "\(AppConstants.employeeData?.id ?? 0)"
            var arrItems = [NSMutableDictionary]()
            for model in arrUserCart{
                let dict = NSMutableDictionary()
                dict["qty"] = model.itemQty ?? 0
                dict["order_item_id"] = model.itemId ?? 0
                if isSalesOrder{
                    dict["price"] = model.itemPrice ?? 0
                }
                arrItems.append(dict)
            }
            param["item"] = AppCommonMethods.convertJsontoString(json: arrItems)
            param["remarks"] = AppConstants.employeeCart?.remark ?? ""
        }else{
            param["remarks"] = userCart.remark ?? ""
            param["s"] = "5"
            param["cart_id"] = "\(userCart.cartId ?? 0)"
        }
        if AppConstants.isInDevlopmentMode{
            print("Place Order Param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: (AppConstants.employeeData != nil) ? AppConstants.APIURLs.kServiceCustomerURL : AppConstants.APIURLs.kServiceCartURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Place Order Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        AppConstants.employeeCart = nil
                        if self.delegate != nil{
                            self.delegate!.userPlacedOrder()
                        }
                        let jsonDecoder = JSONDecoder()
                        if let result = response!["order_detail"] as? NSDictionary, let data = AppCommonMethods.convertToJson(object: result), let aData = try? jsonDecoder.decode(OrderHistoryModel.self, from: data), self.isSalesOrder{
                            self.orderDetail = aData
                            if paymentMode == "cod"{
                                PrintManager.shared.connectToPrinterWithPrint(model: AppConstants.employeeData!)
                                if amount ?? 0 > self.totalPrice{
                                    self.isChangeMessage = true
                                    var settings = AlertSettings()
                                    settings.alertTitle = String.init(format: "Change: %@%.2f", AppConstants.currencySign, (amount ?? 0) - self.totalPrice)
                                    settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_OK
                                    settings.cancelButtonTitle = ""
                                    if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
                                        VC.delegate = self
                                    }
                                    return
                                }
                            }
                            self.isChangeMessage = false
                            var settings = AlertSettings()
                            settings.alertMsg = "Do you want to print order receipt?"
                            settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
                            settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
                            if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
                                VC.delegate = self
                            }
                            return
                        }
                        if let VC = self.sideMenuController?.menuViewController as? SideMenuVC{
                            if AppConstants.employeeData != nil{
                                VC.sideMenuController?.setContentViewController(with: "\(SideMenuCellType.kEmployeeDashboard)", animated: true, completion: {
                                    if let contentVC = VC.sideMenuController?.contentViewController as? UINavigationController{
                                        contentVC.popToRootViewController(animated: true)
                                    }else{
                                        VC.sideMenuController?.contentViewController.navigationController?.popToRootViewController(animated: true)
                                    }
                                })
                            }else{
                                VC.sideMenuController?.setContentViewController(with: "\(SideMenuCellType.kDashboard)", animated: true, completion: {
                                    if let contentVC = VC.sideMenuController?.contentViewController as? UINavigationController{
                                        contentVC.popToRootViewController(animated: true)
                                    }else{
                                        VC.sideMenuController?.contentViewController.navigationController?.popToRootViewController(animated: true)
                                    }
                                })
                            }
                        }
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
    
    func callOrderLayawayAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "134"
        param["customer_name"] = customerName
        if cuid != nil{
            param["cuid"] = "\(cuid ?? 0)"
        }
        if cbid != nil{
            param["cbid"] = "\(cbid ?? 0)"
        }
        param["emp_id"] = "\(AppConstants.employeeData?.id ?? 0)"
        var arrItems = [NSMutableDictionary]()
        for model in arrUserCart{
            let dict = NSMutableDictionary()
            dict["qty"] = model.itemQty ?? 0
            dict["order_item_id"] = model.itemId ?? 0
            if isSalesOrder{
                dict["price"] = model.itemPrice ?? 0
            }
            arrItems.append(dict)
        }
        param["item"] = AppCommonMethods.convertJsontoString(json: arrItems)
        param["remarks"] = AppConstants.employeeCart?.remark ?? ""
        if AppConstants.isInDevlopmentMode{
            print("Order Layaway Param:",param)
        }
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceEmployeeURL) { (message, response) in
            DispatchQueue.main.async {
                AppCommonMethods.stopProgressBar()
                var setting = BottomAlertSettings()
                setting.strMessage = message
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Order Layaway Data:",response!)
                    }
                    if response!.getIntValue("ack") ?? 0 == 1{
                        AppConstants.employeeCart = nil
                        if let VC = self.sideMenuController?.menuViewController as? SideMenuVC{
                            if AppConstants.employeeData != nil{
                                VC.sideMenuController?.setContentViewController(with: "\(SideMenuCellType.kEmployeeDashboard)", animated: true, completion: {
                                    if let contentVC = VC.sideMenuController?.contentViewController as? UINavigationController{
                                        contentVC.popToRootViewController(animated: true)
                                    }else{
                                        VC.sideMenuController?.contentViewController.navigationController?.popToRootViewController(animated: true)
                                    }
                                })
                            }else{
                                VC.sideMenuController?.setContentViewController(with: "\(SideMenuCellType.kDashboard)", animated: true, completion: {
                                    if let contentVC = VC.sideMenuController?.contentViewController as? UINavigationController{
                                        contentVC.popToRootViewController(animated: true)
                                    }else{
                                        VC.sideMenuController?.contentViewController.navigationController?.popToRootViewController(animated: true)
                                    }
                                })
                            }
                        }
                    }
                    setting.strMessage = response!.getStringValue("ack_msg") ?? ""
                }
                AppValidation.shared.showBottomAlertView(settings: setting)
            }
        }
    }
    
    func callGetCartAPI(){
        var param = [String:String]()
        param["key"] = AppConstants.commonConstant.kAPIKey
        param["s"] = "39"
        if AppConstants.subCustomerData != nil{
            param["cuid"] = "\(AppConstants.subCustomerData?.cid ?? 0)"
            param["cbid"] = "\(AppConstants.subCustomerData?.cbid ?? 0)"
        }else{
            param["cuid"] = "\(AppConstants.userData?.id ?? 0)"
            param["cbid"] = "\(AppConstants.selectedBranch?.id ?? 0)"
        }
        
        AppCommonMethods.startProgressBar()
        WebServiceManager.shared.requestAPIwithURlEncoding(params: param, urlString: AppConstants.APIURLs.kServiceCartURL) { (message, response) in
            DispatchQueue.main.async {
                self.userCart = nil
                var aMessage = message
                AppCommonMethods.stopProgressBar()
                if response != nil{
                    if AppConstants.isInDevlopmentMode{
                        print("Cart Data:",response!)
                    }
                    if let result = response!["result"] as? NSDictionary, response!.getIntValue("ack") ?? 0 == 1{
                        if let data = AppCommonMethods.convertToJson(object: result){
                            let jsonDecoder = JSONDecoder()
                            if let aData = try? jsonDecoder.decode(UserCartModel.self, from: data){
                                self.userCart = aData
                                self.arrUserCart = self.userCart.cartitems ?? [CartItemsModel]()
                                self.tblCart.reloadData()
                                self.calculatePrice()
                                return
                            }
                        }
                    }else{
                        aMessage = response!.getStringValue("ack_msg") ?? ""
                    }
                }
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: aMessage, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
            }
            
        }
    }
}

extension UserCartVC:UITableViewDataSource,UITableViewDelegate{
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return arrUserCart.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "ProductViewCell", for: indexPath) as! ProductViewCell
        let model = arrUserCart[indexPath.row]
        cell.lblProductName.text = model.itemName ?? ""
        cell.txtProductQty.tag = indexPath.row
        cell.txtProductQty.text = String.init(format: "%.1f", model.itemQty ?? 0)
        cell.lblProductPrice.text = ""
        if self.isSalesOrder{
            cell.txtProductPrice.superview?.isHidden = false
            cell.txtProductPrice.tag = indexPath.row
            cell.txtProductPrice.accessibilityIdentifier = "101"
            cell.txtProductPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign,model.itemPrice ?? 0)
        }else{
            cell.txtProductPrice.superview?.isHidden = true
            if let data = AppConstants.userData{
                if data.showPrice ?? 0 == 1{
                    cell.lblProductPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign,model.itemPrice ?? 0)
                }
            }else{
                cell.lblProductPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign,model.itemPrice ?? 0)
            }
        }
        cell.btnIncrement.tag = indexPath.row
        cell.btnDecrement.tag = indexPath.row
        cell.imgProduct.sd_setImage(with: URL(string: model.image ?? ""), placeholderImage:UIImage(named: "placeholder"), options: .avoidDecodeImage)
        cell.lblCategory.text = model.categoryName ?? ""
        return cell
    }
}

extension UserCartVC:UITextFieldDelegate{
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        if textField.accessibilityIdentifier ?? "" == "101"{
            let allowedCharacters = CharacterSet(charactersIn:"0123456789")
            let characterSet = CharacterSet(charactersIn: string)
            if allowedCharacters.isSuperset(of: characterSet){
                var strText = (textField.text ?? "").replacingOccurrences(of: AppConstants.currencySign, with: "").replacingOccurrences(of: ".", with: "")
                if string == ""{
                    if strText != "000"{
                        if strText.count == 3{
                            strText.removeLast()
                            strText = "0\(strText)"
                        }else{
                            strText.removeLast()
                        }
                        strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                        textField.text = "\(AppConstants.currencySign)\(strText)"
                    }
                }else{
                    if strText.first! == "0"{
                        strText.removeFirst()
                        strText = "\(strText)\(string)"
                    }else{
                        strText = "\(strText)\(string)"
                    }
                    strText.insert(".", at: strText.index(strText.endIndex, offsetBy: -2))
                    textField.text = "\(AppConstants.currencySign)\(strText)"
                }
            }
            return false
        }else{
            let allowedCharacters = CharacterSet(charactersIn:"0123456789.")
            let characterSet = CharacterSet(charactersIn: string)
            return allowedCharacters.isSuperset(of: characterSet)
        }
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        let item = arrUserCart[textField.tag]
        if textField.accessibilityIdentifier ?? "" == "101"{
            let price = Double((textField.text ?? "").replacingOccurrences(of: AppConstants.currencySign, with: "")) ?? 0
            if price == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_AMOUNT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblCart.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? ProductViewCell{
                    cell.txtProductPrice.text = String.init(format: "%@%.2f", AppConstants.currencySign, item.itemPrice ?? 0)
                }
                return
            }
            item.itemPrice = price
            if self.delegate != nil{
                self.delegate?.userChangePrice(item: item)
            }
        }else{
            let qty = Double(textField.text ?? "") ?? 0
            if qty == 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: AppConstants.GlobalAlert.ALERT_ITEM_NOT_ZERO, withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblCart.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? ProductViewCell{
                    cell.txtProductQty.text = String.init(format: "%.1f", item.itemQty ?? 0)
                }
                return
            }else if qty > item.stockQty ?? 0{
                AppValidation.shared.showAlertView(parentVC: self, withAlertTile: AppConstants.GlobalAlert.ALERT_TITLE, withAlertMsg: String.init(format: "Selected item has %.1f quantity in stock, Please enter below stock quantity to proceed", item.stockQty ?? 0), withActionsTitle: [AppConstants.GlobalAlert.ALERT_BTN_OK], withActionsType: [.default]) { (index) in
                    
                }
                if let cell = tblCart.cellForRow(at: IndexPath.init(row: textField.tag, section: 0)) as? ProductViewCell{
                    cell.txtProductQty.text = String.init(format: "%.1f", item.itemQty ?? 0)
                }
                return
            }
            callAddCartAPI(row: textField.tag, quantity: qty)
        }
        calculatePrice()
    }
}

extension UserCartVC:UserTextViewDelegate{
    func userAddedText(string:String) {
        if AppConstants.employeeData != nil{
            AppConstants.employeeCart!.remark = string
        }else{
            userCart.remark = string
        }
        
        btnAddRemark.setTitle("EDIT COMMENT", for: .normal)
    }
}

extension UserCartVC:alertClickedDelegate{
    func alertButtonClicked(index: Int, value: String) {
        if isChangeMessage{
            self.isChangeMessage = false
            var settings = AlertSettings()
            settings.alertMsg = "Do you want to print order receipt?"
            settings.confirmButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_YES
            settings.cancelButtonTitle = AppConstants.GlobalAlert.ALERT_BTN_NO
            if let VC = CustomeAlertVC.showPopup(parentVC: self, settings: settings){
                VC.delegate = self
            }
        }else{
            if index == 1{
                PrintManager.shared.isSalesOrder = self.isSalesOrder
                PrintManager.shared.connectToPrinterWithPrint(model: self.orderDetail!)
            }
            if let VC = self.sideMenuController?.menuViewController as? SideMenuVC{
                if AppConstants.employeeData != nil{
                    VC.sideMenuController?.setContentViewController(with: "\(SideMenuCellType.kEmployeeDashboard)", animated: true, completion: {
                        if let contentVC = VC.sideMenuController?.contentViewController as? UINavigationController{
                            contentVC.popToRootViewController(animated: true)
                        }else{
                            VC.sideMenuController?.contentViewController.navigationController?.popToRootViewController(animated: true)
                        }
                    })
                }else{
                    VC.sideMenuController?.setContentViewController(with: "\(SideMenuCellType.kDashboard)", animated: true, completion: {
                        if let contentVC = VC.sideMenuController?.contentViewController as? UINavigationController{
                            contentVC.popToRootViewController(animated: true)
                        }else{
                            VC.sideMenuController?.contentViewController.navigationController?.popToRootViewController(animated: true)
                        }
                    })
                }
            }
        }
    }
}
